<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class HiCookieAdminForms
{
    public function __construct($module)
    {
        $this->module = $module;
        $this->name = $module->name;
        $this->context = Context::getContext();
    }

    public function l($string)
    {
        return $this->module->l($string);
    }

    public function renderDesignSettingsForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Design Settings'),
                    'icon' => 'icon-paint-brush',
                ],
                'input' => [
                    [
                        'type' => 'spacing',
                        'label' => $this->l('Padding around banner'),
                        'name' => 'padding',
                        'doc' => 'designPadding',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Banner width'),
                        'name' => 'popup_width',
                        'suffix' => 'px',
                        'doc' => 'bannerWidth',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Background color'),
                        'name' => 'bg_color',
                        'class' => 'hi-color-picker',
                        'doc' => 'backgroundColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Text color'),
                        'name' => 'text_color',
                        'class' => 'hi-color-picker',
                        'doc' => 'textColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Borders color'),
                        'name' => 'borders_color',
                        'class' => 'hi-color-picker',
                        'doc' => 'borderColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Banner border radius'),
                        'name' => 'border_radius',
                        'suffix' => 'px',
                        'doc' => 'borderRadius',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Accept button text color'),
                        'name' => 'accept_text_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 2 ? 'hm-hide' : ''),
                        'doc' => 'acceptTextColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Customize button text color'),
                        'name' => 'customize_text_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 2 ? 'hm-hide' : ''),
                        'doc' => 'customizeTextColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Only necessary cookies button background color'),
                        'name' => 'necessary_button_bg_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 1 ? 'hm-hide' : ''),
                        'doc' => 'onlyNecerssaryButtonBgColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Only necessary cookies button text color'),
                        'name' => 'necessary_button_text_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 1 ? 'hm-hide' : ''),
                        'doc' => 'onlyNecerssaryButtonTextColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Accept all cookies button background color'),
                        'name' => 'accept_all_button_bg_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 1 ? 'hm-hide' : ''),
                        'doc' => 'acceptAllButtonBgColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Accept all cookies button text color'),
                        'name' => 'accept_all_button_text_color',
                        'class' => 'hi-color-picker',
                        'form_group_class' => ($this->module->template == 1 ? 'hm-hide' : ''),
                        'doc' => 'acceptAllButtonTextColor',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('More info button text color'),
                        'name' => 'info_text_color',
                        'class' => 'hi-color-picker',
                        'doc' => 'moreInfoTextColor',
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->l('Custom CSS'),
                        'name' => 'custom_css',
                        'cols' => 100,
                        'rows' => 10,
                        'doc' => 'customCSS',
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'name' => 'submit_design_settings_form',
                    'class' => $this->module->psv >= 1.6 ? 'btn btn-default pull-right' : 'button',
                ],
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $languages = Language::getLanguages(false);
        foreach ($languages as $key => $language) {
            $languages[$key]['is_default'] = (int) ($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->submit_action = 'submitBlockSettings';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink(
            'AdminModules',
            false
        ) . '&configure=' . $this->name . '&tab_module=' . $this->module->tab . '&module_name=' . $this->name . '&' . $this->name . '=design_settings';
        $helper->module = $this->module;
        $helper->tpl_vars = [
            'fields_value' => [
                'bg_color' => $this->module->bg_color,
                'text_color' => $this->module->text_color,
                'borders_color' => $this->module->borders_color,
                'border_radius' => $this->module->border_radius,
                'accept_text_color' => $this->module->accept_text_color,
                'customize_text_color' => $this->module->customize_text_color,
                'info_text_color' => $this->module->info_text_color,
                'custom_css' => $this->module->custom_css,
                'padding' => $this->module->padding,
                'popup_width' => $this->module->popup_width,
                'necessary_button_bg_color' => $this->module->necessary_button_bg_color,
                'necessary_button_text_color' => $this->module->necessary_button_text_color,
                'accept_all_button_bg_color' => $this->module->accept_all_button_bg_color,
                'accept_all_button_text_color' => $this->module->accept_all_button_text_color,
            ],
        ];

        return $helper->generateForm([$fields_form]);
    }

    public function renderSettingsForm()
    {
        $countries = Country::getCountries($this->context->language->id, false, false, false);
        $countries_options = [];
        foreach ($countries as $country) {
            array_push($countries_options, [
                'id' => $country['id_country'],
                'name' => $country['name'],
            ]);
        }

        $cms_pages = CMS::listCms($this->context->language->id);
        $cms_pages_options = [];
        array_push($cms_pages_options, [
            'id' => 0,
            'name' => $this->l('Select CMS Page'),
        ]);
        foreach ($cms_pages as $cms) {
            array_push($cms_pages_options, [
                'id' => $cms['id_cms'],
                'name' => $cms['meta_title'],
            ]);
        }

        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('General Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'label' => $this->l('Template'),
                        'name' => 'template',
                        'doc' => 'template',
                        'options' => [
                            'query' => [
                                [
                                    'id' => '1',
                                    'name' => $this->l('Template 1'),
                                ],
                                [
                                    'id' => '2',
                                    'name' => $this->l('Template 2'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->l('Banner content'),
                        'name' => 'cookielaw_text',
                        'autoload_rte' => true,
                        'lang' => true,
                        'doc' => 'bannerContent',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Display shop logo on banner'),
                        'name' => 'displayShopLogo',
                        'is_bool' => true,
                        'form_group_class' => 'hi-cookie-display-shop-logo',
                        'doc' => 'shopLogo',
                        'values' => [
                            [
                                'id' => 'displayShopLogo_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'displayShopLogo_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'dropzone',
                        'label' => $this->l('Banner Logo'),
                        'name' => 'bannerLogo',
                        'maxFileSize' => Configuration::get('PS_LIMIT_UPLOAD_IMAGE_VALUE'), // MB
                        'maxFiles' => 1,
                        'sortable' => false,
                        'actionUrl' => $this->module->getAdminAjaxUrl(),
                        'uploadUrl' => $this->module->getAdminAjaxUrl(['action' => 'uploadBannerLogo', 'ajax' => 1]),
                        'filesLocation' => _MODULE_DIR_ . $this->name . '/views/img/upload/',
                        'actionForm' => false,
                        'idElement' => 0,
                        'acceptedFiles' => 'image/*',
                        'form_group_class' => 'hi-cookie-banner-logo' . ($this->module->displayShopLogo ? ' hm-hide' : ''),
                        'doc' => 'bannerLogo',
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Banner Position'),
                        'name' => 'position',
                        'doc' => 'bannerPosition',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'topleft',
                                    'name' => $this->l('Top Left'),
                                ],
                                [
                                    'id' => 'topright',
                                    'name' => $this->l('Top Right'),
                                ],
                                [
                                    'id' => 'bottomleft',
                                    'name' => $this->l('Bottom Left'),
                                ],
                                [
                                    'id' => 'bottomright',
                                    'name' => $this->l('Bottom Right'),
                                ],
                                [
                                    'id' => 'popup',
                                    'name' => $this->l('Popup'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('CMS page for more information'),
                        'multiple' => false,
                        'name' => 'info_page_id',
                        'doc' => 'cmsPage',
                        'options' => [
                            'query' => $cms_pages_options,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Disable Cookie banner for selected countries'),
                        'multiple' => true,
                        'name' => 'cookie_countries',
                        'id' => 'cookie_countries',
                        'doc' => 'disableCountries',
                        'options' => [
                            'query' => $countries_options,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Automatically consent on navigation'),
                        'name' => 'consent_on_navigation',
                        'is_bool' => true,
                        'desc' => $this->l('If enabled, cookies law will be consented when user navigates'),
                        'doc' => 'automaticConsent',
                        'values' => [
                            [
                                'id' => 'consent_on_navigation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'consent_on_navigation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Reload the page when cookie law accpeted'),
                        'name' => 'reload_page',
                        'is_bool' => true,
                        'doc' => 'reloadPage',
                        'values' => [
                            [
                                'id' => 'reload_page_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'reload_page_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Block navigation before cookie law accepted'),
                        'name' => 'block_navigation',
                        'is_bool' => true,
                        'doc' => 'blockNavigation',
                        'values' => [
                            [
                                'id' => 'block_navigation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'block_navigation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Display Cookie Settings Button'),
                        'name' => 'cookie_settings',
                        'form_group_class' => 'enable-cookie-settings',
                        'is_bool' => true,
                        'desc' => $this->l('This will add Cookie Settings button so customer can change settings after consent'),
                        'doc' => 'settingsButton',
                        'values' => [
                            [
                                'id' => 'cookie_settings_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'cookie_settings_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'label' => $this->l('Cookie Settings button position'),
                        'name' => 'cookie_settings_position',
                        'type' => 'checkbox',
                        'form_group_class' => 'cookie-settings-position' . (!$this->module->cookie_settings ? ' hm-hide' : ''),
                        'doc' => 'settingsButtonPosition',
                        'values' => [
                            'query' => [
                                [
                                    'id' => 'top',
                                    'name' => $this->l('Top'),
                                    'val' => 1,
                                ],
                                [
                                    'id' => 'custom',
                                    'name' => $this->l('Custom (Add {hook h="displayHiCookieSettings"} to your page tpl file where you want to display)'),
                                    'val' => 1,
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => ($this->module->psv >= 1.7 ? $this->l('You can transplant the settings button to any hook from Design->Positions tab.') : ''),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Disable Cookie Banner for Search Engines'),
                        'name' => 'seo_protection',
                        'is_bool' => true,
                        'doc' => 'disableBannerForSearchEngines',
                        'values' => [
                            [
                                'id' => 'seo_protection_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'seo_protection_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'whitelist_ip',
                        'label' => $this->l('Test IPs'),
                        'name' => 'whitelist_ips',
                        'desc' => $this->l('Cookie banner will always display for these IP addresses'),
                        'remote_ip_address' => Tools::getRemoteAddr(),
                        'doc' => 'testIPs',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Display "Contine Without Accpeting" button'),
                        'name' => 'continueWithoutAccepting',
                        'is_bool' => true,
                        'doc' => 'continueWithoutAccepting',
                        'values' => [
                            [
                                'id' => 'continueWithoutAccepting_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'continueWithoutAccepting_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Hook for banner'),
                        'name' => 'bannerHook',
                        'doc' => 'bannerHook',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'displayHeader',
                                    'name' => $this->l('displayHeader'),
                                ],
                                [
                                    'id' => 'displayFooter',
                                    'name' => $this->l('displayFooter'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Clean Database when module uninstalled'),
                        'name' => 'clean_db',
                        'is_bool' => true,
                        'desc' => $this->l('Not recommended, use this only when you’re not going to use the module'),
                        'doc' => 'cleanDb',
                        'values' => [
                            [
                                'id' => 'clean_db_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'clean_db_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'name' => 'submit_settings_form',
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $languages = Language::getLanguages(false);
        foreach ($languages as $key => $language) {
            $languages[$key]['is_default'] = (int) ($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->submit_action = 'submitBlockSettings';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink(
            'AdminModules',
            false
        ) . '&configure=' . $this->name . '&tab_module=' . $this->module->tab . '&module_name=' . $this->name . '&' . $this->name . '=generel_settings';
        $helper->module = $this->module;
        $helper->tpl_vars = [
            'fields_value' => [
                'template' => $this->module->template,
                'clean_db' => $this->module->clean_db,
                'cookielaw_text' => $this->module->cookielaw_text,
                'position' => $this->module->position,
                'info_page_id' => $this->module->info_page_id,
                'cookie_countries[]' => $this->module->cookie_countries,
                'whitelist_ips' => $this->module->whitelist_ips,
                'consent_on_navigation' => $this->module->consent_on_navigation,
                'reload_page' => $this->module->reload_page,
                'block_navigation' => $this->module->block_navigation,
                'cookie_settings' => $this->module->cookie_settings,
                'cookie_settings_position_top' => $this->module->cookie_settings_top,
                'cookie_settings_position_custom' => $this->module->cookie_settings_custom,
                'continueWithoutAccepting' => $this->module->continueWithoutAccepting,
                'bannerHook' => $this->module->bannerHook,
                'seo_protection' => $this->module->seo_protection,

                'displayShopLogo' => $this->module->displayShopLogo,
                'bannerLogo' => json_encode([
                    [
                        'fileName' => $this->module->bannerLogo,
                        'id' => $this->module->bannerLogo,
                    ],
                ]),
            ],
        ];

        return $helper->generateForm([$fields_form]);
    }

    public function renderBlockModulesForm($id_type)
    {
        $type = new HiCookieType($id_type);

        $modules = Module::getModulesOnDisk(true);

        foreach ($modules as $key => $module) {
            if (!$module->id || $module->name == 'hicookielaw') {
                unset($modules[$key]);
            }
        }

        $this->context->smarty->assign([
            'modules' => $modules,
            '_MODULE_DIR_' => _MODULE_DIR_,
            'request_uri' => $_SERVER['REQUEST_URI'],
            'blocked_modules' => array_map('current', $type->getBlockedModules()),
            'id_type' => (int) $id_type,
        ]);

        return $this->module->display($this->module->module_file, 'views/templates/admin/block-modules.tpl');
    }

    public function renderConsentsList()
    {
        $fields_list = [
            'ip_address' => [
                'title' => $this->l('IP Address'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'id_guest' => [
                'title' => $this->l('ID Guest'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'id_customer' => [
                'title' => $this->l('ID Customer'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'browser' => [
                'title' => $this->l('Browser'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'os' => [
                'title' => $this->l('OS'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'consents' => [
                'title' => $this->l('Consents'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'date_add' => [
                'title' => $this->l('Date Add'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'date_upd' => [
                'title' => $this->l('Date Update'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
        ];
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->no_link = true;
        $helper->identifier = 'id_consent';
        $helper->show_toolbar = false;
        $helper->title = $this->l('User Consents List');
        $helper->table = 'hicookieconsent';
        $helper->module = $this->module;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name . '&' . $this->name . '=consents';
        $helper->toolbar_btn['delete'] = [
            'href' => '#',
            'desc' => $this->l('Clear Logs'),
        ];
        $logs = HiCookieConsent::getConsentsForPreview();
        $helper->listTotal = count($logs);
        $page = ($page = Tools::getValue('submitFilter' . $helper->table)) ? $page : 1;
        $pagination = ($pagination = Tools::getValue($helper->table . '_pagination')) ? $pagination : 50;
        $logs = $this->module->hiPrestaClass->pagination($logs, $page, $pagination);

        return $helper->generateList($logs, $fields_list);
    }

    public function reanderTypeForm($id_type = null)
    {
        $type = new HiCookieType($id_type);

        $modules = Module::getModulesOnDisk(true);
        $modulesList = [];
        foreach ($modules as $key => $module) {
            if (!$module->id || $module->name == 'hicookielaw') {
                unset($modules[$key]);
                continue;
            }

            array_push($modulesList, [
                'id' => $module->name . '-' . $module->id,
                'name' => $module->displayName,
            ]);
        }

        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $id_type ? $this->l('Update Cookie Type') : $this->l('Add New Cookie Type'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'hidden',
                        'name' => 'id_type',
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Name'),
                        'name' => 'name',
                        'lang' => true,
                        'required' => true,
                        'placeholder' => $this->l('Cookie Type'),
                        'desc' => $this->l('Example: Functional cookies, Advertising Cookies, Analytics cookies'),
                        'doc' => 'cookieTypeName',
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->l('Description'),
                        'name' => 'description',
                        'autoload_rte' => true,
                        'lang' => true,
                        'cols' => 200,
                        'rows' => 40,
                        'doc' => 'cookieTypeDescription',
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Default Status'),
                        'name' => 'default_status',
                        'is_bool' => true,
                        'doc' => 'cookieTypeDefaultStatus',
                        'values' => [
                            [
                                'id' => 'default_status_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'default_status_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Required Cookie'),
                        'name' => 'disabled',
                        'is_bool' => true,
                        'doc' => 'cookieTypeRequired',
                        'values' => [
                            [
                                'id' => 'disabled_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'disabled_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                        'desc' => $this->l('If this is enabled, users will not be able to Enable / Disable this type'),
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Modules to disable'),
                        'multiple' => true,
                        'name' => 'cookieDisabledModules',
                        'id' => 'cookieDisabledModules',
                        'doc' => 'cookieTypeModules',
                        'options' => [
                            'query' => $modulesList,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'textarea',
                        'label' => $this->l('Block Custom Script'),
                        'name' => 'customScript',
                        'cols' => 100,
                        'rows' => 10,
                        'doc' => 'customScript',
                    ],
                    [
                        'type' => 'html',
                        'label' => $this->l('Block Script Manually'),
                        'name' => 'manualBlock',
                        'html_content' => $this->module->renderManualBlockInstructions(),
                        'doc' => 'manualBlock',
                    ],
                    [
                        'type' => 'shop',
                        'label' => $this->l('Assign the Cookie Type to these shops'),
                        'name' => 'checkBoxShopAsso',
                    ],
                ],
                'submit' => [
                    'title' => $id_type ? $this->l('Update') : $this->l('Add'),
                    'name' => 'submitSaveCookieType',
                ],
                'buttons' => [
                    [
                        'title' => $this->l('Cancel'),
                        'name' => 'closeCookieTypeForm',
                        'type' => 'button',
                        'icon' => 'process-icon-cancel',
                        'class' => 'btn btn-default pull-left',
                    ],
                ],
            ],
        ];

        $helper = new HelperForm();
        $languages = Language::getLanguages(false);
        foreach ($languages as $key => $language) {
            $languages[$key]['is_default'] = (int) ($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->show_toolbar = false;
        $this->fields_form = [];
        $helper->submit_action = 'submitCookieTypeSettings';
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->module->name . '&' . $this->module->name . '=types';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->module = $this->module;
        $helper->table = 'hicookietype';
        $helper->identifier = 'id_type';
        $helper->id = $id_type;
        $helper->tpl_vars = [
            'fields_value' => $this->getCookieTypeFieldsValues($id_type),
        ];

        return $helper->generateForm([$fields_form]);
    }

    public function getCookieTypeFieldsValues($id_type = null)
    {
        if ($id_type) {
            $type = new HiCookieType($id_type);
            $disabledModulesById = array_map('current', $type->getBlockedModules());
            $disabledModules = [];
            foreach ($disabledModulesById as $idModule) {
                array_push($disabledModules, $this->module->getModuleNameById($idModule) . '-' . $idModule);
            }

            return [
                'id_type' => $id_type,
                'disabled' => $type->disabled,
                'default_status' => $type->default_status,
                'name' => $type->name,
                'description' => $type->description,
                'cookieDisabledModules[]' => $disabledModules,
                'customScript' => $type->script,
            ];
        } else {
            $empty_array = [];
            foreach (Language::getLanguages(false) as $lang) {
                $empty_array[$lang['id_lang']] = '';
            }

            return [
                'id_type' => 0,
                'disabled' => false,
                'default_status' => true,
                'name' => $empty_array,
                'description' => $empty_array,
                'cookieDisabledModules[]' => [],
                'customScript' => '',
            ];
        }
    }

    public function renderCookieTypesList()
    {
        $fields_list = [
            'sort' => [
                'title' => $this->l('Sort'),
                'width' => 60,
                'type' => 'text',
                'search' => false,
            ],
            'id_type' => [
                'title' => $this->l('ID'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'name' => [
                'title' => $this->l('Name'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'disabled' => [
                'title' => $this->l('Required Cookie'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'default_status' => [
                'title' => $this->l('Default Status'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
            'blockedModules' => [
                'title' => $this->l('Blocked Modules'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
            ],
        ];
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->no_link = true;
        $helper->actions = ['edit', 'delete'];
        $helper->identifier = 'id_type';
        $helper->show_toolbar = false;
        $helper->title = $this->l('Cookie Types');
        $helper->table = 'hicookietype';
        $helper->module = $this->module;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name . '&' . $this->name . '=types';
        $helper->toolbar_btn['new'] = [
            'href' => '#',
            'desc' => $this->l('Add New Cookie Type'),
        ];
        $helper->toolbar_btn['help'] = [
            'href' => '#',
            'desc' => $this->l('Help'),
        ];
        $types = HiCookieType::getTypes();
        // add blocked modules
        if (is_array($types) && $types) {
            foreach ($types as $key => $type) {
                $typeObj = new HiCookieType($type['id_type']);
                $blockedModules = $typeObj->getBlockedModules();
                if (is_array($blockedModules) && $blockedModules) {
                    foreach ($blockedModules as $blockedModuleKey => $blockedModule) {
                        $module = Module::getInstanceById($blockedModule['id_module']);
                        if (Validate::isLoadedObject($module)) {
                            $blockedModules[$blockedModuleKey]['displayName'] = $module->displayName;
                        } else {
                            unset($blockedModules[$blockedModuleKey]);
                        }
                    }
                }

                if (!$blockedModules) {
                    $blockedModules = [];
                }

                $types[$key]['blockedModules'] = $blockedModules;
            }
        }
        $helper->listTotal = count($types);

        return $helper->generateList($types, $fields_list);
    }

    public function initForms()
    {
        $fields_form = [
            'form' => [
                'form' => [
                    'id_form' => 'naski',
                ],
                'legend' => [
                    'title' => $this->l('HTML Editor init'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'textarea',
                        'label' => $this->l('HTML Editor init'),
                        'name' => 'init_editor',
                        'autoload_rte' => true,
                    ],
                ],
            ],
        ];

        $helper = new HelperForm();
        $languages = Language::getLanguages(false);
        foreach ($languages as $key => $language) {
            $languages[$key]['is_default'] = (int) ($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->show_toolbar = false;
        $this->fields_form = [];
        $helper->submit_action = '';
        $helper->currentIndex = '';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->module = $this;
        $helper->tpl_vars = [
            'fields_value' => [
                'init_editor' => '',
            ],
        ];

        return $helper->generateForm([$fields_form]);
    }

    public function renderGoogleConsentsModeForm()
    {
        $cookieTypes = HiCookieType::getTypes();

        $cookieTypeOptions = [];
        array_push($cookieTypeOptions, [
            'id' => 0,
            'name' => $this->l('Select Cookie Type'),
        ]);
        foreach ($cookieTypes as $cookieType) {
            array_push($cookieTypeOptions, [
                'id' => $cookieType['id_type'],
                'name' => $cookieType['name'],
            ]);
        }

        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Google Consent Mode Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Google Consent Mode'),
                        'name' => 'consentMode',
                        'is_bool' => true,
                        'doc' => 'enableGoogleConsentMode',
                        'values' => [
                            [
                                'id' => 'consentMode_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'consentMode_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Select Cookie Type for Analytics Storage'),
                        'multiple' => false,
                        'name' => 'analyticsStorageCookieType',
                        'doc' => 'analyticsStorageCookieType',
                        'options' => [
                            'query' => $cookieTypeOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Default value for Analytics Storage'),
                        'name' => 'analyticsStorageDefaultValue',
                        'doc' => 'analyticsStorageDefaultValue',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'denied',
                                    'name' => $this->l('Denied'),
                                ],
                                [
                                    'id' => 'granted',
                                    'name' => $this->l('Granted'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => $this->l('If granted, Analytics Cookies will be installed before user consent'),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable URL Passthrough'),
                        'name' => 'urlPassthrough',
                        'is_bool' => true,
                        'doc' => 'urlPassthrough',
                        'values' => [
                            [
                                'id' => 'urlPassthrough_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'urlPassthrough_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                        'desc' => $this->l('If Analytics Storage is set to denied, URL passthrough can be used to send event and session-based analytics (including conversions) without cookies across pages'),
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Select Cookie Type for Ad Storage'),
                        'multiple' => false,
                        'name' => 'adStorageCookieType',
                        'doc' => 'adStorageCookieType',
                        'options' => [
                            'query' => $cookieTypeOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Default value for Ad Storage'),
                        'name' => 'adStorageDefaultValue',
                        'doc' => 'adStorageDefaultValue',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'denied',
                                    'name' => $this->l('Denied'),
                                ],
                                [
                                    'id' => 'granted',
                                    'name' => $this->l('Granted'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => $this->l('If granted, Ad Cookies will be installed before user consent'),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Ads Data Redaction'),
                        'name' => 'adsDataRedaction',
                        'is_bool' => true,
                        'doc' => 'adsDataRedaction',
                        'values' => [
                            [
                                'id' => 'adsDataRedaction_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ],
                            [
                                'id' => 'adsDataRedaction_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ],
                        ],
                        'desc' => $this->l('Enable Ads Data Redaction to further redact your ads data when Ad Storage is set to denied'),
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Select Cookie Type for Ad User Data'),
                        'multiple' => false,
                        'name' => 'adUserDataCookieType',
                        'doc' => 'adUserDataCookieType',
                        'options' => [
                            'query' => $cookieTypeOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Default value for Ad User Data'),
                        'name' => 'adUserDataDefaultValue',
                        'doc' => 'adUserDataDefaultValue',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'denied',
                                    'name' => $this->l('Denied'),
                                ],
                                [
                                    'id' => 'granted',
                                    'name' => $this->l('Granted'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => $this->l('If granted, user data related to advertising to Google will be sent before user consent.'),
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Select Cookie Type for Ad Personalization'),
                        'multiple' => false,
                        'name' => 'adPersonalizationCookieType',
                        'doc' => 'adPersonalizationCookieType',
                        'options' => [
                            'query' => $cookieTypeOptions,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Default value for Ad Personalization'),
                        'name' => 'adPersonalizationDefaultValue',
                        'doc' => 'adPersonalizationDefaultValue',
                        'options' => [
                            'query' => [
                                [
                                    'id' => 'denied',
                                    'name' => $this->l('Denied'),
                                ],
                                [
                                    'id' => 'granted',
                                    'name' => $this->l('Granted'),
                                ],
                            ],
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => $this->l('If granted, personalized advertising will be enabled before user consent.'),
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'name' => 'submitGoogleConsentModeForm',
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $languages = Language::getLanguages(false);
        foreach ($languages as $key => $language) {
            $languages[$key]['is_default'] = (int) ($language['id_lang'] == Configuration::get('PS_LANG_DEFAULT'));
        }
        $helper->languages = $languages;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->submit_action = 'submitBlockSettings';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = $this->context->link->getAdminLink(
            'AdminModules',
            false
        ) . '&configure=' . $this->name . '&tab_module=' . $this->module->tab . '&module_name=' . $this->name . '&' . $this->name . '=googleConsentMode';
        $helper->module = $this->module;
        $helper->tpl_vars = [
            'fields_value' => [
                'consentMode' => $this->module->consentMode,
                'adStorageDefaultValue' => $this->module->adStorageDefaultValue,
                'analyticsStorageDefaultValue' => $this->module->analyticsStorageDefaultValue,
                'adStorageCookieType' => $this->module->adStorageCookieType,
                'analyticsStorageCookieType' => $this->module->analyticsStorageCookieType,
                'adsDataRedaction' => $this->module->adsDataRedaction,
                'urlPassthrough' => $this->module->urlPassthrough,
                'adUserDataDefaultValue' => $this->module->adUserDataDefaultValue,
                'adUserDataCookieType' => $this->module->adUserDataCookieType,
                'adPersonalizationDefaultValue' => $this->module->adPersonalizationDefaultValue,
                'adPersonalizationCookieType' => $this->module->adPersonalizationCookieType,
            ],
        ];

        return $helper->generateForm([$fields_form]);
    }
}
