<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminHiCookieController extends ModuleAdminController
{
    public function __construct()
    {
        parent::__construct();

        $this->adminForms = $this->module->adminForms;
    }

    public function init()
    {
        parent::init();

        if (!$this->ajax) {
            Tools::redirectAdmin($this->module->hiPrestaClass->getModuleUrl());
        }
    }

    protected function ajaxRender($value = null, $controller = null, $method = null)
    {
        if (method_exists(get_parent_class($this), 'ajaxRender')) {
            return parent::ajaxRender($value, $controller, $method);
        }

        if ($controller === null) {
            $controller = get_class($this);
        }

        if ($method === null) {
            $bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
            $method = $bt[1]['function'];
        }

        /* @deprecated deprecated since 1.6.1.1 */
        Hook::exec('actionAjaxDieBefore', ['controller' => $controller, 'method' => $method, 'value' => $value]);

        /*
         * @deprecated deprecated since 1.6.1.1
         * use 'actionAjaxDie'.$controller.$method.'Before' instead
         */
        Hook::exec('actionBeforeAjaxDie' . $controller . $method, ['value' => $value]);
        Hook::exec('actionAjaxDie' . $controller . $method . 'Before', ['value' => $value]);
        header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');

        echo $value;
    }

    protected function ajaxDie($value = null, $controller = null, $method = null)
    {
        if (ob_get_contents()) {
            ob_end_clean();
        }
        header('Content-Type: application/json');

        $this->ajaxRender($value, $controller, $method);
        exit;
    }

    public function displayAjaxDeleteBannerLogo()
    {
        if (!$this->module->bannerLogo) {
            $this->ajaxDie(json_encode([
                'error' => $this->module->l('Banner logo has already been deleted.'),
            ]));
        }

        if (file_exists(_PS_MODULE_DIR_ . $this->module->name . '/views/img/upload/' . $this->module->bannerLogo)) {
            unlink(_PS_MODULE_DIR_ . $this->module->name . '/views/img/upload/' . $this->module->bannerLogo);
        }

        Configuration::updateValue('HI_COOKIE_BANNER_LOGO', false);

        exit(json_encode([
            'error' => false,
            'message' => $this->module->l('Image Successfully Deleted'),
        ]));
    }

    public function displayAjaxRenderCookieTypeForm()
    {
        $this->ajaxDie(json_encode([
            'content' => $this->adminForms->reanderTypeForm(Tools::getValue('idItem')),
            // 'contentType' => 'modal'
        ]));
    }

    public function displayAjaxSaveCookieType()
    {
        $this->module->saveCookieType();

        $this->ajaxDie(json_encode([
            'error' => false,
            'message' => $this->module->l('Successfully saved'),
            'content' => $this->adminForms->renderCookieTypesList((int) Tools::getValue('id_type')),
            // 'contentType' => 'modal'
        ]));
    }

    public function displayAjaxRenderCookieTypeList()
    {
        $this->ajaxDie(json_encode([
            'error' => false,
            'content' => $this->adminForms->renderCookieTypesList((int) Tools::getValue('id_type')),
        ]));
    }

    public function displayAjaxDeleteCookieType()
    {
        $idType = (int) Tools::getValue('idElement');
        $type = new HiCookieType($idType);

        if ($type->delete()) {
            $this->ajaxDie(json_encode([
                'error' => false,
                'message' => $this->module->l('Cookie Type successfully deleted'),
                'content' => $this->adminForms->renderCookieTypesList(),
            ]));
        }

        $this->ajaxDie(json_encode([
            'error' => $this->module->l('Something went wrong, please refresh the page and try again'),
        ]));
    }

    public function displayAjaxSortCookieTypeElements()
    {
        $sortedItems = Tools::getValue('sortedItems');
        if (is_array($sortedItems) && $sortedItems) {
            $i = 0;
            foreach ($sortedItems as $idType) {
                Db::getInstance()->Execute('
                    UPDATE ' . _DB_PREFIX_ . 'hicookietype 
                    SET position=' . (int) $i . '
                    WHERE id_type =' . (int) $idType . '
                ');

                ++$i;
            }

            $this->ajaxDie(json_encode([
                'error' => false,
                'message' => $this->module->l('Successfully updated'),
            ]));
        }

        $this->ajaxDie(json_encode([
            'error' => $this->module->l('Something went wrong, please refresh the page and try again.'),
        ]));
    }

    public function displayAjaxUpdateCookieTypeStatus()
    {
        if (Tools::getValue('statusType') == 'defaultStatus') {
            $type = new HiCookieType((int) Tools::getValue('idElement'));
            $type->default_status = $type->default_status ? 0 : 1;
            $type->update();

            $this->ajaxDie(json_encode([
                'error' => false,
                'message' => $this->module->l('Status successfully changed'),
                'content' => $this->adminForms->renderCookieTypesList(),
            ]));
        }

        if (Tools::getValue('statusType') == 'requiredCookie') {
            $type = new HiCookieType((int) Tools::getValue('idElement'));
            $type->disabled = $type->disabled ? 0 : 1;
            $type->update();

            $this->ajaxDie(json_encode([
                'error' => false,
                'message' => $this->module->l('Status successfully changed'),
                'content' => $this->adminForms->renderCookieTypesList(),
            ]));
        }

        $this->ajaxDie(json_encode([
            'error' => $this->module->l('Action not found.'),
        ]));
    }

    public function displayAjaxHideMenuModule()
    {
        $this->context->cookie->hideMenuModule = 1;
        $this->context->cookie->write();

        $this->ajaxDie(json_encode([
            'error' => false,
        ]));
    }

    public function displayAjaxClearLogs()
    {
        HiCookieConsent::clearLogs();

        exit(json_encode([
            'error' => false,
            'content' => $this->adminForms->renderConsentsList(),
        ]));
    }

    public function displayAjaxUploadBannerLogo()
    {
        $res = $this->module->uploadImage('bannerLogo');

        if ($res['error']) {
            $this->ajaxDie(json_encode([
                'error' => $res['error'],
            ]));
        }

        Configuration::updateValue('HI_COOKIE_BANNER_LOGO', $res['fileName']);

        $this->ajaxDie(json_encode([
            'error' => false,
            'fileName' => $res['fileName'],
            'id' => $res['fileName'],
        ]));
    }
}
