<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @version   1.4.6
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

include_once dirname(__FILE__) . '/classes/HiPrestaModule.php';
include_once dirname(__FILE__) . '/classes/adminForms.php';
include_once dirname(__FILE__) . '/classes/consent.php';
include_once dirname(__FILE__) . '/classes/type.php';

class HiCookieLaw extends Module implements WidgetInterface
{
    public $psv;
    public $errors = [];
    public $success = [];
    public $clean_db;
    public $module_hooks = [];

    /* Settings */
    public $template;
    public $cookielaw_text;
    public $position;
    public $bg_color;
    public $text_color;
    public $borders_color;
    public $border_radius;
    public $accept_text_color;
    public $customize_text_color;
    public $info_text_color;
    public $custom_css;
    public $padding;
    public $popup_width;
    public $necessary_button_bg_color;
    public $necessary_button_text_color;
    public $accept_all_button_bg_color;
    public $accept_all_button_text_color;

    public $displayShopLogo;
    public $bannerLogo;

    public $info_page_id;
    public $cookie_countries = [];
    public $whitelist_ips;
    public $whitelist_ips_array = [];

    public $consent_on_navigation;
    public $reload_page;
    public $block_navigation;

    public $cookie_settings;
    public $cookie_settings_top;
    public $cookie_settings_custom;

    public $seo_protection;

    // Google Consent Mode
    public $consentMode;
    public $adStorageDefaultValue;
    public $analyticsStorageDefaultValue;
    public $adStorageCookieType;
    public $analyticsStorageCookieType;
    public $adsDataRedaction;
    public $urlPassthrough;

    // Google Consent Mode V2
    public $adUserDataDefaultValue;
    public $adUserDataCookieType;
    public $adPersonalizationDefaultValue;
    public $adPersonalizationCookieType;

    private $settingsTemplateFile;

    public $continueWithoutAccepting;
    public $bannerHook;

    public function __construct()
    {
        $this->name = 'hicookielaw';
        $this->tab = 'front_office_features';
        $this->version = '1.4.6';
        $this->author = 'hipresta';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;
        $this->module_key = 'ed785e289986ec635b77555b5ac57856';
        parent::__construct();
        $this->globalVars();
        $this->displayName = $this->l('Cookie Law GDPR - France, Spain, Italy ... + Google Consent Mode V2');
        $this->description = $this->l('An easy way to comply with the EU Data Protection Regulation (GDPR) law on cookies by adding a notification banner with an Absolutely Innovative Design.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->hiPrestaClass = new HiPrestaCookieLaw($this);
        $this->adminForms = new HiCookieAdminForms($this);

        $this->settingsTemplateFile = 'module:' . $this->name . '/views/templates/hook/cookie-settings.tpl';

        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
    }

    public function install()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }
        if (!parent::install()
            || !$this->registerHook('displayHeader')
            || !$this->registerHook('displayFooter')
            || !$this->registerHook('displayNav2')
            || !$this->registerHook('displayTop')
            || !$this->registerHook('displayHiCookieSettings')
            || !$this->hiPrestaClass->createTabs('AdminHiCookie', 'AdminHiCookie', 'CONTROLLER_TABS_HICOOKIE', 0)
        ) {
            return false;
        }
        $this->proceedDb();

        return true;
    }

    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }
        if (Configuration::get('CLEAN_HI_COOKIELAW_DB')) {
            $this->proceedDb(true);
        }
        $this->hiPrestaClass->deleteTabs('CONTROLLER_TABS_HICOOKIE');

        return true;
    }

    private function proceedDb($drop = false)
    {
        if (!$drop) {
            Configuration::updateValue('HI_COOKIE_TEMPLATE', 1);
            Configuration::updateValue('CLEAN_HI_COOKIELAW_DB', false);
            foreach (Language::getLanguages(false) as $lang) {
                Configuration::updateValue('HI_COOKIE_LAW_TEXT', [$lang['id_lang'] => 'We use cookies to personalise content and ads, to provide social media features and to analyse our traffic. We also share information about your use of our site with our social media, advertising and analytics partners who may combine it with other information that you’ve provided to them or that they’ve collected from your use of their services.']);
            }
            Configuration::updateValue('HI_COOKIE_DISPLAY_SHOP_LOGO', 1);
            Configuration::updateValue('HI_COOKIE_BANNER_LOGO', false);
            Configuration::updateValue('HI_COOKIE_POSITION', 'bottomleft');
            Configuration::updateValue('HI_COOKIE_BG_COLOR', '#ffffff');
            Configuration::updateValue('HI_COOKIE_TEXT_COLOR', '#484848');
            Configuration::updateValue('HI_COOKIE_BORDERS_COLOR', '#f0f0f0');
            Configuration::updateValue('HI_COOKIE_BORDER_RADIUS', '8');
            Configuration::updateValue('HI_COOKIE_ACCEPT_TEXT_COLOR', '#7a7a7a');
            Configuration::updateValue('HI_COOKIE_CUSTOMIZE_TEXT_COLOR', '#7a7a7a');
            Configuration::updateValue('HI_COOKIE_INFO_TEXT_COLOR', '#7a7a7a');
            Configuration::updateValue('HI_COOKIE_COUNTRIES', json_encode([]));
            Configuration::updateValue('HI_COOKIE_CONSENT_ON_NAVIGATION', false);
            Configuration::updateValue('HI_COOKIE_RELOAD_PAGE', false);
            Configuration::updateValue('HI_COOKIE_BLOCK_NAVIGATION', false);
            Configuration::updateValue('HI_COOKIE_SETTING_BUTTON', false);
            Configuration::updateValue('HI_COOKIE_SETTING_BUTTON_TOP', 1);
            Configuration::updateValue('HI_COOKIE_SETTING_BUTTON_CUSTOM', 0);
            Configuration::updateValue('HI_COOKIE_SEO_PROTECTION', true);

            Configuration::updateValue('HI_COOKIE_PADDING_TOP', 25);
            Configuration::updateValue('HI_COOKIE_PADDING_RIGHT', 20);
            Configuration::updateValue('HI_COOKIE_PADDING_BOTTOM', 0);
            Configuration::updateValue('HI_COOKIE_PADDING_LEFT', 20);
            Configuration::updateValue('HI_COOKIE_POPUP_WIDTH', 400);

            Configuration::updateValue('HI_COOKIE_NEC_B_BG_COLOR', '#f0f0f0');
            Configuration::updateValue('HI_COOKIE_NEC_B_TEXT_COLOR', '#505050');
            Configuration::updateValue('HI_COOKIE_ALL_B_BG_COLOR', '#478cff');
            Configuration::updateValue('HI_COOKIE_ALL_B_TEXT_COLOR', '#ffffff');

            // Google Consent Mode
            Configuration::updateValue('HI_COOKIE_CONSENT_MODE', 0);
            Configuration::updateValue('HI_COOKIE_AD_STORAGE', 'denied');
            Configuration::updateValue('HI_COOKIE_ANALYTICS_STORAGE', 'denied');
            Configuration::updateValue('HI_COOKIE_AD_STORAGE_TYPE', 0);
            Configuration::updateValue('HI_COOKIE_ANALYTICS_STORAGE_TYPE', 0);
            Configuration::updateValue('HI_COOKIE_ADS_DATA_REDACTION', 0);
            Configuration::updateValue('HI_COOKIE_URL_PASSTHROUGH', 0);

            // Google Consent Mode V2
            Configuration::updateValue('HI_COOKIE_AD_USER_DATA', 'denied');
            Configuration::updateValue('HI_COOKIE_AD_PERSONALIZATION', 'denied');
            Configuration::updateValue('HI_COOKIE_AD_USER_DATA_TYPE', 0);
            Configuration::updateValue('HI_COOKIE_AD_PERSONALIZATION_TYPE', 0);

            Configuration::updateValue('HI_COOKIE_CONTINUE_WITHOUT_ACCEPTING', false);
            Configuration::updateValue('HI_COOKIE_BANNER_HOOK', 'displayFooter');

            Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'hicookieconsent` (
                    `id_consent` int(10) unsigned NOT NULL AUTO_INCREMENT,
                    `id_shop` int(10) NOT NULL,
                    `ip_address` varchar(255) NOT NULL,
                    `id_guest` int(10) NOT NULL,
                    `id_customer` int(10) NOT NULL,
                    `id_web_browser` int(10) NOT NULL,
                    `id_operating_system` int(10) NOT NULL,
                    `consents` text NOT NULL,
                    `date_add` datetime NOT NULL,
                    `date_upd` datetime NOT NULL,
                    PRIMARY KEY (`id_consent`)
                ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=UTF8;
            ');

            Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'hicookietype` (
                    `id_type` int(10) unsigned NOT NULL AUTO_INCREMENT,
                    `disabled` tinyint unsigned NOT NULL,
                    `default_status` tinyint unsigned NOT NULL,
                    `script` text NOT NULL,
                    `position` int(10) unsigned NOT NULL,
                    `date_add` datetime NOT NULL,
                    `date_upd` datetime NOT NULL,
                    PRIMARY KEY (`id_type`)
                ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=UTF8;
            ');

            Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'hicookietype_lang` (
                    `id_type` int(10) unsigned NOT NULL,
                    `id_lang` int(10) unsigned NOT NULL,
                    `name` varchar(255) NOT NULL,
                    `description` text not null,
                    PRIMARY KEY (`id_type`, `id_lang`)
                ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=UTF8;
            ');

            Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'hicookietype_shop` (
                    `id_type` int(10) unsigned NOT NULL,
                    `id_shop` int(10) unsigned NOT NULL,
                    PRIMARY KEY (`id_type`, `id_shop`)
                ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=UTF8;
            ');

            Db::getInstance()->execute('
                CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'hicookietypemodule` (
                    `id_type` int(10) unsigned NOT NULL,
                    `id_module` int(10) unsigned NOT NULL,
                    PRIMARY KEY (`id_type`, `id_module`)
                ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=UTF8;
            ');
        } else {
            Configuration::deleteByName('HI_COOKIE_TEMPLATE');
            Configuration::deleteByName('CLEAN_HI_COOKIELAW_DB');
            Configuration::deleteByName('HI_COOKIE_LAW_TEXT');
            Configuration::deleteByName('HI_COOKIE_DISPLAY_SHOP_LOGO');
            Configuration::deleteByName('HI_COOKIE_BANNER_LOGO');
            Configuration::deleteByName('HI_COOKIE_POSITION');
            Configuration::deleteByName('HI_COOKIE_BG_COLOR');
            Configuration::deleteByName('HI_COOKIE_TEXT_COLOR');
            Configuration::deleteByName('HI_COOKIE_BORDERS_COLOR');
            Configuration::deleteByName('HI_COOKIE_BORDER_RADIUS');
            Configuration::deleteByName('HI_COOKIE_ACCEPT_TEXT_COLOR');
            Configuration::deleteByName('HI_COOKIE_CUSTOMIZE_TEXT_COLOR');
            Configuration::deleteByName('HI_COOKIE_INFO_TEXT_COLOR');
            Configuration::deleteByName('HI_COOKIE_CUSTOM_CSS');
            Configuration::deleteByName('HI_COOKIE_INFO_PAGE_ID');
            Configuration::deleteByName('HI_COOKIE_COUNTRIES');
            Configuration::deleteByName('HI_COOKIE_WHITELIST_IPS');
            Configuration::deleteByName('HI_COOKIE_CONSENT_ON_NAVIGATION');
            Configuration::deleteByName('HI_COOKIE_RELOAD_PAGE');
            Configuration::deleteByName('HI_COOKIE_BLOCK_NAVIGATION');
            Configuration::deleteByName('HI_COOKIE_SETTING_BUTTON');
            Configuration::deleteByName('HI_COOKIE_SETTING_BUTTON_TOP');
            Configuration::deleteByName('HI_COOKIE_SETTING_BUTTON_CUSTOM');
            Configuration::deleteByName('HI_COOKIE_SEO_PROTECTION');

            Configuration::deleteByName('HI_COOKIE_PADDING_TOP');
            Configuration::deleteByName('HI_COOKIE_PADDING_RIGHT');
            Configuration::deleteByName('HI_COOKIE_PADDING_BOTTOM');
            Configuration::deleteByName('HI_COOKIE_PADDING_LEFT');
            Configuration::deleteByName('HI_COOKIE_POPUP_WIDTH');

            Configuration::deleteByName('HI_COOKIE_NEC_B_BG_COLOR');
            Configuration::deleteByName('HI_COOKIE_NEC_B_TEXT_COLOR');
            Configuration::deleteByName('HI_COOKIE_ALL_B_BG_COLOR');
            Configuration::deleteByName('HI_COOKIE_ALL_B_TEXT_COLOR');

            // Google Consent Mode
            Configuration::deleteByName('HI_COOKIE_CONSENT_MODE');
            Configuration::deleteByName('HI_COOKIE_AD_STORAGE');
            Configuration::deleteByName('HI_COOKIE_ANALYTICS_STORAGE');
            Configuration::deleteByName('HI_COOKIE_AD_STORAGE_TYPE');
            Configuration::deleteByName('HI_COOKIE_ANALYTICS_STORAGE_TYPE');
            Configuration::deleteByName('HI_COOKIE_ADS_DATA_REDACTION');
            Configuration::deleteByName('HI_COOKIE_URL_PASSTHROUGH');

            // Google Consent Mode V2
            Configuration::deleteByName('HI_COOKIE_AD_USER_DATA');
            Configuration::deleteByName('HI_COOKIE_AD_PERSONALIZATION');
            Configuration::deleteByName('HI_COOKIE_AD_USER_DATA_TYPE');
            Configuration::deleteByName('HI_COOKIE_AD_PERSONALIZATION_TYPE');

            Configuration::deleteByName('HI_COOKIE_CONTINUE_WITHOUT_ACCEPTING');
            Configuration::deleteByName('HI_COOKIE_BANNER_HOOK');

            DB::getInstance()->Execute('DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'hicookieconsent');
            DB::getInstance()->Execute('DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'hicookietype');
            DB::getInstance()->Execute('DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'hicookietype_lang');
            DB::getInstance()->Execute('DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'hicookietype_shop');
            DB::getInstance()->Execute('DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'hicookietypemodule');
        }
    }

    private function globalVars()
    {
        $this->psv = (float) Tools::substr(_PS_VERSION_, 0, 3);
        $this->clean_db = (bool) Configuration::get('CLEAN_HI_COOKIELAW_DB');
        foreach (Language::getLanguages(false) as $language) {
            $this->cookielaw_text[$language['id_lang']] = Configuration::get('HI_COOKIE_LAW_TEXT', $language['id_lang']);
        }
        $this->displayShopLogo = (bool) Configuration::get('HI_COOKIE_DISPLAY_SHOP_LOGO');
        $this->bannerLogo = Configuration::get('HI_COOKIE_BANNER_LOGO');
        $this->template = (int) Configuration::get('HI_COOKIE_TEMPLATE');
        if (!$this->template) {
            $this->template = 1;
        }
        $this->position = Configuration::get('HI_COOKIE_POSITION');
        $this->bg_color = Configuration::get('HI_COOKIE_BG_COLOR');
        $this->text_color = Configuration::get('HI_COOKIE_TEXT_COLOR');
        $this->borders_color = Configuration::get('HI_COOKIE_BORDERS_COLOR');
        $this->border_radius = (int) Configuration::get('HI_COOKIE_BORDER_RADIUS');
        $this->accept_text_color = Configuration::get('HI_COOKIE_ACCEPT_TEXT_COLOR');
        $this->customize_text_color = Configuration::get('HI_COOKIE_CUSTOMIZE_TEXT_COLOR');
        $this->info_text_color = Configuration::get('HI_COOKIE_INFO_TEXT_COLOR');
        $this->custom_css = Configuration::get('HI_COOKIE_CUSTOM_CSS');

        $this->cookie_countries = json_decode(Configuration::get('HI_COOKIE_COUNTRIES'), true);

        $this->whitelist_ips = trim(Configuration::get('HI_COOKIE_WHITELIST_IPS'));
        if ($this->whitelist_ips) {
            $this->whitelist_ips_array = explode(',', $this->whitelist_ips);
        }

        $this->info_page_id = (int) Configuration::get('HI_COOKIE_INFO_PAGE_ID');
        $this->consent_on_navigation = (bool) Configuration::get('HI_COOKIE_CONSENT_ON_NAVIGATION');
        $this->reload_page = (bool) Configuration::get('HI_COOKIE_RELOAD_PAGE');
        $this->block_navigation = (bool) Configuration::get('HI_COOKIE_BLOCK_NAVIGATION');
        $this->cookie_settings = (bool) Configuration::get('HI_COOKIE_SETTING_BUTTON');
        $this->cookie_settings_top = (bool) Configuration::get('HI_COOKIE_SETTING_BUTTON_TOP');
        $this->cookie_settings_custom = (bool) Configuration::get('HI_COOKIE_SETTING_BUTTON_CUSTOM');
        $this->seo_protection = (bool) Configuration::get('HI_COOKIE_SEO_PROTECTION');

        $this->padding = [
            'top' => (int) Configuration::get('HI_COOKIE_PADDING_TOP'),
            'right' => (int) Configuration::get('HI_COOKIE_PADDING_RIGHT'),
            'bottom' => (int) Configuration::get('HI_COOKIE_PADDING_BOTTOM'),
            'left' => (int) Configuration::get('HI_COOKIE_PADDING_LEFT'),
        ];
        $this->popup_width = (int) Configuration::get('HI_COOKIE_POPUP_WIDTH');
        if (!$this->popup_width) {
            $this->popup_width = 400;
        }
        $this->necessary_button_bg_color = Configuration::get('HI_COOKIE_NEC_B_BG_COLOR');
        $this->necessary_button_text_color = Configuration::get('HI_COOKIE_NEC_B_TEXT_COLOR');
        $this->accept_all_button_bg_color = Configuration::get('HI_COOKIE_ALL_B_BG_COLOR');
        $this->accept_all_button_text_color = Configuration::get('HI_COOKIE_ALL_B_TEXT_COLOR');

        $this->module_file = __FILE__;

        // Google Consent Mode
        $this->consentMode = (bool) Configuration::get('HI_COOKIE_CONSENT_MODE');
        $this->adStorageDefaultValue = Configuration::get('HI_COOKIE_AD_STORAGE');
        $this->analyticsStorageDefaultValue = Configuration::get('HI_COOKIE_ANALYTICS_STORAGE');
        $this->adStorageCookieType = (int) Configuration::get('HI_COOKIE_AD_STORAGE_TYPE');
        $this->analyticsStorageCookieType = (int) Configuration::get('HI_COOKIE_ANALYTICS_STORAGE_TYPE');
        $this->adsDataRedaction = (bool) Configuration::get('HI_COOKIE_ADS_DATA_REDACTION');
        $this->urlPassthrough = (bool) Configuration::get('HI_COOKIE_URL_PASSTHROUGH');

        // Google Consent Mode V2
        $this->adUserDataDefaultValue = Configuration::get('HI_COOKIE_AD_USER_DATA');
        $this->adUserDataCookieType = (int) Configuration::get('HI_COOKIE_AD_USER_DATA_TYPE');
        $this->adPersonalizationDefaultValue = Configuration::get('HI_COOKIE_AD_PERSONALIZATION');
        $this->adPersonalizationCookieType = (int) Configuration::get('HI_COOKIE_AD_PERSONALIZATION_TYPE');

        $this->continueWithoutAccepting = (bool) Configuration::get('HI_COOKIE_CONTINUE_WITHOUT_ACCEPTING');
        $this->bannerHook = Configuration::get('HI_COOKIE_BANNER_HOOK');
    }

    public function getBlockedModules()
    {
        if (!isset($this->blocked_modules)) {
            $this->blocked_modules = array_map('current', HiCookieType::getAllBlockedModules());
        }

        return $this->blocked_modules;
    }

    public function renderMenuTabs()
    {
        $tabs = [
            'generel_settings' => [
                'title' => $this->l('General Settings'),
                'icon' => 'icon-cog',
            ],
            'types' => [
                'title' => $this->l('Cookie Types'),
                'icon' => 'icon-globe',
            ],
            'design_settings' => [
                'title' => $this->l('Design Settings'),
                'icon' => 'icon-paint-brush',
            ],
            'googleConsentMode' => [
                'title' => $this->l('Google Consent Mode'),
                'icon' => 'icon-google',
            ],
            'consents' => [
                'title' => $this->l('Consents'),
                'icon' => 'icon-user',
            ],
            'rateMe' => [
                'title' => $this->l('Leave a review'),
                'icon' => 'icon-star',
                'url' => $this->getRateUrl(),
            ],
            'contactUs' => [
                'title' => $this->l('Contact Us'),
                'icon' => 'icon-support',
                'url' => $this->getContactUrl(),
            ],
            'version' => [
                'title' => $this->l('Version'),
                'icon' => 'icon-info',
            ],
        ];

        $more_modules = $this->hiPrestaClass->getModuleContent('A_LAW');
        if ($more_modules) {
            $tabs['more_modules'] = [
                'title' => $this->l('More Modules'),
                'icon' => 'icon-plus-square',
            ];
        }

        $displayMenuModule = true;

        if (!empty($this->context->cookie->hideMenuModule)) {
            $displayMenuModule = false;
        }

        $this->context->smarty->assign(
            [
                'psv' => $this->psv,
                'tabs' => $tabs,
                'module_version' => $this->version,
                'module_url' => $this->hiPrestaClass->getModuleUrl(),
                'module_tab_key' => $this->name,
                'active_tab' => Tools::getValue($this->name),
                'displayMenuModule' => $displayMenuModule,
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/menu_tabs.tpl');
    }

    public function getRateUrl()
    {
        $langIsoCode = $this->context->language->iso_code;
        $psLanguages = ['en', 'fr', 'es', 'de', 'it', 'nl', 'pl', 'pt', 'ru'];

        if (in_array($langIsoCode, $psLanguages)) {
            return 'https://addons.prestashop.com/' . $this->context->language->iso_code . '/ratings.php';
        }

        return 'https://addons.prestashop.com/en/ratings.php';
    }

    public function getContactUrl()
    {
        $langIsoCode = $this->context->language->iso_code;
        $psLanguages = ['en', 'fr', 'es', 'de', 'it', 'nl', 'pl', 'pt', 'ru'];

        if (in_array($langIsoCode, $psLanguages)) {
            return 'https://addons.prestashop.com/' . $this->context->language->iso_code . '/contact-us?id_product=48223';
        }

        return 'https://addons.prestashop.com/en/contact-us?id_product=48223';
    }

    public function renderVersionForm()
    {
        $changelog = '';
        if (file_exists(dirname(__FILE__) . '/changelog.txt')) {
            $changelog = Tools::file_get_contents(dirname(__FILE__) . '/changelog.txt');
        }
        $this->context->smarty->assign('changelog', $changelog);

        return $this->display(__FILE__, 'views/templates/admin/version.tpl');
    }

    public function renderShopGroupError()
    {
        $this->context->smarty->assign(
            [
                'psv' => $this->psv,
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/shop_group_error.tpl');
    }

    public function renderModuleAdminVariables()
    {
        $this->context->smarty->assign(
            [
                'psv' => $this->psv,
                'id_lang' => $this->context->language->id,
                'hicookie_secure_key' => $this->secure_key,
                'hicookie_admin_controller' => $this->context->link->getAdminLink('AdminHiCookie'),
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/variables.tpl');
    }

    public function renderDisplayForm($content)
    {
        $this->context->smarty->assign(
            [
                'psv' => $this->psv,
                'errors' => $this->errors,
                'success' => $this->success,
                'content' => $content,
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/display_form.tpl');
    }

    public function renderDocumentation()
    {
        $this->context->smarty->assign([
            'moduleAssetsDir' => _MODULE_DIR_ . $this->name . '/libs/hi-modules-doc/img/',
            'contactLink' => $this->getContactUrl(),
        ]);

        return $this->display(__FILE__, 'libs/hi-modules-doc/doc.tpl');
    }

    public function getAdminAjaxUrl($params = [], $controller = false)
    {
        if (!$controller) {
            $controller = 'AdminHiCookie';
        }

        if ($this->psv >= 1.7) {
            return $this->context->link->getAdminLink($controller, true, [], $params);
        }

        return $this->context->link->getAdminLink($controller) . '&' . http_build_query($params);
    }

    public function uploadImage($name)
    {
        if (isset($_FILES[$name]) && !empty($_FILES[$name]['name'])) {
            if (!isset($_FILES[$name]) || empty($_FILES[$name]['tmp_name'])) {
                return [
                    'error' => $this->l('Something went wrong, please try again'),
                ];
            }

            $type = Tools::strtolower(Tools::substr(strrchr($_FILES[$name]['name'], '.'), 1));
            $imagesize = [];
            $imagesize = getimagesize($_FILES[$name]['tmp_name']);
            $allowedFiles = ['jpg', 'gif', 'jpeg', 'png'];

            if (empty($imagesize) || !in_array(Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), $allowedFiles) || !in_array($type, $allowedFiles)) {
                return [
                    'error' => $this->l('Invalid file is uploaded, allowed files are: jpg, jpeg, gif, png'),
                ];
            }

            $imageName = $_FILES[$name]['name'];
            $filesLocation = dirname(__FILE__) . '/views/img/upload/';
            // get available file name
            if (file_exists($filesLocation . $imageName)) {
                $i = 0;
                $ext_array = explode('.', $imageName);
                $ext = array_pop($ext_array);
                $fileName = Tools::substr($imageName, 0, -1 * Tools::strlen($ext));
                do {
                    ++$i;
                    $imageName = $fileName . $i . '.' . $ext;
                } while (file_exists($filesLocation . $imageName));
            }

            $originalFilePath = _PS_MODULE_DIR_ . $this->name . '/views/img/upload/' . $imageName;

            if (!move_uploaded_file($_FILES[$name]['tmp_name'], $originalFilePath)) {
                return [
                    'error' => $this->l('We coulnd\'t upload the file, please try again'),
                ];
            }

            return [
                'error' => false,
                'fileName' => $imageName,
            ];
        }

        return [
            'error' => $this->l('File not found!'),
        ];
    }

    public function postProcess()
    {
        $languages = Language::getLanguages(false);
        if (Tools::isSubmit('submit_settings_form')) {
            if (Tools::getValue('cookielaw_text_' . Configuration::get('PS_LANG_DEFAULT')) == '') {
                $this->errors[] = $this->l('Invalide value for Cookie law text');
            } else {
                Configuration::updateValue('CLEAN_HI_COOKIELAW_DB', (bool) Tools::getValue('clean_db'));
                foreach ($languages as $lang) {
                    Configuration::updateValue('HI_COOKIE_LAW_TEXT', [$lang['id_lang'] => Tools::getValue('cookielaw_text_' . $lang['id_lang'])], true);
                }
                if ($this->template != Tools::getValue('template')) {
                    if ((int) Tools::getValue('template') == 1) {
                        Configuration::updateValue('HI_COOKIE_PADDING_RIGHT', 20);
                        Configuration::updateValue('HI_COOKIE_PADDING_BOTTOM', 0);
                        Configuration::updateValue('HI_COOKIE_PADDING_LEFT', 20);
                        Configuration::updateValue('HI_COOKIE_POPUP_WIDTH', 400);
                    } elseif ((int) Tools::getValue('template') == 2) {
                        Configuration::updateValue('HI_COOKIE_PADDING_RIGHT', 40);
                        Configuration::updateValue('HI_COOKIE_PADDING_BOTTOM', 30);
                        Configuration::updateValue('HI_COOKIE_PADDING_LEFT', 40);
                        Configuration::updateValue('HI_COOKIE_POPUP_WIDTH', 520);
                    }
                }
                Configuration::updateValue('HI_COOKIE_TEMPLATE', (int) Tools::getValue('template'));
                Configuration::updateValue('HI_COOKIE_POSITION', Tools::getValue('position'));
                Configuration::updateValue('HI_COOKIE_COUNTRIES', json_encode(Tools::getValue('cookie_countries')));
                Configuration::updateValue('HI_COOKIE_WHITELIST_IPS', Tools::getValue('whitelist_ips'));
                Configuration::updateValue('HI_COOKIE_INFO_PAGE_ID', (int) Tools::getValue('info_page_id'));
                Configuration::updateValue('HI_COOKIE_CONSENT_ON_NAVIGATION', (bool) Tools::getValue('consent_on_navigation'));
                Configuration::updateValue('HI_COOKIE_RELOAD_PAGE', (bool) Tools::getValue('reload_page'));
                Configuration::updateValue('HI_COOKIE_BLOCK_NAVIGATION', (bool) Tools::getValue('block_navigation'));
                Configuration::updateValue('HI_COOKIE_SETTING_BUTTON', (int) Tools::getValue('cookie_settings'));
                Configuration::updateValue('HI_COOKIE_SETTING_BUTTON_TOP', (int) Tools::getValue('cookie_settings_position_top'));
                Configuration::updateValue('HI_COOKIE_SETTING_BUTTON_CUSTOM', (int) Tools::getValue('cookie_settings_position_custom'));
                Configuration::updateValue('HI_COOKIE_SEO_PROTECTION', (bool) Tools::getValue('seo_protection'));

                Configuration::updateValue('HI_COOKIE_DISPLAY_SHOP_LOGO', (bool) Tools::getValue('displayShopLogo'));
                Configuration::updateValue('HI_COOKIE_CONTINUE_WITHOUT_ACCEPTING', (bool) Tools::getValue('continueWithoutAccepting'));
                Configuration::updateValue('HI_COOKIE_BANNER_HOOK', Tools::getValue('bannerHook'));

                $this->success[] = $this->l('Successfully saved');
            }
        } elseif (Tools::isSubmit('submit_design_settings_form')) {
            if (!Validate::isColor(trim(Tools::getValue('bg_color'))) || Tools::substr(Tools::getValue('bg_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for background color');
            } elseif (!Validate::isColor(trim(Tools::getValue('text_color'))) || Tools::substr(Tools::getValue('text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for text color');
            } elseif (!Validate::isColor(trim(Tools::getValue('borders_color'))) || Tools::substr(Tools::getValue('borders_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for borders color');
            } elseif (!Validate::isInt(trim(Tools::getValue('border_radius')))) {
                $this->errors[] = $this->l('Invalide value for banner border radius');
            } elseif (!Validate::isColor(trim(Tools::getValue('accept_text_color'))) || Tools::substr(Tools::getValue('accept_text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for accept buton text color');
            } elseif (!Validate::isColor(trim(Tools::getValue('customize_text_color'))) || Tools::substr(Tools::getValue('customize_text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for customize buton text color');
            } elseif (!Validate::isColor(trim(Tools::getValue('info_text_color'))) || Tools::substr(Tools::getValue('info_text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for more info buton text color');
            } elseif (!Validate::isInt(Tools::getValue('padding_top'))) {
                $this->errors[] = $this->l('Padding top is not valid');
            } elseif (!Validate::isInt(Tools::getValue('padding_right'))) {
                $this->errors[] = $this->l('Padding right is not valid');
            } elseif (!Validate::isInt(Tools::getValue('padding_bottom'))) {
                $this->errors[] = $this->l('Padding bottom is not valid');
            } elseif (!Validate::isInt(Tools::getValue('padding_left'))) {
                $this->errors[] = $this->l('Padding left is not valid');
            } elseif (!Validate::isInt(Tools::getValue('popup_width'))) {
                $this->errors[] = $this->l('Banner width is not valid');
            } elseif (!Validate::isColor(trim(Tools::getValue('necessary_button_bg_color'))) || Tools::substr(Tools::getValue('necessary_button_bg_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for Only necessary cookies button background color');
            } elseif (!Validate::isColor(trim(Tools::getValue('necessary_button_text_color'))) || Tools::substr(Tools::getValue('necessary_button_text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for Only necessary cookies button text color');
            } elseif (!Validate::isColor(trim(Tools::getValue('accept_all_button_bg_color'))) || Tools::substr(Tools::getValue('accept_all_button_bg_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for Accept all cookies button background color');
            } elseif (!Validate::isColor(trim(Tools::getValue('accept_all_button_text_color'))) || Tools::substr(Tools::getValue('accept_all_button_text_color'), 0, 1) != '#') {
                $this->errors[] = $this->l('Invalide value for Accept all cookies button text color');
            } else {
                Configuration::updateValue('HI_COOKIE_BG_COLOR', Tools::getValue('bg_color'));
                Configuration::updateValue('HI_COOKIE_TEXT_COLOR', Tools::getValue('text_color'));
                Configuration::updateValue('HI_COOKIE_BORDERS_COLOR', Tools::getValue('borders_color'));
                Configuration::updateValue('HI_COOKIE_BORDER_RADIUS', Tools::getValue('border_radius'));
                Configuration::updateValue('HI_COOKIE_ACCEPT_TEXT_COLOR', Tools::getValue('accept_text_color'));
                Configuration::updateValue('HI_COOKIE_CUSTOMIZE_TEXT_COLOR', Tools::getValue('customize_text_color'));
                Configuration::updateValue('HI_COOKIE_INFO_TEXT_COLOR', Tools::getValue('info_text_color'));
                Configuration::updateValue('HI_COOKIE_CUSTOM_CSS', Tools::getValue('custom_css'));

                Configuration::updateValue('HI_COOKIE_PADDING_TOP', (int) Tools::getValue('padding_top'));
                Configuration::updateValue('HI_COOKIE_PADDING_RIGHT', (int) Tools::getValue('padding_right'));
                Configuration::updateValue('HI_COOKIE_PADDING_BOTTOM', (int) Tools::getValue('padding_bottom'));
                Configuration::updateValue('HI_COOKIE_PADDING_LEFT', (int) Tools::getValue('padding_left'));
                Configuration::updateValue('HI_COOKIE_POPUP_WIDTH', (int) Tools::getValue('popup_width'));

                Configuration::updateValue('HI_COOKIE_NEC_B_BG_COLOR', Tools::getValue('necessary_button_bg_color'));
                Configuration::updateValue('HI_COOKIE_NEC_B_TEXT_COLOR', Tools::getValue('necessary_button_text_color'));
                Configuration::updateValue('HI_COOKIE_ALL_B_BG_COLOR', Tools::getValue('accept_all_button_bg_color'));
                Configuration::updateValue('HI_COOKIE_ALL_B_TEXT_COLOR', Tools::getValue('accept_all_button_text_color'));

                $this->success[] = $this->l('Successfully Saved');
            }
        } elseif (Tools::isSubmit('submitGoogleConsentModeForm')) {
            Configuration::updateValue('HI_COOKIE_CONSENT_MODE', (bool) Tools::getValue('consentMode'));
            Configuration::updateValue('HI_COOKIE_AD_STORAGE', Tools::getValue('adStorageDefaultValue'));
            Configuration::updateValue('HI_COOKIE_ANALYTICS_STORAGE', Tools::getValue('analyticsStorageDefaultValue'));
            Configuration::updateValue('HI_COOKIE_AD_STORAGE_TYPE', (int) Tools::getValue('adStorageCookieType'));
            Configuration::updateValue('HI_COOKIE_ANALYTICS_STORAGE_TYPE', (int) Tools::getValue('analyticsStorageCookieType'));
            Configuration::updateValue('HI_COOKIE_ADS_DATA_REDACTION', (bool) Tools::getValue('adsDataRedaction'));
            Configuration::updateValue('HI_COOKIE_URL_PASSTHROUGH', (bool) Tools::getValue('urlPassthrough'));

            // Google Consent Mode V2
            Configuration::updateValue('HI_COOKIE_AD_USER_DATA', Tools::getValue('adUserDataDefaultValue'));
            Configuration::updateValue('HI_COOKIE_AD_USER_DATA_TYPE', (int) Tools::getValue('adUserDataCookieType'));
            Configuration::updateValue('HI_COOKIE_AD_PERSONALIZATION', Tools::getValue('adPersonalizationDefaultValue'));
            Configuration::updateValue('HI_COOKIE_AD_PERSONALIZATION_TYPE', (int) Tools::getValue('adPersonalizationCookieType'));

            $this->success[] = $this->l('Successfully Saved');
        }
    }

    public function renderModuleAdvertisingForm()
    {
        $this->hiPrestaClass->getModuleContent('A_LAW', false, '', false);

        return $this->display(__FILE__, 'views/templates/admin/moduleadvertising.tpl');
    }

    public function displayForm()
    {
        if ($this->psv < 1.7) {
            Media::addJsDef(['baseDir' => __PS_BASE_URI__]);
        }

        $html = '';
        $content = '';
        if (!$this->hiPrestaClass->isSelectedShopGroup()) {
            $html .= $this->renderMenuTabs();
            switch (Tools::getValue($this->name)) {
                case 'generel_settings':
                    $content .= $this->adminForms->renderSettingsForm();
                    break;
                case 'types':
                    $content .= $this->renderTreeJS();
                    $content .= $this->renderModal();
                    $content .= $this->adminForms->initForms();
                    $content .= $this->adminForms->renderCookieTypesList();
                    break;
                case 'design_settings':
                    $content .= $this->adminForms->renderDesignSettingsForm();
                    break;
                case 'googleConsentMode':
                    $content .= $this->adminForms->renderGoogleConsentsModeForm();
                    break;
                case 'consents':
                    $content .= $this->adminForms->renderConsentsList();
                    break;
                case 'version':
                    $content .= $this->renderVersionForm();
                    break;
                case 'more_modules':
                    $content .= $this->renderModuleAdvertisingForm();
                    break;
                case 'free_module':
                    $content .= $this->renderFreeModuleAdvertisingForm();
                    break;
                default:
                    $content .= $this->adminForms->renderSettingsForm();
                    break;
            }

            $content .= $this->renderDocumentation();

            $html .= $this->renderDisplayForm($content);
        } else {
            $html .= $this->renderShopGroupError();
        }

        // Select2
        $this->context->controller->addCSS($this->_path . 'libs/select2/select2.min.css', 'all');
        $this->context->controller->addCSS($this->_path . 'libs/select2/select2-bootstrap.min.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/select2/select2.min.js');

        if (Tools::getValue($this->name) == 'design_settings' || Tools::getValue($this->name) == 'types') {
            $this->context->controller->addCSS($this->_path . 'libs/codemirror/codemirror.css', 'all');
            $this->context->controller->addJS($this->_path . 'libs/codemirror/codemirror.js');
            $this->context->controller->addJS($this->_path . 'libs/codemirror/css.js');
            $this->context->controller->addJS($this->_path . 'libs/codemirror/javascript.js');
        }

        $this->context->controller->addCSS($this->_path . 'libs/magnific-popup/magnific-popup.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/magnific-popup/jquery.magnific-popup.min.js');

        $this->context->controller->addCSS($this->_path . 'libs/hi-modules-table/table.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/hi-modules-table/table.js');

        $this->context->controller->addCSS($this->_path . 'libs/hi-modules-doc/doc.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/hi-modules-doc/doc.js');

        $this->context->controller->addCSS($this->_path . 'libs/dropzone/dropzone.min.css', 'all');
        $this->context->controller->addCSS($this->_path . 'libs/dropzone/hi-module-dropzone.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/dropzone/dropzone.min.js');
        $this->context->controller->addJS($this->_path . 'libs/dropzone/hi-module-dropzone.js');

        $this->context->controller->addCSS($this->_path . 'views/css/admin.css', 'all');
        $this->context->controller->addJS($this->_path . 'views/js/admin.js');

        $this->context->controller->addJqueryUI('ui.sortable');

        // color picker
        $this->context->controller->addCSS($this->_path . 'libs/spectrum/spectrum.min.css', 'all');
        $this->context->controller->addJS($this->_path . 'libs/spectrum/spectrum.min.js');

        $html .= $this->renderModuleAdminVariables();

        return $html;
    }

    public function isBot()
    {
        if (!$this->seo_protection) {
            return false;
        }

        return isset($_SERVER['HTTP_USER_AGENT']) && preg_match('/bot|crawl|slurp|spider|mediapartners/i', $_SERVER['HTTP_USER_AGENT']);
    }

    public function getContent()
    {
        if (Tools::isSubmit('submit_settings_form')
            || Tools::isSubmit('submit_modules_list')
            || Tools::isSubmit('submit_design_settings_form')
            || Tools::isSubmit('submitGoogleConsentModeForm')) {
            $this->postProcess();
        }
        $this->globalVars();

        return $this->displayForm();
    }

    public function hookDisplayHeader()
    {
        $this->context->controller->addCSS($this->_path . 'views/css/front.css', 'all');
        $this->context->controller->addJS($this->_path . 'views/js/front.js');
        if ($this->position == 'popup' || $this->cookie_settings) {
            $this->context->controller->addCSS($this->_path . 'libs/magnific-popup/magnific-popup.css', 'all');
            $this->context->controller->addJS($this->_path . 'libs/magnific-popup/jquery.magnific-popup.min.js');
        }

        $cookieTypes = HiCookieType::getTypes();
        $acceptedCookies = [];
        if (is_array($cookieTypes) && $cookieTypes) {
            foreach ($cookieTypes as $cookieType) {
                if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $cookieType['id_type']}) && $this->context->cookie->{'hiThirdPartyCookies_' . $cookieType['id_type']}) {
                    $acceptedCookies[$cookieType['id_type']] = true;
                } else {
                    $acceptedCookies[$cookieType['id_type']] = false;
                }
            }
        }

        $consentSettings = [
            'consentMode' => $this->consentMode,
            'adStorageDefaultValue' => $this->adStorageDefaultValue,
            'analyticsStorageDefaultValue' => $this->analyticsStorageDefaultValue,
            'adStorageCookieType' => $this->adStorageCookieType,
            'analyticsStorageCookieType' => $this->analyticsStorageCookieType,
            'adsDataRedaction' => $this->adsDataRedaction,
            'urlPassthrough' => $this->urlPassthrough,

            // Google Consent Mode V2
            'adUserDataDefaultValue' => $this->adUserDataDefaultValue,
            'adUserDataCookieType' => $this->adUserDataCookieType,
            'adPersonalizationDefaultValue' => $this->adPersonalizationDefaultValue,
            'adPersonalizationCookieType' => $this->adPersonalizationCookieType,

            'cookiesAccepted' => (isset($this->context->cookie->hicookie) && $this->context->cookie->hicookie) ? 1 : 0,
        ];

        $consentSettings['adStorageCookieTypeGranted'] = 'denied';
        if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $this->adStorageCookieType})) {
            if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->adStorageCookieType} == 1) {
                $consentSettings['adStorageCookieTypeGranted'] = 'granted';
            }
        }

        $consentSettings['analyticsStorageCookieTypeGranted'] = 'denied';
        if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $this->analyticsStorageCookieType})) {
            if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->analyticsStorageCookieType} == 1) {
                $consentSettings['analyticsStorageCookieTypeGranted'] = 'granted';
            }
        }

        $consentSettings['adUserDataCookieTypeGranted'] = 'denied';
        if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $this->adUserDataCookieType})) {
            if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->adUserDataCookieType} == 1) {
                $consentSettings['adUserDataCookieTypeGranted'] = 'granted';
            }
        }

        $consentSettings['adPersonalizationCookieTypeGranted'] = 'denied';
        if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $this->adPersonalizationCookieType})) {
            if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->adPersonalizationCookieType} == 1) {
                $consentSettings['adPersonalizationCookieTypeGranted'] = 'granted';
            }
        }

        $this->context->smarty->assign([
            'hiCookieConsentMode' => $this->consentMode,
            'cookieTypes' => $cookieTypes,
            'hiAcceptedCookies' => $acceptedCookies,
        ]);

        Media::addJsDef([
            'hiCookieGoogleConsentSettings' => $consentSettings,
            'hiAcceptedCookies' => $acceptedCookies,
        ]);

        $bannerContent = '';
        if ($this->bannerHook == 'displayHeader') {
            $bannerContent = $this->renderBannerContent();
        }

        return $bannerContent . $this->display(__FILE__, 'header.tpl');
    }

    public function renderBannerContent()
    {
        if ($this->isBot()) {
            return;
        }

        $display_banner = true;
        $cookiesAccepted = false;
        if (isset($this->context->cookie->hicookie) && $this->context->cookie->hicookie) {
            $display_banner = false;
            $cookiesAccepted = true;
        }

        if ($display_banner && $this->cookie_countries) {
            $address = json_decode(Tools::file_get_contents('http://geoplugin.net/json.gp?ip=' . trim(Tools::getRemoteAddr())));
            if (isset($address->geoplugin_countryCode) && $address->geoplugin_countryCode) {
                $id_country = Country::getByIso($address->geoplugin_countryCode);
                if ($id_country && in_array($id_country, $this->cookie_countries)) {
                    $display_banner = false;
                }
            }
        }

        if ($this->isIPWhiteListed()) {
            $display_banner = true;
        }

        if ($display_banner || $this->cookie_settings) {
            if (isset($this->context->cookie->hiThirdPartyCookies) && $this->context->cookie->hiThirdPartyCookies) {
                $hiThirdPartyCookies = true;
            } else {
                $hiThirdPartyCookies = false;
            }

            $block_navigation = $this->block_navigation;

            if ($this->info_page_id && Tools::getValue('id_cms') == $this->info_page_id) {
                $block_navigation = false;
            }

            $cookieTypes = HiCookieType::getTypes();
            foreach ($cookieTypes as $key => $cookieType) {
                if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $cookieType['id_type']}) && $this->context->cookie->{'hiThirdPartyCookies_' . $cookieType['id_type']}) {
                    $cookieTypes[$key]['accepted'] = true;
                } else {
                    $cookieTypes[$key]['accepted'] = false;
                }
            }

            $this->context->smarty->assign([
                'psv' => $this->psv,
                'cookielaw_text' => $this->cookielaw_text[$this->context->language->id],
                'position' => $this->position,
                'bg_color' => $this->bg_color,
                'text_color' => $this->text_color,
                'borders_color' => $this->borders_color,
                'border_radius' => $this->border_radius,
                'accept_text_color' => $this->accept_text_color,
                'customize_text_color' => $this->customize_text_color,
                'info_text_color' => $this->info_text_color,
                'custom_css' => $this->custom_css,
                'info_page_id' => $this->info_page_id,
                'hicookie_front_url' => $this->context->link->getModuleLink($this->name, 'cookie'),
                'consent_on_navigation' => $this->consent_on_navigation,
                'reload_page' => $this->reload_page,
                'block_navigation' => $block_navigation,
                'cookie_settings' => $this->cookie_settings,
                'display_banner' => $display_banner,
                'thirdPartyCookies' => $hiThirdPartyCookies,
                'secure_key' => $this->secure_key,
                'banner_padding' => $this->padding,
                'popup_width' => $this->popup_width,
                'necessary_button_bg_color' => $this->necessary_button_bg_color,
                'necessary_button_text_color' => $this->necessary_button_text_color,
                'accept_all_button_bg_color' => $this->accept_all_button_bg_color,
                'accept_all_button_text_color' => $this->accept_all_button_text_color,
                'hiCookieTypes' => $cookieTypes,
                'hiCookiesAccepted' => $cookiesAccepted,
                'displayShopLogo' => $this->displayShopLogo,
                'bannerLogo' => $this->bannerLogo,
                'cookieAssetsDir' => __PS_BASE_URI__ . 'modules/' . $this->name . '/views/',
                'continueWithoutAccepting' => $this->continueWithoutAccepting,
            ]);

            return $this->display(__FILE__, 'footer.tpl') . $this->display(__FILE__, 'cookie-banner-template-' . $this->template . '.tpl');
        }
    }

    public function hookDisplayFooter()
    {
        if ($this->bannerHook == 'displayFooter') {
            return $this->renderBannerContent();
        }
    }

    public function displayCookieSettings($hookName = null)
    {
        if ($this->cookie_settings && $this->context->cookie->hicookie) {
            $this->context->smarty->assign('hookName', $hookName);

            return $this->display(__FILE__, 'cookie-settings.tpl');
        }
    }

    public function hookDisplayNav2($params)
    {
        if ($this->psv >= 1.7 && $this->cookie_settings_top) {
            return $this->displayCookieSettings();
        }
    }

    public function hookDisplayTop($params)
    {
        if ($this->psv < 1.7 && $this->cookie_settings_top) {
            return $this->displayCookieSettings();
        }
    }

    public function hookDisplayHiCookieSettings($params)
    {
        if ($this->cookie_settings_custom) {
            return $this->displayCookieSettings();
        }
    }

    public function isIPWhiteListed()
    {
        if (in_array(Tools::getRemoteAddr(), $this->whitelist_ips_array)) {
            return true;
        }

        return false;
    }

    public function isModuleDisabled($id_module)
    {
        // selected modules
        if (!in_array($id_module, $this->getBlockedModules())) {
            return false;
        }

        // is admin page
        if (isset($this->context->employee)) {
            return false;
        }

        $is_ip_whitelisted = $this->isIPWhiteListed();

        // if consent is given
        // get cookie types based on id_module
        $types = HiCookieType::getCookieTypesByModule($id_module);
        $accepted_cookies_count = 0;
        foreach ($types as $type) {
            if (isset($this->context->cookie->{'hiThirdPartyCookies_' . $type['id_type']}) && $this->context->cookie->{'hiThirdPartyCookies_' . $type['id_type']} && !$is_ip_whitelisted) {
                ++$accepted_cookies_count;
            }
        }

        if ($accepted_cookies_count == count($types)) {
            return false;
        }

        // if disabled for current country
        // To-Do: optimize, no need API request for each module.
        if ($this->cookie_countries && !$is_ip_whitelisted) {
            $address = json_decode(Tools::file_get_contents('http://geoplugin.net/json.gp?ip=' . trim(Tools::getRemoteAddr())));
            if (isset($address->geoplugin_countryCode) && $address->geoplugin_countryCode) {
                $id_country = Country::getByIso($address->geoplugin_countryCode);
                if ($id_country && in_array($id_country, $this->cookie_countries)) {
                    return false;
                }
            }
        }

        // SEO Protection
        if ($this->isBot()) {
            return false;
        }

        return true;
    }

    public function getBrowser($userAgent)
    {
        $browserArray = [
            'Chrome' => 'Chrome/',
            'Safari' => 'Safari',
            'Safari iPad' => 'iPad',
            'Firefox' => 'Firefox/',
            'Opera' => 'Opera',
            'IE 11' => 'Trident',
            'IE 10' => 'MSIE 10',
            'IE 9' => 'MSIE 9',
            'IE 8' => 'MSIE 8',
            'IE 7' => 'MSIE 7',
            'IE 6' => 'MSIE 6',
        ];
        foreach ($browserArray as $k => $value) {
            if (strstr($userAgent, $value)) {
                $result = Db::getInstance()->getRow('
                SELECT `id_web_browser`
                FROM `' . _DB_PREFIX_ . 'web_browser` wb
                WHERE wb.`name` = \'' . pSQL($k) . '\'');

                return $result['id_web_browser'];
            }
        }

        return null;
    }

    public function getOs($userAgent)
    {
        $osArray = [
            'Windows 10' => 'Windows NT 10',
            'Windows 8.1' => 'Windows NT 6.3',
            'Windows 8' => 'Windows NT 6.2',
            'Windows 7' => 'Windows NT 6.1',
            'Windows Vista' => 'Windows NT 6.0',
            'Windows XP' => 'Windows NT 5',
            'MacOsX' => 'Mac OS X',
            'Android' => 'Android',
            'Linux' => 'X11',
        ];

        foreach ($osArray as $k => $value) {
            if (strstr($userAgent, $value)) {
                $result = Db::getInstance()->getRow('
                SELECT `id_operating_system`
                FROM `' . _DB_PREFIX_ . 'operating_system` os
                WHERE os.`name` = \'' . pSQL($k) . '\'');

                return $result['id_operating_system'];
            }
        }

        return null;
    }

    public function renderModal($class = null)
    {
        $this->context->smarty->assign(
            [
                'psv' => $this->psv,
                'modal_class' => $class,
            ]
        );

        return $this->display(__FILE__, 'views/templates/admin/modal.tpl');
    }

    public function renderManualBlockInstructions()
    {
        return $this->display(__FILE__, 'views/templates/admin/manual-block-instructions.tpl');
    }

    public function renderTreeJS()
    {
        if ($this->psv > 1.6) {
            return;
        }

        $admin_webpath = str_ireplace(_PS_CORE_DIR_, '', _PS_ADMIN_DIR_);
        $admin_webpath = preg_replace('/^' . preg_quote(DIRECTORY_SEPARATOR, '/') . '/', '', $admin_webpath);
        $bo_theme = ((Validate::isLoadedObject($this->context->employee)
            && $this->context->employee->bo_theme) ? $this->context->employee->bo_theme : 'default');

        if (!file_exists(_PS_BO_ALL_THEMES_DIR_ . $bo_theme . DIRECTORY_SEPARATOR . 'template')) {
            $bo_theme = 'default';
        }

        $js_path = __PS_BASE_URI__ . $admin_webpath . '/themes/' . $bo_theme . '/js/tree.js?v=' . _PS_VERSION_;

        $this->context->controller->addJs($js_path);
    }

    public function displayAjaxError($message)
    {
        exit(json_encode([
            'error' => $message,
        ]));
    }

    public function isContentSizeValid($content, $size)
    {
        if (iconv_strlen($content) > $size) {
            return false;
        }

        return true;
    }

    public function saveCookieType()
    {
        $languages = Language::getLanguages(false);
        $id_type = (int) Tools::getValue('id_type');
        $type = new HiCookieType($id_type);

        $type->disabled = (int) Tools::getValue('disabled');
        $type->default_status = (int) Tools::getValue('default_status');
        $type->script = Tools::getValue('customScript');

        if (!$id_type) {
            $type->position = HiCookieType::getPosition();
        }

        $id_default_lang = Configuration::get('PS_LANG_DEFAULT');
        foreach ($languages as $lang) {
            $name = Tools::getValue('name_' . $lang['id_lang']);
            if (!$name) {
                $name = Tools::getValue('name_' . $id_default_lang);
            }
            if (!$name) {
                $this->displayAjaxError($this->l('Cookie Name is required'));
            } elseif (!$this->isContentSizeValid($name, 255)) {
                $this->displayAjaxError($this->l('Cookie Name size is not valid, you can use max 255 characters'));
            }

            $description = Tools::getValue('description_' . $lang['id_lang']);
            if (!$description) {
                $description = Tools::getValue('description_' . $id_default_lang);
            }

            $type->name[$lang['id_lang']] = $name;
            $type->description[$lang['id_lang']] = $description ? $description : '';
        }

        if (!$type->save()) {
            $this->displayAjaxError($this->l('We couldn\'t save the Cookie Type, please refresh the page and try again'));
        }

        if (!$id_type) {
            $id_type = $type->id;
        }

        // save blocked modules
        $type->deleteModules();
        $modules = Tools::getValue('cookieDisabledModules');

        if (is_array($modules) && $modules) {
            foreach ($modules as $module) {
                $module = explode('-', $module);
                if (empty($module[1])) {
                    continue;
                }

                $type->blockModule($module[1]);
            }
        }

        $this->assignCookieTypeToShops($id_type);

        return $id_type;
    }

    public function getModuleNameById($idModule)
    {
        return Db::getInstance()->getValue('SELECT `name` FROM `' . _DB_PREFIX_ . 'module` WHERE id_module = ' . (int) $idModule);
    }

    public function assignCookieTypeToShops($id_type)
    {
        $shop_ids = [];

        if (Shop::isFeatureActive()) {
            $shop_group = Tools::getValue('checkBoxShopGroupAsso_hicookietype');
            if (is_array($shop_group) && $shop_group) {
                foreach ($shop_group as $shops) {
                    foreach (ShopGroup::getShopsFromGroup($shops) as $shop) {
                        $shop_ids[] = $shop['id_shop'];
                    }
                }
            }
            $shops = Tools::getValue('checkBoxShopAsso_hicookietype');
            if (is_array($shops) && $shops) {
                foreach ($shops as $id) {
                    if (!in_array($id, $shop_ids)) {
                        $shop_ids[] = $id;
                    }
                }
            }

            // The Cookie Type should be assigned at least to 1 shop.
            if (!$shop_ids) {
                $shop_ids[] = $this->context->shop->id;
            }
        } else {
            $shop_ids[] = $this->context->shop->id;
        }

        Db::getInstance()->execute('DELETE FROM `' . _DB_PREFIX_ . 'hicookietype_shop` WHERE id_type = ' . (int) $id_type);
        if (is_array($shop_ids) && $shop_ids) {
            foreach ($shop_ids as $id) {
                Db::getInstance()->insert('hicookietype_shop', [
                    'id_type' => (int) $id_type,
                    'id_shop' => (int) $id,
                ]);
            }
        }
    }

    public function saveCookieTypeModules()
    {
        $id_type = (int) Tools::getValue('id_type');

        if (!$id_type) {
            $this->displayAjaxError($this->l('Somethine went wrong, please refresh the page and try again'));
        }

        $type = new HiCookieType($id_type);
        $type->deleteModules();

        $modules = Tools::getValue('hicookie_block_module');

        if (is_array($modules) && $modules) {
            foreach ($modules as $id_module) {
                $type->blockModule($id_module);
            }
        }

        return true;
    }

    public function renderWidget($hookName, array $params)
    {
        $this->smarty->assign($this->getWidgetVariables($hookName, $params));

        return $this->fetch($this->settingsTemplateFile);
    }

    public function getWidgetVariables($hookName, array $params)
    {
        return [
            'hookName' => $hookName,
        ];
    }
}
