/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
$(function() {
    $('#form-hicookietype').hiPrestaTable({
        friendlyName: 'CookieType',
        secureKey: hicookie_secure_key,
        ajaxUrl: hicookie_admin_controller,
        identifier: 'id_type',
        onFormDisplay: function() {
            function formatState (state) {
                if (!state.id) {
                  return state.text;
                }
                var baseUrl = "/modules";
                var $state = $(
                  '<span class="hi-cookie-module-img"><img src="' + (baseDir != '/' ? baseDir : '') + baseUrl + '/' + state.element.value.split('-')[0] + '/logo.png"> ' + state.text + '</span>'
                );
                return $state;
            };
            $('#cookieDisabledModules').select2({
                multiple: true,
                theme: 'bootstrap',
                width: '90%',
                placeholder: select2Placeholder,
                templateResult: formatState
            });

            setTimeout(function() {
                var editor = CodeMirror.fromTextArea(document.getElementById("customScript"), {
                    lineNumbers: true,
                    mode: 'javascript'
                });
            }, 500);
        },
        beforeFormSubmit: function () {
            var editor = document.querySelector('.CodeMirror').CodeMirror;
            editor.save();
        }
    });

    $(document)
        .on('click', '#desc-hicookietype-help', function(e) {
            e.preventDefault();
            e.stopPropagation();

            $('#hmd-modal .hmd-item').hide();
            $('#hmd-modal .hmd-item[data-doc="cookieTypesHelp"]').show();
            $('#hmd-modal').addClass('hmd-sidebar-open');
        })
});

hicookie = {
    updateTypeDisabledStatus: function(id_type, $sel) {
        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data: {
                ajax: true,
                action: 'updateTypeDisabledStatus',
                secure_key: hicookie_secure_key,
                id_type: id_type
            },
            beforeSend: function() {
                $sel.find('i').removeClass('icon-check').addClass('icon-refresh icon-spin');
            },
            success: function(response) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-check');

                if (response.error) {
                    showErrorMessage(response.error);
                } else {
                    showSuccessMessage(response.message);

                    $('#form-hicookietype').replaceWith(response.content);

                    // hicookie.initSort();
                }
            },
            error: function(jqXHR, error, errorThrown) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-check');

                if (jqXHR.status && jqXHR.status == 400) {
                    showErrorMessage(jqXHR.responseText);
                } else {
                    showErrorMessage(ajaxErrorMessage);
                }
            }
        });
    },

    updateTypeDefaultStatus: function(id_type, $sel) {
        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data: {
                ajax: true,
                action: 'updateTypeDefaultStatus',
                secure_key: hicookie_secure_key,
                id_type: id_type
            },
            beforeSend: function() {
                $sel.find('i').removeClass('icon-check').addClass('icon-refresh icon-spin');
            },
            success: function(response) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-check');

                if (response.error) {
                    showErrorMessage(response.error);
                } else {
                    showSuccessMessage(response.message);

                    $('#form-hicookietype').replaceWith(response.content);

                    // hicookie.initSort();
                }
            },
            error: function(jqXHR, error, errorThrown) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-check');

                if (jqXHR.status && jqXHR.status == 400) {
                    showErrorMessage(jqXHR.responseText);
                } else {
                    showErrorMessage(ajaxErrorMessage);
                }
            }
        });
    },

    displayBlockModulesForm: function(id_type, $sel) {
        $.ajax({
            type: 'POST',
            dataType: 'json',
            url: hicookie_admin_controller,
            data:{
                ajax : true,
                action : "renderBlockModules",
                secure_key : hicookie_secure_key,
                id_type : id_type
            },
            beforeSend: function(){
                $sel.find('i').removeClass('icon-puzzle-piece').addClass('icon-refresh icon-spin');
            },
            success: function(response){
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-puzzle-piece');

                if (typeof response.error !== 'undefined' && response.error) {
                    showErrorMessage(response.error);
                } else {
                    $("#hi-presta-module-modal .content").html(response.content);
                    $('#hi-presta-module-modal').modal('show');
                }
            },
            error: function(jqXHR, error, errorThrown) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-puzzle-piece');

                if (jqXHR.status && jqXHR.status == 400) {
                    showErrorMessage(jqXHR.responseText);
                } else {
                    showErrorMessage(ajaxErrorMessage);
                }
            }
        });
    },

    saveModuleList: function() {
        var $button = $('[name="submit_modules_list"]');
        var formdata = new FormData($('#hi-cookie-modules-form')[0]);
        formdata.append('action', 'saveCookieModules');
        formdata.append('secure_key', hicookie_secure_key);
        formdata.append('ajax', true);

        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data: formdata,
            contentType: false,
            processData: false,
            beforeSend: function(){
                $button.find('i.process-icon-save').removeClass('process-icon-save').addClass('process-icon-refresh icon-spin');
            },
            success: function(response) {
                $button.find('i.process-icon-refresh').removeClass('process-icon-refresh icon-spin').addClass('process-icon-save');
                if (response.error) {
                    showErrorMessage(response.error);
                } else {
                    showSuccessMessage(response.message);

                    $('#form-hicookietype').replaceWith(response.content);
                    $('#hi-presta-module-modal').modal('hide');

                    // hicookie.initSort();
                }
            },
            error: function(jqXHR, error, errorThrown) {
                $button.find('i.process-icon-refresh').removeClass('process-icon-refresh icon-spin').addClass('process-icon-save');

                if (jqXHR.status && jqXHR.status == 400) {
                    showErrorMessage(jqXHR.responseText);
                } else {
                    showErrorMessage(ajaxErrorMessage);
                }
            }
        });
    },

    deleteBannerLogo: function($sel) {
        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data: {
                ajax: true,
                action: 'deleteBannerLogo',
                secure_key: hicookie_secure_key
            },
            beforeSend: function() {
                $sel.find('i').removeClass('icon-trash').addClass('icon-refresh icon-spin');
            },
            success: function(response) {
                if (response.error) {
                    $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-trash');

                    showErrorMessage(response.error);
                } else {
                    showSuccessMessage(response.message);

                    $sel.closest('.help-block').remove();
                }
            },
            error: function(jqXHR, error, errorThrown) {
                $sel.find('i').removeClass('icon-refresh icon-spin').addClass('icon-trash');

                if (jqXHR.status && jqXHR.status == 400) {
                    showErrorMessage(jqXHR.responseText);
                } else {
                    showErrorMessage(ajaxErrorMessage);
                }
            }
        });
    },
    
    hideMenuModule: function() {
        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data: {
                ajax: true,
                action: 'hideMenuModule',
                secure_key: hicookie_secure_key
            },
            beforeSend: function() {
                $('.hipresta-modules-menu-ad').remove();
            },
            success: function(response) {},
            error: function(jqXHR, error, errorThrown) {}
        });
    }
};

$(function() {
    $('#init_editor').closest('form').hide();

    $('.hi-cookie-add-remote-ip').on('click', function() {
        var ip_address = $(this).attr('data-ip-address');
        if ($('#whitelist_ips').attr('value').length) {
            if ($('#whitelist_ips').attr('value').indexOf(ip_address) < 0) {
                $('#whitelist_ips').attr('value', $('#whitelist_ips').attr('value') + ',' + ip_address);
            }
        } else {
            $('#whitelist_ips').attr('value', ip_address);
        }

        return false;
    });

    if ($('#custom_css').length > 0) {
        var editor = CodeMirror.fromTextArea(document.getElementById("custom_css"), {
            lineNumbers: true,
            mode: 'css'
        });

        $(".hi-color-picker").spectrum({
            type: "component",
            preferredFormat: "hex"
        });
    }

    $(document).on('click', '#desc-hicookieconsent-delete', function(e) {
        e.preventDefault();
        var $this = $(this);
        $.ajax({
            type: 'POST',
            dataType: 'JSON',
            url: hicookie_admin_controller,
            data:{
                ajax : true,
                action : 'clearLogs',
                secure_key : hicookie_secure_key,
            },
            beforeSend: function() {
                $this.find('i').removeClass('process-icon-delete').addClass('process-icon-refresh icon-spin');
            },
            success: function(response) {
                $this.find('i').removeClass('process-icon-refresh icon-spin').addClass('process-icon-delete');

                var form = $this.closest('form').attr('id');
                $('#'+form).replaceWith(response.content);
            }
        });
    });

    $(document).on('click', '.hicookietype .hi-cookie-disabled-status', function() {
        var id_type = $(this).attr('data-id');
        hicookie.updateTypeDisabledStatus(id_type, $(this));

        return false;
    });

    $(document).on('click', '.hicookietype .hi-cookie-default-status', function() {
        var id_type = $(this).attr('data-id');
        hicookie.updateTypeDefaultStatus(id_type, $(this));

        return false;
    });

    $(document).on('click', '.add-type-modules', function(e) {
        var id_type = $(this).attr('data-id-type');
        hicookie.displayBlockModulesForm(id_type, $(this));

        return false;
    });

    $(document).on('click', '[name="submit_modules_list"]', function(e){
        hicookie.saveModuleList();

        return false;
    });

    $('#cookie_countries').select2({
        multiple: true,
        theme: 'bootstrap',
        width: '90%',
        placeholder: select2Placeholder
    });

    hiDropZone.init();

    $(document)
        .on('change', '[name="displayShopLogo"]', function() {
            if ($('[name="displayShopLogo"]:checked').val() == 1) {
                $('.hi-cookie-banner-logo').addClass('hm-hide');
            } else {
                $('.hi-cookie-banner-logo').removeClass('hm-hide');
            }
        })
        .on('change', '[name="cookie_settings"]', function() {
            if ($('[name="cookie_settings"]:checked').val() == 1) {
                $('.cookie-settings-position').removeClass('hm-hide');
            } else {
                $('.cookie-settings-position').addClass('hm-hide');
            }
        })
        .on('click', '.hi-delete-banner-logo', function(e) {
            e.preventDefault();

            hicookie.deleteBannerLogo($(this));
        })
        .on('click', '.hide-hm-menu-module', function(e) {
            e.preventDefault();

            hicookie.hideMenuModule();
        });
});