<?php
/**
 * advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.3
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCMS extends Module
{
    public $vt = 't17';
    private static $queries = array(
        'ad_cms' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms` (
            `id_ad_cms` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `active` tinyint(1) NOT NULL DEFAULT \'0\',
            `embedding` tinyint(1) NOT NULL DEFAULT \'1\',
            `column_left` tinyint(1) NOT NULL DEFAULT \'0\',
            `column_right` tinyint(1) NOT NULL DEFAULT \'0\',
            `display_header` tinyint(1) NOT NULL DEFAULT \'1\',
            `date_add` datetime NOT NULL,
            `date_upd` datetime NOT NULL,
            PRIMARY KEY (`id_ad_cms`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8',

        'ad_cms_lang' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms_lang` (
            `id_ad_cms` int(10) unsigned NOT NULL,
            `id_lang` int(10) unsigned NOT NULL,
            `name` varchar(128) NOT NULL,
            `link_rewrite` varchar(128) NOT NULL,
            `meta_description` varchar(255) DEFAULT NULL,
            `meta_keywords` varchar(255) DEFAULT NULL,
            `meta_title` varchar(128) DEFAULT NULL,
            PRIMARY KEY (`id_ad_cms`, `id_lang`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8',

        'ad_cms_container' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms_container` (
            `id_ad_cms_container` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `id_ad_cms` int(10) unsigned NOT NULL,
            `css_class` varchar(255) DEFAULT NULL,
            `css_class_w` varchar(255) DEFAULT NULL,
            `position` int(10) unsigned NOT NULL,
            `wrapper` tinyint(1) NOT NULL DEFAULT \'0\',
            `divider` tinyint(1) NOT NULL DEFAULT \'0\',
            `active` tinyint(1) NOT NULL DEFAULT \'0\',
            `date_add` datetime NOT NULL,
            `date_upd` datetime NOT NULL,
            PRIMARY KEY (`id_ad_cms_container`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8',

        'ad_cms_block' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms_block` (
            `id_ad_cms_block` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `id_ad_cms_container` int(10) unsigned NOT NULL,
            `type` varchar(255) DEFAULT NULL,
            `controller` varchar(255) DEFAULT NULL,
            `reference` varchar(255) DEFAULT NULL,
            `css_class` varchar(255) DEFAULT NULL,
            `settings` text DEFAULT NULL,
            `position` int(10) unsigned NOT NULL,
            `active` tinyint(1) NOT NULL DEFAULT \'0\',
            `date_add` datetime NOT NULL,
            `date_upd` datetime NOT NULL,
            PRIMARY KEY (`id_ad_cms_block`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8',

        'ad_cms_block_lang' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms_block_lang` (
            `id_ad_cms_block` int(10) unsigned NOT NULL,
            `id_lang` int(10) unsigned NOT NULL,
            `title` text DEFAULT NULL,
            `contents` text DEFAULT NULL,
            PRIMARY KEY (`id_ad_cms_block`, `id_lang`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8',

        'ad_cms_shop' => '
            CREATE TABLE IF NOT EXISTS `%PREFIX%ad_cms_shop` (
            `id_ad_cms` int(10) NOT NULL,
            `id_shop` int(10) NOT NULL,
            PRIMARY KEY (`id_ad_cms`,`id_shop`),
            KEY `id_shop` (`id_shop`)
            ) ENGINE=%ENGINE% DEFAULT CHARSET=utf8'
        );

    private $content_shortcuts;

    public function __construct()
    {
        $this->name = 'advancedcms';
        $this->version = '1.2.3';
        $this->tab = 'content_management';
        $this->author = 'Silbersaiten';
        $this->bootstrap = true;
        $this->module_key = '1b9672cb5f1b955270c61a2274ba3a85';
        $this->controllers = array('adcms');

        if (version_compare('1.7.0.0', _PS_VERSION_, '>')) {
            $this->vt = 't16';
        }
        parent::__construct();

        $this->displayName = $this->l('Advanced CMS');
        $this->description = $this->l('Flexible Content Management for Landing Pages');

        $this->setContentShortcuts();

        require_once(dirname(__FILE__).'/classes/AdvancedCMSPage.php');
        require_once(dirname(__FILE__).'/classes/AdvancedCMSContainer.php');
        require_once(dirname(__FILE__).'/classes/AdvancedCMSBlock.php');
        require_once(dirname(__FILE__).'/classes/AdvancedCmsBlockModel.php');
    }

    public function install($delete_params = true)
    {
        if (parent::install()
            && $this->registerHook('actionHtaccessCreate')
            && $this->registerHook('displayHeader')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayAdvancedCmsMenu')
            && $this->registerHook('actionAdvancedCmsAdminJs')
            && $this->registerHook('actionAdvancedCmsAdminCss')
            && $this->registerHook('actionAdminAdCmsBlockValidate')
            && $this->registerHook('moduleRoutes')
            && $this->registerHook('displayAdminForm')) {
            if ($delete_params) {
                foreach (self::$queries as $query) {
                    $query = strtr($query, array('%PREFIX%' => _DB_PREFIX_, '%ENGINE%' => _MYSQL_ENGINE_));

                    if (!Db::getInstance()->Execute($query)) {
                        $this->uninstall();

                        return false;
                    }
                }
            }

            Configuration::updateValue('ADCMS_LANDING', 0);
            Configuration::updateValue('ADCMS_IMAGE_PATH', 0);

            $this->installTab('AdminAdvancedCMSSettings', 'Advanced CMS');

            return true;
        }

        return false;
    }

    public function uninstall($delete_params = true)
    {
        if (parent::uninstall()) {
            if ($delete_params) {
                foreach (array_keys(self::$queries) as $table) {
                    Db::getInstance()->Execute('DROP TABLE IF EXISTS `'._DB_PREFIX_.pSQL($table).'`');
                }

                Configuration::deleteByName('ADCMS_LANDING');
                Configuration::deleteByName('ADCMS_IMAGE_PATH');
            }

            $this->uninstallTab('AdminAdvancedCMSSettings');
            
            foreach (new DirectoryIterator($this->getLocalPath().'views/css/pages/') as $f) {
                if (!$f->isDot() && !$f->getFilename() == 'index.php') {
                    unlink($f->getPathname());
                }
            }

            return true;
        }

        return false;
    }

    public function reset()
    {
        if (!$this->uninstall(false)) {
            return false;
        }
        if (!$this->install(false)) {
            return false;
        }

        return true;
    }

    public function installTab($tab_class, $tab_name, $parent = 'AdminParentPreferences')
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = $tab_class;
        $tab->name = array();

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tab_name;
        }

        $tab->id_parent = (int)Tab::getIdFromClassName($parent);
        $tab->module = $this->name;

        return $tab->add();
    }

    public function uninstallTab($tab_class)
    {
        $id_tab = (int)Tab::getIdFromClassName($tab_class);

        if ($id_tab) {
            $tab = new Tab($id_tab);

            return $tab->delete();
        }

        return false;
    }

    private function setContentShortcuts()
    {
        $this->content_shortcuts = array(
            'cms' => array(
                'context_class' => 'link',
                'method' => 'getCMSLink',
                'requires_id' => true,
                'description' => $this->l('Returns CMS link by a given CMS id')
            ),
            'product' => array(
                'context_class' => 'link',
                'method' => 'getProductLink',
                'requires_id' => true,
                'description' => $this->l('Returns Product link by a given Product id')
            ),
            'category' => array(
                'context_class' => 'link',
                'method' => 'getCategoryLink',
                'requires_id' => true,
                'description' => $this->l('Returns Category link by a given Category id')
            ),
            'supplier' => array(
                'context_class' => 'link',
                'method' => 'getSupplierLink',
                'requires_id' => true,
                'description' => $this->l('Returns Supplier link by a given Supplier id')
            ),
            'manufacturer' => array(
                'context_class' => 'link',
                'method' => 'getManufacturerLink',
                'requires_id' => true,
                'description' => $this->l('Returns Manufacturer link by a given Manufacturer id')
            ),
            'siteroot' => array(
                'class' => false,
                'method' => false,
                'requires_id' => false,
                'value' => __PS_BASE_URI__,
                'description' => $this->l('Returns your page root')
            ),
            'theme' => array(
                'class' => false,
                'method' => false,
                'requires_id' => false,
                'value' => _THEME_DIR_,
                'description' => $this->l('Returns a relative path to your current theme directory')
            ),
            'theme_img' => array(
                'class' => false,
                'method' => false,
                'requires_id' => false,
                'value' => _THEME_IMG_DIR_,
                'description' => $this->l('Returns a relative path to your current theme\'s images directory')
            ),
            'modules' => array(
                'class' => false,
                'method' => false,
                'requires_id' => false,
                'value' => _MODULE_DIR_,
                'description' => $this->l('Returns a relative path to your modules directory')
            )
        );
    }

    public function getContentShortcuts()
    {
        return $this->content_shortcuts;
    }

    public function hookModuleRoutes($params)
    {
        unset($params);
        $context = Context::getContext();
        $id_lang = $context->language->id;
        $word = Configuration::get('ADCMS_URL_WORD', (int)$id_lang);
        return array(
            'module-advancedcms-adcms' => array(
                'controller' => 'adcms',
                'rule' => ($word == '' || $word == false ? 'cms' : $word).'/{id_cms}/{rewrite}',
                'keywords' => array(
                    'id_cms' => array('regexp' => '[0-9]+', 'param' => 'id_cms'),
                    'rewrite' => array('regexp' => '[_a-zA-Z0-9-\pL]*', 'param' => 'rewrite'),
                    'module' => array('regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'module'),
                    'controller' => array('regexp' => '[_a-zA-Z0-9_-]+', 'param' => 'controller'),
                ),
                'params' => array(
                    'fc' => 'module',
                    'module' => 'advancedcms',
                    'controller' => 'adcms',
                )
            )
        );
    }

    public function hookDisplayBackOfficeHeader($params)
    {
        unset($params);
        if ($this->context->controller instanceof AdminAdvancedCMSSettingsController) {
            $this->context->controller->addJquery();
            $this->context->controller->addJqueryPlugin('autocomplete');

            return '
                <script type="text/javascript">
                    var adcmsTrans = {
                        "Are you sure you want to delete this block?": "'.$this->l('Are you sure you want to delete this block?').'",
                        "Are you sure you want to delete this image?": "'.$this->l('Are you sure you want to delete this image?').'",
                        "Are you sure you want to delete this container? All it\'s blocks will also be deleted!": "'.$this->l('Are you sure you want to delete this container? All it\'s blocks will also be deleted!').'",
                        "Please add or select a container for this block first.": "'.$this->l('Please add or select a container for this block first.').'",
                        "Enter link": "'.$this->l('Enter link').'",
                    };
                    var PS_ALLOW_ACCENTED_CHARS_URL = '.(int)Configuration::get('PS_ALLOW_ACCENTED_CHARS_URL').';
                </script>';
        }
    }

    public function hookDisplayAdminForm($params)
    {
        unset($params);
        if ($this->context->controller instanceof AdminAdvancedCMSSettingsController) {
            $this->context->smarty->assign('shortcuts', $this->getContentShortcuts());

            return $this->display(__FILE__, 'cmsshortcuts.tpl');
        }
    }

    public function getTabLink($tab, $params = false)
    {
        $link = 'index.php?controller='.$tab.'&token='.Tools::getAdminTokenLite($tab, $this->context);

        if (is_array($params) && count($params)) {
            foreach ($params as $k => $v) {
                $link .= '&'.$k.'='.$v;
            }
        }

        return $link;
    }

    public function getContent()
    {
        Tools::redirectAdmin($this->getTabLink('AdminAdvancedCMSSettings'));
    }
    
    public function hookDisplayHeader($params)
    {
        $this->context->smarty->registerFilter('output', array($this, 'parseCmsTags'));
        if ($this->vt == 't16') {
            $this->context->controller->addCSS(_MODULE_DIR_.$this->name.'/views/css/cmsfront_base.css');
        } else {
            $this->context->controller->registerStylesheet(
                'modules-'.$this->name.'-front',
                'modules/'.$this->name.'/views/css/cmsfront_base.css',
                array('media' => 'all', 'priority' => 150)
            );
        }
        Media::addJsDef(array('GrowlAddToCart' => $this->l('added to cart')));
        AdvancedCmsBlockModel::loadManifestFiles($this->context, 'front');
    }
    
    public function parseCmsTags($tpl_output, Smarty_Internal_Template $template)
    {
        preg_match_all('/\<p\>\[\s?displayCms\sid\s?=\s?([0-9]*)\s?\]\<\/p\>/', $tpl_output, $m, PREG_SET_ORDER);
        preg_match_all('/\[\s?linkCms\sid\s?=\s?([0-9]*)\s?\]/', $tpl_output, $l, PREG_SET_ORDER);

        if (count($m)) {
            foreach ($m as $cms_call) {
                $id_cms = (int)$cms_call[1];
                
                $cms_tpl = self::displayCms($id_cms);
                $tpl_output = str_replace($cms_call[0], $cms_tpl, $tpl_output);
            }
        }

        if (count($l)) {
            foreach ($l as $cms_call) {
                $id_cms = (int)$cms_call[1];
                
                $link = self::getLink($id_cms);
                $tpl_output = str_replace($cms_call[0], $link, $tpl_output);
            }
        }
    
        return $tpl_output;
    }

    public static function getLink($id_cms)
    {
        $context = Context::getContext();
        $cms = new AdvancedCMSPage((int)$id_cms, $context->language->id);

        return $context->link->getModuleLink(
            'advancedcms',
            'adcms',
            array('id_cms' => $id_cms, 'rewrite' => $cms->link_rewrite)
        );
    }
    
    public static function displayCms($id_cms)
    {
        require_once(dirname(__FILE__).'/classes/AdvancedCMSPage.php');
        require_once(dirname(__FILE__).'/classes/AdvancedCMSContainer.php');
        require_once(dirname(__FILE__).'/classes/AdvancedCMSBlock.php');

        $vt = 't17';

        if (version_compare('1.7.0.0', _PS_VERSION_, '>')) {
            $vt = 't16';
        }
        
        $context = Context::getContext();
        $cms = new AdvancedCMSPage((int)$id_cms, $context->language->id);

        if (!Validate::isLoadedObject($cms) || !$cms->embedding) {
            return false;
        }

        $cms_data = $cms->getCmsData($context->language->id, true);
        
        if (!$cms_data) {
            return false;
        }

        if (Shop::isFeatureActive() && !$cms->isAssociatedToShop($context->shop->id)) {
            return false;
        }

        $context->smarty->assign(array(
            'verp'     => 'cms_embedded',
            'cms'      => $cms,
            'cms_data' => $cms_data,
            'landing'  => Configuration::get('ADCMS_LANDING', null, null, (int)$context->shop->id)
        ));
        
        return $context->smarty->fetch(dirname(__FILE__).'/views/templates/front/'.$vt.'/cms.tpl');
    }

    public function hookActionHtaccessCreate($params)
    {
        $path = _PS_ROOT_DIR_.'/.htaccess';
        
        if (file_exists($path)) {
            $content = Tools::file_get_contents($path);
            
            $domains = array();
            foreach (ShopUrl::getShopUrls() as $shop_url) {
                /** @var ShopUrl $shop_url */
                if (!isset($domains[$shop_url->domain])) {
                    $domains[$shop_url->domain] = array();
                }
    
                $domains[$shop_url->domain][] = array(
                    'physical' =>    $shop_url->physical_uri,
                    'virtual' =>    $shop_url->virtual_uri,
                    'id_shop' =>    $shop_url->id_shop
                );
    
                if ($shop_url->domain == $shop_url->domain_ssl) {
                    continue;
                }
    
                if (!isset($domains[$shop_url->domain_ssl])) {
                    $domains[$shop_url->domain_ssl] = array();
                }
    
                $domains[$shop_url->domain_ssl][] = array(
                    'physical' =>    $shop_url->physical_uri,
                    'virtual' =>    $shop_url->virtual_uri,
                    'id_shop' =>    $shop_url->id_shop
                );
            }
            
            if (!$write_fd = @fopen($path, 'w')) {
                return false;
            }
            
            $start = '# ~~advancedcms landing start~~';
            $end = '# ~~advancedcms landing end~~';
            $start_pos = Tools::strpos($content, $start);
            $end_pos = Tools::strrpos($content, $end);
            
            if ($start_pos !== false && $end_pos !== false) {
                $delete = Tools::substr($content, $start_pos, ($end_pos + Tools::strlen($end)) - $start_pos);
                
                $content = trim(str_replace($delete, '', $content), "\n");
            }
            
            $tmp = '';
            
            foreach ($domains as $domain => $list_uri) {
                $physicals = array();
                foreach ($list_uri as $uri) {
                    $id_landing_page = Configuration::get('ADCMS_LANDING', null, null, (int)$uri['id_shop']);
                    
                    if ($id_landing_page) {
                        $landing = new AdvancedCMSPage((int)$id_landing_page, $this->context->language->id);
                    } else {
                        $landing = false;
                    }
                    
                    $landing = Validate::isLoadedObject($landing) ? $landing : false;
                    
                    if ($landing) {
                        $tmp.= "# ~~advancedcms landing start~~\n";
                        $tmp.= "<IfModule mod_rewrite.c>\n";
                        $tmp.= "\t<IfModule mod_env.c>\n";
                        $tmp.= "\t\tSetEnv HTTP_MOD_REWRITE On\n";
                        $tmp.= "\t</IfModule>\n\n";
                        $tmp.= "\tOptions +FollowSymlinks -MultiViews\n";
                        $tmp.= "\tRewriteEngine on\n";
                        $tmp.= "\tRewriteRule . - [E=REWRITEBASE:".$uri['physical']."]\n";
                        $tmp.= "\tRewriteCond %{REQUEST_URI} ".$uri['physical']."$\n";
                        $tmp.= "\tRewriteCond %{QUERY_STRING} !(^|&)rand=\n"; // to work correctly "Add to Cart" button (ajax)
                        $tmp.= "\tRewriteCond %{QUERY_STRING} !(^|&)mylogout=\n"; // to work correctly logout link (ajax)
                        $tmp.= "\tRewriteRule ^.*$ %{ENV:REWRITEBASE}index.php?fc=module&module=".$this->name."&controller=adcms&id_cms=".(int)$landing->id."&rewrite=".$landing->link_rewrite." [L]\n";
                        $tmp.= "</IfModule>\n";
                        $tmp.= "# ~~advancedcms landing end~~\n\n";
                    }
                }
            }
            
            $content = $tmp.$content;
            
            fwrite($write_fd, $content);
            
            fclose($write_fd);
        }
    }
}
