<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCMSBlock extends ObjectModel
{
    public $id_ad_cms_container;
    public $type;
    public $title;
    public $controller;
    public $reference;
    public $css_class;
    public $settings;
    public $position;
    public $contents;
    public $active;
    public $date_add;
    public $date_upd;
    private static $native_block_types = array('text', 'html', 'image', 'pslider', 'cslider', 'mslider', 'product');

    public static $definition = array(
        'table'     => 'ad_cms_block',
        'primary'   => 'id_ad_cms_block',
        'multilang' => true,
        'fields'    => array(
            'id_ad_cms_container' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'type'                => array('type' => self::TYPE_STRING, 'validate' => 'isGenericName', 'required' => true),
            'controller'          => array('type' => self::TYPE_STRING, 'validate' => 'isFileName'),
            'title'               => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName'),
            'reference'           => array('type' => self::TYPE_STRING, 'validate' => 'isCatalogName'),
            'css_class'           => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'settings'            => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'position'            => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId'),
            'active'              => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'date_add'            => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
            'date_upd'            => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
            'contents'            => array('type' => self::TYPE_NOTHING, 'lang' => true, 'validate' => 'isAnything'),
        )
    );

    public static function getNativeBlockTypes()
    {
        return self::$native_block_types;
    }

    public static function isNativeBlock($type)
    {
        return in_array($type, self::$native_block_types);
    }

    public static function getBlock($id_cms, $id_block, $id_lang = null)
    {
        $i = Db::getInstance()->getValue(
            'SELECT
                ib.`id_ad_cms_block`
            FROM
                `'._DB_PREFIX_.'ad_cms_block` ib
            LEFT JOIN
                `'._DB_PREFIX_.'ad_cms_container` ic ON (ib.`id_ad_cms_container` = ic.`id_ad_cms_container`)
            WHERE
                ib.`id_ad_cms_block` = '.(int)$id_block.' AND ic.`id_ad_cms` = '.(int)$id_cms
        );

        if ($i) {
            $block = new AdvancedCMSBlock((int)$i, Validate::isUnsignedId($id_lang) ? $id_lang : null);

            return Validate::isLoadedObject($block) ? $block : false;
        }

        return false;
    }

    public static function getBlocksFromContainer($id_ad_cms_container, $active = false, $id_lang = null)
    {
        if (!Validate::isUnsignedId($id_ad_cms_container)) {
            return false;
        }

        if (Validate::isUnsignedId($id_lang)) {
            return Db::getInstance()->ExecuteS(
                'SELECT
                    b.*,
                    bl.*
                FROM
                    `'._DB_PREFIX_.'ad_cms_block` b
                LEFT JOIN
                    `'._DB_PREFIX_.'ad_cms_block_lang` bl
                    ON (b.`id_ad_cms_block` = bl.`id_ad_cms_block` AND bl.`id_lang` = '.(int)$id_lang.')
                WHERE
                    b.`id_ad_cms_container` = '.(int)$id_ad_cms_container.'
                    '.($active ? ' AND b.`active` = 1' : '').'
                ORDER BY
                    b.`position` ASC'
            );
        } else {
            $blocks = Db::getInstance()->ExecuteS(
                'SELECT
                    *
                FROM
                    `'._DB_PREFIX_.'ad_cms_block`
                WHERE
                    `id_ad_cms_container` = '.(int)$id_ad_cms_container
            );

            if ($blocks) {
                foreach ($blocks as &$block) {
                    $lang_data = Db::getInstance()->ExecuteS(
                        'SELECT * FROM
                            `'._DB_PREFIX_.'ad_cms_block_lang`
                        WHERE
                            `id_ad_cms_block` = '.(int)$block['id_ad_cms_block']
                    );

                    if ($lang_data) {
                        foreach ($lang_data as $block_lang) {
                            if (!array_key_exists('contents', $block)) {
                                $block['contents'] = array();
                            }
                            
                            if (!array_key_exists('title', $block)) {
                                $block['title'] = array();
                            }

                            $block['contents'][$block_lang['id_lang']] = $block_lang['contents'];
                            $block['title'][$block_lang['id_lang']] = $block_lang['title'];
                        }
                    }
                }

                return $blocks;
            }
        }

        return false;
    }
}
