<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCMSPage extends ObjectModel
{
    public $name;
    public $link_rewrite;
    public $description;
    public $meta_description;
    public $meta_keywords;
    public $meta_title;
    public $display_header = true;
    public $column_left;
    public $column_right;
    public $embedding = true;
    public $active;
    public $date_add;
    public $date_upd;
    public $associated_shops;
    private $module;

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        parent::__construct($id, $id_lang, $id_shop);

        $this->module = Module::getInstanceByName('advancedcms');
    }
    
    public function getModule()
    {
        return $this->module;
    }

    public static $definition = array(
        'table'     => 'ad_cms',
        'primary'   => 'id_ad_cms',
        'multilang' => true,
        'fields'    => array(
            'embedding'        => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'active'           => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'date_add'         => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
            'date_upd'         => array('type' => self::TYPE_DATE, 'validate' => 'isDateFormat'),
            'display_header'   => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'column_left'      => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'column_right'     => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'meta_description' => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 255),
            'meta_keywords'    => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 255),
            'meta_title'       => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isGenericName', 'size' => 128),
            'name'             => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isCatalogName', 'required' => true, 'size' => 128),
            'link_rewrite'     => array('type' => self::TYPE_STRING, 'lang' => true, 'validate' => 'isLinkRewrite', 'required' => true, 'size' => 128)
        )
    );
    
    public static function getCmsPages($active = true, $id_lang = false, $id_shop = false)
    {
        $context = Context::getContext();
        
        if (!$id_lang) {
            $id_lang = $context->language->id;
        }
        
        $r = Db::getInstance()->ExecuteS(
            'SELECT
                c.`id_ad_cms`,
                cl.`name`,
                cl.`link_rewrite`
            FROM
                `'._DB_PREFIX_.'ad_cms` c
            LEFT JOIN
                `'._DB_PREFIX_.'ad_cms_lang` cl ON (c.`id_ad_cms` = cl.`id_ad_cms` AND cl.`id_lang` = '.(int)$id_lang.')'.
            ($id_shop && is_int($id_shop) ?
            'LEFT JOIN
                `'._DB_PREFIX_.'ad_cms_shop` cs ON (c.`id_ad_cms` = cs.`id_ad_cms` AND cs.`id_shop` = '.(int)$id_shop.')' : '').
            ($active ? ' WHERE c.`active` = 1' : '')
        );
        
        return $r;
    }

    public function add($autodate = true, $null_values = false)
    {
        if (parent::add($autodate, $null_values)) {
            $this->setCssContents();
            
            $form_data = Tools::getValue('cms_form_data');

            if (is_array($form_data)) {
                foreach ($form_data as $container) {
                    if (is_array($container['items']) && count($container['items'])
                        && !$this->containerContainsOnlyDeletedItems($container)) {
                        $container_obj = new AdvancedCMSContainer();
    
                        $container_obj->id_ad_cms = $this->id;
                        $container_obj->css_class = $container['css_class'];
                        $container_obj->css_class = isset($container['css_class_w']) ? (int)$container['css_class_w'] : '';
                        $container_obj->wrapper   = isset($container['wrapper']) ? (int)$container['wrapper'] : 0;
                        $container_obj->divider   = isset($container['divider']) ? (int)$container['divider'] : 0;
                        $container_obj->active    = (int)$container['active'];
                        $container_obj->position  = (int)$container['position'];
    
                        if ($container_obj->add()) {
                            $this->setContainerBlocks($container_obj->id, $container['items']);
                        }
                    }
                }
            }

            return true;
        }

        return false;
    }

    public function update($null_values = false)
    {
        if (parent::update($null_values)) {
            $this->setCssContents();
            
            $form_data = Tools::getValue('cms_form_data');

            if ($form_data) {
                foreach ($form_data as $container) {
                    if (is_array($container['items']) && count($container['items'])) {
                        $container_obj = new AdvancedCMSContainer(array_key_exists('id', $container) ? (int)$container['id'] : null);

                        if (isset($container['deleted']) && $container['deleted']
                            || $this->containerContainsOnlyDeletedItems($container)) {
                            if (Validate::isLoadedObject($container_obj)) {
                                $container_obj->delete();
                            }
                        } elseif (!array_key_exists('id', $container) || Validate::isLoadedObject($container_obj)) {
                            $container_obj->id_ad_cms   = $this->id;
                            $container_obj->css_class   = $container['css_class'];
                            $container_obj->css_class_w = array_key_exists('css_class_w', $container) ? $container['css_class_w'] : '';
                            $container_obj->wrapper     = array_key_exists('wrapper', $container) ? (int)$container['wrapper'] : 0;
                            $container_obj->divider     = array_key_exists('divider', $container) ? (int)$container['divider'] : 0;
                            $container_obj->active      = (int)$container['active'];
                            $container_obj->position    = (int)$container['position'];

                            if ($container_obj->save()) {
                                $this->setContainerBlocks($container_obj->id, $container['items']);
                            }
                        }
                    }
                }
            }

            $this->generateXMLSitemap();
            return true;
        }

        return false;
    }
    
    public function setCssContents()
    {
        $languages = Language::getLanguages();
        
        foreach ($languages as $language) {
            $css_content = Tools::getValue('css_content_'.$language['id_lang']);
            
            if ($css_content !== false) {
                $css_content = trim($css_content);
                $file_exists = $this->getCssFile($language['id_lang']);
                
                if ($file_exists) {
                    if (Tools::strlen($css_content) == 0) {
                        $this->removeCssFile($language['id_lang']);
                    } else {
                        $this->writeCssFile($language['id_lang'], $css_content);
                    }
                } elseif (Tools::strlen($css_content) > 0) {
                    $this->writeCssFile($language['id_lang'], $css_content);
                }
            }
        }
    }
    
    public function getCssContents()
    {
        $languages = Language::getLanguages();
        $result = array();
        
        foreach ($languages as $language) {
            $file_name = $this->getCssFileName($language['id_lang']);
            
            if (file_exists($this->module->getLocalPath().'views/css/pages/'.$file_name)) {
                $result[$language['id_lang']] = Tools::file_get_contents($this->module->getLocalPath().'views/css/pages/'.$file_name);
            } else {
                $result[$language['id_lang']] = '';
            }
        }
        
        return $result;
    }
    
    public function getCssFile($id_lang = false)
    {
        $context = Context::getContext();
        
        if (!$id_lang) {
            $id_lang = (int)$context->language->id;
        }
        
        $file_name = $this->getCssFileName($id_lang);
        
        if (file_exists($this->module->getLocalPath().'views/css/pages/'.$file_name)) {
            return $this->module->getPathUri().'views/css/pages/'.$file_name;
        }
        
        return false;
    }
    
    public function getCssFileName($id_lang)
    {
        return implode('_', array('cms', $this->id, $id_lang)).'.css';
    }
    
    public function removeCssFile($id_lang = false)
    {
        if (!$id_lang) {
            $languages = Language::getLanguages();
            
            foreach ($languages as $language) {
                if ($this->getCssFile($language['id_lang'])) {
                    $file_name = $this->getCssFileName($language['id_lang']);
                    
                    unlink($this->module->getLocalPath().'views/css/pages/'.$file_name);
                }
            }
        } else {
            if ($this->getCssFile($id_lang)) {
                $file_name = $this->getCssFileName($id_lang);
                
                unlink($this->module->getLocalPath().'views/css/pages/'.$file_name);
            }
        }
    }
    
    public function writeCssFile($id_lang, $content)
    {
        $path = $this->module->getLocalPath().'views/css/pages/'.$this->getCssFileName($id_lang);
        
        $f = fopen($path, 'w');
        
        if ($f) {
            fwrite($f, $content);
            
            fclose($f);
            
            return true;
        }
        
        return false;
    }
    
    public function duplicateObject()
    {
        $old_obj = $this;
        
        if ($new_object = parent::duplicateObject()) {
            $containers = AdvancedCMSContainer::getCmsContainers($old_obj->id);
            
            if ($containers) {
                foreach ($containers as $container) {
                    $container_blocks = AdvancedCMSBlock::getBlocksFromContainer($container['id_ad_cms_container']);
                    
                    if ($container_blocks) {
                        unset($container['id_ad_cms_container']);
                        
                        $container['id_ad_cms'] = $new_object->id;
                        $container['date_add'] = $new_object->date_add;
                        $container['date_upd'] = $new_object->date_upd;
                        
                        if (Db::getInstance()->insert('ad_cms_container', $container)) {
                            $id_container = Db::getInstance()->Insert_ID();
                            
                            foreach ($container_blocks as $container_block) {
                                $lang_content = $container_block['contents'];
                                $lang_title = isset($container_block['title']) ? $container_block['title'] : false;
                                unset($container_block['id_ad_cms_block']);
                                unset($container_block['contents']);
                                unset($container_block['title']);
                                
                                $container_block['id_ad_cms_container'] = $id_container;
                                $container_block['date_add'] = $new_object->date_add;
                                $container_block['date_upd'] = $new_object->date_upd;
                                
                                if (Db::getInstance()->insert('ad_cms_block', $container_block)) {
                                    $block_id = Db::getInstance()->Insert_ID();
                                    $lang_data = array();
                                    
                                    foreach ($lang_content as $id_lang => $contents) {
                                        if (!array_key_exists($id_lang, $lang_data)) {
                                            $lang_data[$id_lang] = array(
                                                'id_ad_cms_block' => (int)$block_id,
                                                'id_lang' => (int)$id_lang,
                                                'contents' => false,
                                                'title' => false
                                            );
                                        }

                                        $lang_data[$id_lang]['contents'] = pSQL($contents, true);
                                    }
                                    
                                    if ($lang_title) {
                                        foreach ($lang_title as $id_lang => $title) {
                                            if (array_key_exists($id_lang, $lang_data)) {
                                                $lang_data[$id_lang]['title'] = $title;
                                            }
                                        }
                                    }
                                    
                                    foreach ($lang_data as $insert_array) {
                                        Db::getInstance()->insert('ad_cms_block_lang', $insert_array);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $languages = Language::getLanguages();
            $path = $this->module->getLocalPath().'views/css/pages/';
        
            foreach ($languages as $language) {
                $old_file_name = $old_obj->getCssFileName($language['id_lang']);
                if (file_exists($path.$old_file_name)) {
                    $file_name = $new_object->getCssFileName($language['id_lang']);
                    copy($path.$old_file_name, $path.$file_name);
                }
            }
            
            return $new_object;
        }
        
        return false;
    }

    private function containerContainsOnlyDeletedItems($container)
    {
        if (is_array($container['items']) && count($container['items'])) {
            foreach ($container['items'] as $item) {
                if (!isset($item['deleted']) || !$item['deleted']) {
                    return false;
                }
            }
        }

        return true;
    }

    public function delete()
    {
        if (parent::delete()) {
            $containers = AdvancedCMSContainer::getCmsContainers($this->id);

            if ($containers) {
                foreach ($containers as $container) {
                    if (Validate::isLoadedObject($container_obj = new AdvancedCMSContainer((int)$container['id_ad_cms_container']))) {
                        $container_obj->delete();
                    }
                }
            }
                        
            $this->removeCssFile();

            return true;
        }

        return false;
    }

    private function setContainerBlocks($id_container, $blocks)
    {
        foreach ($blocks as $block) {
            $settings = (is_array($block['settings']) && count($block['settings'])) ? Tools::jsonEncode($block['settings']) : false;

            $block_obj = new AdvancedCMSBlock($block['id'] ? (int)$block['id'] : null);

            if (isset($block['deleted']) && $block['deleted']) {
                if (Validate::isLoadedObject($block_obj)) {
                    $block_obj->delete();
                }
            } elseif (!$block['id'] || Validate::isLoadedObject($block_obj)) {
                $block_obj->id_ad_cms_container = $id_container;
                $block_obj->position = (int)$block['position'];
                $block_obj->reference = $block['reference'];
                $block_obj->css_class = $block['css_class'];
                $block_obj->settings = $settings;
                $block_obj->active = (int)$block['active'];
                $block_obj->type = $block['type'];
                $block_obj->controller = isset($block['controller']) ? $block['controller'] : null;
                $block_obj->contents = $this->createLanguageContentField($block['content'], $block['type'], true);
                
                if (isset($block['name'])) {
                    $block_obj->title = $this->createLanguageContentField($block['name'], $block['type'], false);
                }

                $block_obj->save();
            }
        }
    }

    private function createLanguageContentField($block_language_data, $block_type, $psql)
    {
        $r = array();

        foreach ($block_language_data as $data) {
            if (isset($data['id_lang'])) {
                if (is_array($data['value'])) {
                    foreach ($data['value'] as &$block) {
                        if (is_array($block) && array_key_exists('name', $block)
                            && ($block_type == 'pslider' || $block_type == 'cslider' || $block_type == 'mslider' || $block_type == 'product')) {
                            if (!$psql) {
                                $block['name'] = pSQL($block['name']);
                            }
                        }
                    }
                    $v = Tools::jsonEncode($data['value']);
                } else {
                    $v = html_entity_decode($data['value']);
                }

                if ($psql) {
                    $v = pSQL($v, true);
                }

                $r[$data['id_lang']] = $v;
            }
        }

        return $r;
    }

    private static function validateIsJson($string)
    {
        if (!Tools::strlen($string)) {
            return false;
        }

        $decoded = Tools::jsonDecode($string, true);

        return (is_array($decoded) && count($decoded));
    }

    public function getCmsData($id_lang = null, $active = false)
    {
        $r = array();
        $i = 1;

        $containers = AdvancedCMSContainer::getCmsContainers($this->id, $active);
        $block_objects = AdvancedCmsBlockModel::getInstalledBlockInstancesFromPage($this);
        $block_index = 1;

        if ($containers) {
            foreach ($containers as $container) {
                $r[$i] = array(
                    'id'          => (int)$container['id_ad_cms_container'],
                    'active'      => (int)$container['active'],
                    'wrapper'     => (int)$container['wrapper'],
                    'divider'     => (int)$container['divider'],
                    'position'    => (int)$container['position'],
                    'deleted'     => 0,
                    'css_class'   => $container['css_class'],
                    'css_class_w' => $container['css_class_w'],
                    'items'       => array()
                );

                $blocks = AdvancedCMSBlock::getBlocksFromContainer(
                    $container['id_ad_cms_container'],
                    $active,
                    $id_lang
                );

                if ($blocks) {
                    foreach ($blocks as $block) {
                        $settings = self::validateIsJson($block['settings']) ? Tools::jsonDecode($block['settings'], true) : array();
                        $settings['id_block'] = (int)$block['id_ad_cms_block'];

                        if (!Validate::isUnsignedId($id_lang)) {
                            $lang_content = array();
                            $title = array();

                            if (isset($block['contents']) && is_array($block['contents'])) {
                                foreach ($block['contents'] as $language => $data) {
                                    $lang_content['lang_'.$language] = array(
                                        'id_lang' => $language,
                                        'value'   => self::validateIsJson($data) ? Tools::jsonDecode($data, true) : $data,
                                    );
                                }
                            }
                            
                            if (isset($block['title']) && is_array($block['title'])) {
                                foreach ($block['title'] as $language => $data) {
                                    $title['lang_'.$language] = array(
                                        'id_lang' => $language,
                                        'value'   => $data,
                                    );
                                }
                            }
                        } else {
                            $lang_content = self::validateIsJson($block['contents']) ? Tools::jsonDecode($block['contents'], true) : $block['contents'];
                            $title = $block['title'];

                            if (!is_array($lang_content)) {
                                $lang_content = $this->replaceLinkAnchors($lang_content);
                            }

                            if (Tools::strlen($block['controller']) > 0
                                && $block_objects
                                && isset($block_objects[$block['controller']])) {
                                if (method_exists($block_objects[$block['controller']], 'prefilterResults')) {
                                    $lang_content = $block_objects[$block['controller']]->{'prefilterResults'}($lang_content, $settings);
                                }
                                
                                if ($template = $block_objects[$block['controller']]->displayTemplate()) {
                                    $lang_content = $template;
                                }

                                if ($block['type'] == 'text') {
                                    $lang_content = nl2br($lang_content);
                                }
                            } else {
                                Hook::exec(
                                    'actionBlockDataPrefilter',
                                    array(
                                        'type' => $block['type'],
                                        'contents' => &$lang_content, 'settings' => $settings
                                    )
                                );
                            }
                        }

                        $r[$i]['items'][$block_index] = array(
                            'reference' => $block['reference'],
                            'active'    => (int)$block['active'],
                            'css_class' => $block['css_class'],
                            'id'        => (int)$block['id_ad_cms_block'],
                            'name'      => $title,
                            'position'  => (int)$block['position'],
                            'deleted'   => 0,
                            'content'   => $lang_content,
                            'settings'  => $settings,
                            'type'      => $block['type'],
                            'controller' => $block['controller']
                        );

                        $block_index++;
                    }
                }

                $i++;
            }
        }
        
        return count($r) ? $r : false;
    }

    public function replaceLinkAnchors($content)
    {
        preg_match_all('/\{([A-z_]*)([\s]*?\:[\s]*?([0-9]*))?\}/', $content, $m, PREG_SET_ORDER);

        $shortcuts = $this->module->getContentShortcuts();

        if ($shortcuts && is_array($m) && count($m)) {
            $context = Context::getContext();

            foreach ($m as $match) {
                if (array_key_exists($match[1], $shortcuts)) {
                    if ($shortcuts[$match[1]]['requires_id'] && count($match) != 4) {
                        continue;
                    }

                    if (count($match) == 4) {
                        $context_class = $shortcuts[$match[1]]['context_class'];
                        $method = $shortcuts[$match[1]]['method'];

                        if (property_exists($context, $context_class)
                            && method_exists($context->{$context_class}, $method)) {
                            $content = str_replace(
                                $match[0],
                                $context->{$context_class}->{$method}((int)$match[3]),
                                $content
                            );
                        }
                    } elseif (count($match) == 2 && isset($shortcuts[$match[1]]['value'])) {
                        $content = str_replace($match[0], $shortcuts[$match[1]]['value'], $content);
                    }
                }
            }
        }

        return $content;
    }
    
    public function getAssociatedShops()
    {
        $r = Db::getInstance()->ExecuteS(
            'SELECT `id_shop` FROM `'._DB_PREFIX_.'ad_cms_shop` WHERE `id_ad_cms` = '.(int)$this->id
        );
        $list = array();
        
        foreach ($r as $shop) {
            $list[] = (int)$shop['id_shop'];
        }
        
        return $list;
    }

    public function generateXMLSitemap($active_only = true)
    {
        $shops = Shop::getShops();
        $context = Context::getContext();

        foreach ($shops as $shop) {
            $languages = Language::getLanguages(true, (int)$shop['id_shop']);

            foreach ($languages as $lang) {
                $cms = self::getCmsPages($active_only, (int)$lang['id_lang'], (int)$shop['id_shop']);

                $xml = new SimpleXMLElement('<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:image="http://www.google.com/schemas/sitemap-image/1.1" />');
                $filename = 'advancedcms_map_'.$shop['id_shop'].'_'.$lang['iso_code'].'.xml';

                foreach ($cms as $cms_one) {
                    $loc = $context->link->getPageLink(
                        'module-'.$this->module->name.'-adcms',
                        null,
                        (int)$lang['id_lang'],
                        array('id_cms' => $cms_one['id_ad_cms'], 'rewrite' => $cms_one['link_rewrite'], 'module' => $this->module->name, 'controller' => 'adcms'),
                        false,
                        $shop['id_shop']
                    );

                    $priority = 0.5;
                    $change_freq = 'weekly';

                    $url = $xml->addChild('url');
                    $url->addChild('loc', $loc);
                    $url->addChild('priority', number_format($priority, 1, '.', ''));
                    $url->addChild('changefreq', $change_freq);
                }

                $file_path = dirname(__FILE__).'/../xml/'.$filename;

                $xml->asXML($file_path);
            }
        }
    }
    
    public static function backUpPages($active_only = false)
    {
        $cms = self::getCmsPages($active_only);
        $languages = Language::getLanguages(false);
        $xml = new SimpleXMLElement('<pages />');
        $filename = 'advancedcms_'.(date('Y.m.d')).'_'.uniqid().'.xml';
        
        if ($cms && $cms = self::getObjectCollection($cms, 'AdvancedCMSPage')) {
            foreach ($cms as $cms_obj) {
                $cms_obj->associated_shops = $cms_obj->getAssociatedShops();
                $containers = AdvancedCMSContainer::getCmsContainers($cms_obj->id, false);
                
                if ($containers) {
                    $xml_page = self::writeXmlObject($xml, 'page', $cms_obj, $languages);
                    
                    $containers = self::getObjectCollection($containers, 'AdvancedCMSContainer');
                    
                    if ($containers) {
                        $xml_containers = $xml_page->addChild('containers');
                        
                        foreach ($containers as $container) {
                            $xml_container = self::writeXmlObject(
                                $xml_containers,
                                'container',
                                $container,
                                $languages,
                                array('id_ad_cms')
                            );
                            
                            $blocks = AdvancedCMSBlock::getBlocksFromContainer($container->id, false);
                            
                            if ($blocks && $blocks = self::getObjectCollection($blocks, 'AdvancedCMSBlock')) {
                                $xml_container_blocks = $xml_container->addChild('blocks');
                                
                                foreach ($blocks as $block) {
                                    $xml_container_block = self::writeXmlObject(
                                        $xml_container_blocks,
                                        'block',
                                        $block,
                                        $languages,
                                        array('id_ad_cms_container')
                                    );
                                }
                            }
                        }
                    }
                }
            }
            
            $file_path = dirname(__FILE__).'/../backup/'.$filename;

            $xml->asXML($file_path);
            
            return file_exists($file_path) ? $filename : false;
        }
    }
    
    private static function writeXmlObject(&$parent, $container, $item, $languages, $exclude = false)
    {
        $xml_item = $parent->addChild($container);
        
        $definition = ObjectModel::getDefinition($item);
            
        $item_id = $xml_item->addChild($container.'_id', $item->id);
        $item_id->addAttribute('lang', 0);
        
        foreach ($definition['fields'] as $k => $v) {
            if (is_array($exclude) && in_array($k, $exclude)) {
                continue;
            }

            if (isset($item->{$k})) {
                if (is_array($item->{$k}) && isset($v['lang']) && $v['lang']) {
                    $page_prop = $xml_item->addChild($k);
                    
                    $page_prop->addAttribute('lang', 1);
                    
                    foreach ($item->{$k} as $id_lang => $lang_val) {
                        foreach ($languages as $language) {
                            if ($language['id_lang'] == $id_lang) {
                                $page_prop->addChild('lang', htmlspecialchars($lang_val))->addAttribute('iso', $language['iso_code']);
                            }
                        }
                    }
                } else {
                    $page_prop = $xml_item->addChild($k, htmlspecialchars($item->{$k}));
                    $page_prop->addAttribute('lang', 0);
                }
            }
        }
        
        if ($item instanceof AdvancedCMSPage) {
            $shops = $xml_item->addChild('shops');
            
            foreach ($item->associated_shops as $id_shop) {
                $shops->addChild('shop', (int)$id_shop);
            }
        }
        
        return $xml_item;
    }
    
    private static function getObjectCollection($data, $class)
    {
        if (!is_array($data) || !count($data)) {
            return false;
        }
        
        $definition = ObjectModel::getDefinition($class);
        
        if (!is_array($definition) || !isset($definition['primary'])) {
            return false;
        }
        
        $list = array();
        
        foreach ($data as $v) {
            if (isset($v[$definition['primary']]) && Validate::isLoadedObject($obj = new $class((int)$v[$definition['primary']]))) {
                $list[] = $obj;
            }
        }
        
        return count($list) ? $list : false;
    }
    
    public static function restoreFromBackup($xml_path, $remove)
    {
        $xml = simplexml_load_file($xml_path);
        $languages = Language::getLanguages(false);
        $added_entries = 0;
        $installed_blocks = AdvancedCmsBlockModel::getInstalledBlocks();
        
        if ($xml) {
            if ($remove) {
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms`');
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms_lang`');
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms_container`');
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms_block`');
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms_block_lang`');
                Db::getInstance()->Execute('TRUNCATE `'._DB_PREFIX_.'ad_cms_shop`');

                Configuration::updateValue('ADCMS_LANDING', 0);
            }

            foreach ($xml as $key => $page) {
                if ($key != 'page') {
                    continue;
                }

                if ($remove && property_exists($page, 'page_id')) {
                    $page_obj = new AdvancedCMSPage($page->page_id);
                } else {
                    $page_obj = new AdvancedCMSPage();
                }
                
                $containers = isset($page->children()->containers) ? $page->children()->containers : false;
                
                self::fillObject($page_obj, $page->children(), $languages, array('containers', 'shops'));
                
                $shops = isset($page->children()->shops) ? $page->children()->shops : false;
                $shop_list = array();
                
                if ($shops) {
                    foreach ($shops->children() as $id_shop) {
                        $shop_list[] = (int)$id_shop;
                    }
                    
                    $shop_list = array_unique($shop_list);
                }
                
                if ($page_obj->add()) {
                    if (Shop::isFeatureActive() && count($shop_list)) {
                        $existing_shops = Shop::getCompleteListOfShopsID();
                        
                        foreach ($shop_list as $id_shop) {
                            if (count($existing_shops) && in_array($id_shop, $existing_shops)) {
                                Db::getInstance()->insert('ad_cms_shop', array('id_ad_cms' => $page_obj->id, 'id_shop' => (int)$id_shop));
                            }
                        }
                    }
                    
                    foreach ($containers->children() as $container) {
                        if ($remove && property_exists($container, 'container_id')) {
                            $container_obj = new AdvancedCMSContainer($container->container_id);
                        } else {
                            $container_obj = new AdvancedCMSContainer();
                        }
                        
                        self::fillObject($container_obj, $container->children(), $languages, array('blocks'));
                        
                        $container_obj->id_ad_cms = (int)$page_obj->id;
                        
                        if ($container_obj->add()) {
                            $blocks = isset($container->children()->blocks) ? $container->children()->blocks : false;
                            
                            if ($blocks) {
                                foreach ($blocks->children() as $block) {
                                    if ($remove && property_exists($block, 'block_id')) {
                                        $block_obj = new AdvancedCMSBlock($block->block_id);
                                    } else {
                                        $block_obj = new AdvancedCMSBlock();
                                    }

                                    self::fillObject($block_obj, $block->children(), $languages);

                                    if (!$block_obj->controller && is_array($installed_blocks)) {
                                        foreach ($installed_blocks as $controller => $installed_block) {
                                            if ($block_obj->type == $installed_block->type) {
                                                $block_obj->controller = $controller;
                                                
                                                break;
                                            }
                                        }
                                    }
                                    
                                    $block_obj->id_ad_cms_container = $container_obj->id;
                                    
                                    $block_obj->add();
                                }
                            }
                        }
                    }
                    
                    $added_entries++;
                }
            }
            
            return $added_entries > 0;
        }
        
        return false;
    }
    
    private static function fillObject(&$obj, $xml_block, $languages, $exclude = false)
    {
        foreach ($xml_block as $k => $child) {
            if (is_array($exclude) && in_array((string)$k, $exclude)) {
                continue;
            }
            
            if (property_exists($obj, $k)) {
                if ((int)$child->attributes()->lang == 1) {
                    foreach ($child->children() as $lang) {
                        $value = htmlspecialchars_decode((string)$lang);
                        
                        $lang_iso = (string)$lang->attributes()->iso;
                        
                        if ($id_lang = self::getLanguageIdByIsoCode($languages, $lang_iso)) {
                            if (!is_array($obj->{$k})) {
                                $obj->{$k} = array();
                            }
                            
                            $obj->{$k}[$id_lang] = pSQL($value, true);
                        }
                    }
                } else {
                    $obj->{$k} = htmlspecialchars_decode((string)$child);
                }
            }
        }
    }
    
    private static function getLanguageIdByIsoCode($languages, $iso_code)
    {
        foreach ($languages as $language) {
            if ($language['iso_code'] == $iso_code) {
                return $language['id_lang'];
            }
        }
        
        return false;
    }
}
