<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

abstract class AdvancedCmsBlockModel
{
    public $type;
    public $name;
    public $manifest = array();
    public $vt = 't17';
    
    protected $context;
    protected $smarty;
    
    public $ad_cms_page; // only available when viewing in FO
    
    private static $module_instance;
    private static $blocks_path;
    
    protected static $loaded_modules;
    
    public function __construct($context = false)
    {
        if (!isset(self::$blocks_path)) {
            self::setBlocksPath();
        }
        
        $this->context = $context ? $context : Context::getContext();
        if (is_object($this->context->smarty)) {
            $this->smarty = $this->context->smarty->createData($this->context->smarty);
        }

        if (version_compare('1.7.0.0', _PS_VERSION_, '>')) {
            $this->vt = 't16';
        }
    }
    
    private static function setBlocksPath()
    {
        self::$blocks_path = dirname(__FILE__).'/blocks/';
    }
    
    public static function loadAvailableBlocks()
    {
        if (!isset(self::$blocks_path)) {
            self::setBlocksPath();
        }
            
        $blocks = scandir(self::$blocks_path);
        
        foreach ($blocks as $name) {
            if (is_file(self::$blocks_path.$name) && preg_match('/^AdvancedCmsBlock(.*).php/', $name)) {
                require_once(self::$blocks_path.$name);
                
                $class_name = Tools::substr($name, 0, -(Tools::strlen((pathinfo($name, PATHINFO_EXTENSION))) + 1));
                
                if (!class_exists($class_name)) {
                    throw new PrestaShopException(
                        Tools::displayError('Invalid advanced CMS block detected!').' - "'.$name.'"'
                    );
                } else {
                    if (!is_array(self::$loaded_modules)) {
                        self::$loaded_modules = array();
                    }
                    
                    self::$loaded_modules[$class_name] = new $class_name();
                }
            }
        }
    }
    
    public static function getInstalledBlocks()
    {
        if (!is_array(self::$loaded_modules)) {
            self::loadAvailableBlocks();
        }
        
        return self::$loaded_modules;
    }
    
    public static function getModuleInstance()
    {
        if (self::$module_instance) {
            return self::$module_instance;
        }
        
        self::$module_instance = Module::getInstanceByName('advancedcms');
        
        if (!self::$module_instance) {
            throw new PrestaShopException(Tools::displayError('Unable to obtain module\'s instance!'));
        }
        
        return self::getModuleInstance();
    }
    
    public function l($string, $specific = false)
    {
        return Translate::getModuleTranslation(self::getModuleInstance(), $string, get_class($this));
    }
    
    public function displayTemplate()
    {
        if ($this->ad_cms_page) {
            $module = $this->ad_cms_page->getModule();

            if (file_exists($module->getLocalPath().'views/templates/hook/blocks/'.$this->vt.'/'.get_class($this).'.tpl')) {
                return $module->display($module->getLocalPath(), 'blocks/'.$this->vt.'/'.get_class($this).'.tpl');
            }
        }
        
        return false;
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        
    }
    
    public function prefilterResults($language_content, $settings)
    {
        return $language_content;
    }
    
    public static function getInstalledBlockInstancesFromPage($cms)
    {
        $containers = AdvancedCMSContainer::getCmsContainers($cms->id, true);
        $controllers = array();
        $instances = array();
        $context = Context::getContext();

        if ($containers) {
            foreach ($containers as $container) {
                $blocks = AdvancedCMSBlock::getBlocksFromContainer(
                    $container['id_ad_cms_container'],
                    true,
                    $context->language->id
                );
                
                if ($blocks) {
                    foreach ($blocks as $block) {
                        if (Tools::strlen($block['controller']) > 0 && Validate::isFileName($block['controller'])) {
                            $controllers[] = $block['controller'];
                        }
                    }
                }
            }
        }
        
        if (count($controllers)) {
            if (!isset(self::$blocks_path)) {
                self::setBlocksPath();
            }
            
            $controllers = array_unique($controllers);
            
            foreach ($controllers as $controller) {
                if (file_exists(self::$blocks_path.$controller.'.php')) {
                    require_once(self::$blocks_path.$controller.'.php');
                    
                    if (class_exists($controller)) {
                        $instances[$controller] = new $controller();
                        
                        if (is_object($instances[$controller])) {
                            $instances[$controller]->ad_cms_page = $cms;
                        }
                    }
                }
            }
        }
        
        return count($instances) ? $instances : false;
    }
    
    public static function callBlockMethod($class, $method_name, $params = false)
    {
        if (!array_key_exists($class, self::$loaded_modules)
            || !method_exists(self::$loaded_modules[$class], $method_name)) {
            return false;
        }
        
        if (!$params) {
            $params = array();
        }
        
        return call_user_func_array(array(self::$loaded_modules[$class], $method_name), $params);
    }
    
    public static function loadManifestFiles(Context $context, $key, $blocks = false)
    {
        $module = self::getModuleInstance();
        
        if (!$blocks) {
            $blocks = self::getInstalledBlocks();
        }
        
        if (is_array($blocks) && count($blocks)) {
            $plugins_to_add = array();
            
            foreach ($blocks as $block) {
                if ($key == 'admin') {
                    if ($module->vt == 't16') {
                        $context->controller->addJS($module->getPathUri().'/views/js/blocks/'.(get_class($block)).'.js');
                    } else {
                        $context->controller->registerJavascript(
                            'modules-advancedcms-'.(get_class($block)),
                            'modules/'.$module->name.'/views/js/blocks/'.(get_class($block)).'.js',
                            array('position' => 'bottom', 'priority' => 150)
                        );
                    }
                }
                
                if (array_key_exists('js', $block->manifest) && array_key_exists($key, $block->manifest['js'])) {
                    foreach ($block->manifest['js'][$key] as $v) {
                        if ($module->vt == 't16') {
                            $context->controller->addJS($module->getPathUri().$v);
                        } else {
                            $context->controller->registerJavascript(
                                'modules-advancedcms-block-'.(get_class($block)),
                                'modules/'.$module->name.'/'.$v,
                                array('position' => 'bottom', 'priority' => 150)
                            );
                        }
                    }
                }
                
                if (array_key_exists('jquery_ui', $block->manifest)
                    && array_key_exists($key, $block->manifest['jquery_ui'])) {
                    $plugins_to_add = array_merge($plugins_to_add, $block->manifest['jquery_ui'][$key]);
                }
                
                if (array_key_exists('css', $block->manifest) && array_key_exists($key, $block->manifest['css'])) {
                    foreach ($block->manifest['css'][$key] as $v) {
                        if ($module->vt == 't16') {
                            $context->controller->addCSS($module->getPathUri().$v);
                        } else {
                            $context->controller->registerStylesheet(
                                'modules-advancedcms-block-'.(get_class($block)),
                                'modules/'.$module->name.'/'.$v,
                                array('media' => 'all', 'priority' => 150)
                            );
                        }
                    }
                }
            }
            
            if (count($plugins_to_add)) {
                $plugins_to_add = array_unique($plugins_to_add);
                
                call_user_func_array(array($context->controller, 'addJqueryPlugin'), $plugins_to_add);
            }
        }
    }
}
