<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCmsBlockCategorySlider extends AdvancedCmsBlockModel
{
    public $manifest = array(
        'css' => array(
            'admin' => array(
                'views/css/blocks/AdvancedCmsBlockCategorySlider/admin/AdvancedCmsBlockCategorySlider.css'
            ),
            'front' => array(
                'views/css/blocks/AdvancedCmsBlockCategorySlider/front/AdvancedCmsBlockCategorySlider.css'
            )
        ),
        'jquery_ui' => array(
            'front' => array(
                'bxslider'
            )
        ),
        'js' => array(
            'front' => array(
                'views/js/blocks/AdvancedCmsBlockCategorySlider/front/AdvancedCmsBlockCategorySlider.js'
            )
        )
    );
    
    public function __construct()
    {
        $this->type = 'cslider';
        
        parent::__construct();
        
        $this->name = $this->l('Category Slider Block');
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        $this->context->controller->ajax = true;
        $languages = Language::getLanguages();
        $count_languages = count($languages);
        $category_trees = array();
        $selected_categories = array();
        
        if (Validate::isLoadedObject($block)) {
            foreach ($block->contents as $id_lang => $language_categories) {
                if (Tools::strlen($language_categories)) {
                    $selected_categories[$id_lang] = Tools::jsonDecode($language_categories);
                }
            }
        }

        for ($i = 0; $i < $count_languages; $i++) {
            $tree_categories_helper = new HelperTreeCategories(
                'categories-treeview-'.$languages[$i]['id_lang'],
                null,
                $this->context->language->id
            );
            $tree_categories_helper->setRootCategory((Shop::getContext() == Shop::CONTEXT_SHOP ?
                Category::getRootCategory()->id_category : 0))->setUseCheckBox(true);
            
            if (isset($selected_categories[$languages[$i]['id_lang']])) {
                $tree_categories_helper->setSelectedCategories($selected_categories[$languages[$i]['id_lang']]);
            }

            $category_trees[$languages[$i]['id_lang']] = $tree_categories_helper->render();
        }
        
        $this->context->smarty->assign(array(
            'category_image_types' => ImageType::getImagesTypes('categories'),
            'category_trees' => $category_trees
        ));
    }
    
    public function prefilterResults($data, $settings)
    {
        if (!is_array($data) || !count($data)) {
            return;
        }

        $sql_groups_where = '';
        $sql_groups_join = '';
        $context = Context::getContext();
        $id_lang = $context->language->id;

        if (Group::isFeatureActive()) {
            $sql_groups_join = 'LEFT JOIN `'._DB_PREFIX_.'category_group` cg ON (cg.`id_category` = c.`id_category`)';
            $groups = FrontController::getCurrentCustomerGroups();
            $sql_groups_where = 'AND cg.`id_group` '.(count($groups) ? 'IN ('.
                implode(',', $groups).')' : '='.(int)Group::getCurrent()->id);
        }

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
            'SELECT
                c.*,
                cl.id_lang,
                cl.name,
                cl.description,
                cl.link_rewrite,
                cl.meta_title,
                cl.meta_keywords,
                cl.meta_description
            FROM
                `'._DB_PREFIX_.'category` c
            '.Shop::addSqlAssociation('category', 'c').'
            LEFT JOIN
                `'._DB_PREFIX_.'category_lang` cl ON (c.`id_category` = cl.`id_category` AND `id_lang` = '.
            (int)$id_lang.' '.Shop::addSqlRestrictionOnLang('cl').')'.$sql_groups_join.'
            WHERE
                c.`id_category` IN ('.implode(',', $data).')
            '.$sql_groups_where.'
            GROUP BY
                c.`id_category`
            ORDER BY
                `level_depth` ASC,
                category_shop.`position` ASC'
        );

        if (is_array($settings) && isset($settings['image_type'])
            && Validate::isLoadedObject($type = new ImageType((int)$settings['image_type']))) {
            $type = $type->name;
        }

        $settings['image_type'] = $type;

        foreach ($result as &$row) {
            $row['id_image'] = Tools::file_exists_cache(_PS_CAT_IMG_DIR_.$row['id_category'].'.jpg')
                ? (int)$row['id_category']
                : false;
            $row['cat_image'] = $context->link->getCatImageLink($row['link_rewrite'], $row['id_image'], $type);
        }

        if (!count($result)) {
            return;
        }

        $context->smarty->assign(array(
            'settings'      => $settings,
            'category_list' => $result
        ));
    }
}
