<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCmsBlockImage extends AdvancedCmsBlockModel
{
    private static $upload_path = '';
    private static $upload_folder = 'uploads/AdvancedCmsBlockImage/';
    private static $valid_image_types = array('jpg', 'jpeg', 'png', 'gif');
    
    public $manifest = array(
        'js' => array(
            'admin' => array(
                'views/js/blocks/AdvancedCmsBlockImage/jquery.ui.widget.js',
                'views/js/blocks/AdvancedCmsBlockImage/jquery.iframe-transport.js',
                'views/js/blocks/AdvancedCmsBlockImage/jquery.fileupload.js'
            )
        ),
        'css' => array(
            'admin' => array(
                'views/css/blocks/AdvancedCmsBlockImage/admin/AdvancedCmsBlockImage.css'
            )
        )
    );
    
    public function __construct()
    {
        $this->type = 'image';

        parent::__construct();
        
        if (Configuration::get('ADCMS_IMAGE_PATH') && (Tools::getValue('id_ad_cms') || Tools::getValue('id_cms'))) {
            $old_up = $this->getModuleInstance()->getLocalPath().self::$upload_folder;

            if (Tools::getValue('id_ad_cms')) {
                self::$upload_folder .= Tools::getValue('id_ad_cms').'/';
            } elseif (Tools::getValue('id_cms')) {
                self::$upload_folder .= Tools::getValue('id_cms').'/';
            }
            
            $up = $this->getModuleInstance()->getLocalPath().self::$upload_folder;

            if (!file_exists($up) && Tools::getValue('action') == 'getBlockData') {
                mkdir($up, 0777, true);
                copy($old_up.'index.php', $up.'index.php');
            }
        }

        self::$upload_path = $this->getModuleInstance()->getLocalPath().self::$upload_folder;
        
        $this->name = $this->l('Image Block');
        
        require_once(dirname(__FILE__).'/../ImageResize.php');
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        $this->context->smarty->assign(array(
            'image_gallery' => $this->getGalleryImages(),
            'gallery_path'  => $this->getUploadPathRelative(),
        ));
    }
    
    public function prefilterResults($data, $settings)
    {
        if (!$data || Tools::isEmpty($data)) {
            return;
        }

        $context = Context::getContext();

        if (isset($settings['image_url'])) {
            $settings['image_url'] = $this->ad_cms_page->replaceLinkAnchors($settings['image_url']);
        }

        $context->smarty->assign(array(
            'settings' => $settings,
            'image' => $this->getUploadPathRelative().$data
        ));
    }
    
    public function getGalleryImages()
    {
        $handle = opendir(self::$upload_path);

        if ($handle) {
            $result = array();

            while ($file = readdir($handle)) {
                if ($file !== '.' && $file !== '..' && !is_dir(self::$upload_path.$file)
                    && in_array(ImageResize::getExtension($file), self::$valid_image_types)) {
                    $extension = ImageResize::getExtension($file);
                    $file_name = Tools::substr($file, 0, -(Tools::strlen($extension) + 1));

                    $tmp = Tools::strpos($file_name, '_thumb');

                    if ($tmp !== false) {
                        $file_name = Tools::substr($file_name, 0, $tmp);
                    }

                    if (!array_key_exists($file_name, $result)) {
                        $result[$file_name] = array();
                    }

                    $result[$file_name][($tmp !== false) ? 'thumb' : 'original'] = $file;
                    if ($tmp == false) {
                        $result[$file_name]['size'] = getimagesize(self::$upload_path.$file);
                    }
                }
            }

            return count($result) ? $result : false;
        }

        return false;
    }

    public function getUploadPathReal()
    {
        return self::$upload_folder;
    }

    public function getUploadPathRelative()
    {
        return _MODULE_DIR_.$this->getModuleInstance()->name.'/'.self::$upload_folder;
    }

    public function uploadGalleryImage()
    {
        $return = array('status' => 'success', 'message' => 'false');

        if (isset($_FILES['adcms_image'])) {
            $file = $_FILES['adcms_image'];
            $new_filename = uniqid();
            $extension = self::getExtension($file['name']);

            if (!self::checkExtension($file['name'], array('jpg', 'jpeg', 'png', 'gif', 'bmp'))) {
                $return['status'] = 'error';
                $return['message'] = $this->l('Invalid file extension');
            } elseif (!move_uploaded_file($file['tmp_name'], self::$upload_path.implode(
                '.',
                array($new_filename, $extension)
            ))) {
                $return['status'] = 'error';
                $return['message'] = $this->l('Unable to upload file').' "'.$file['name'].'"';
            } else {
                $resize = new ImageResize(self::$upload_path.implode('.', array($new_filename, $extension)));
                $resize->resizeImage(100, 100, 'crop');
                $resize->saveImage(self::$upload_path.implode('.', array($new_filename.'_thumb', $extension)));

                $relative_path = $this->getUploadPathRelative();

                $return['filename'] = implode('.', array($new_filename, $extension));
                $return['thumbnail'] = $relative_path.implode('.', array($new_filename.'_thumb', $extension));
            }
        }

        return Tools::jsonEncode($return);
    }
    
    public function deleteGalleryImage($image)
    {
        $return = array('status' => 'success', 'message' => 'false');
        
        $extension = self::getExtension($image);
        $filename = Tools::substr($image, 0, -(Tools::strlen($extension) + 1));
        
        $path_full = self::$upload_path.implode('.', array($filename, $extension));
        $path_thumb = self::$upload_path.implode('.', array($filename.'_thumb', $extension));
        
        if (!unlink($path_full) || !unlink($path_thumb)) {
            $return['status'] = 'error';
            $return['message'] = $this->l('Unable to delete this image');
        }
        
        return Tools::jsonEncode($return);
    }

    private static function getExtension($file)
    {
        return Tools::strtolower(pathinfo($file, PATHINFO_EXTENSION));
    }

    private static function checkExtension($file, $allowed_extensions)
    {
        if (!is_array($allowed_extensions)) {
            return true;
        }

        $extension = self::getExtension($file);

        foreach ($allowed_extensions as $allowed_extension) {
            if ($extension == Tools::strtolower(trim($allowed_extension))) {
                return true;
            }
        }

        return false;
    }
}
