<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCmsBlockManufacturerSlider extends AdvancedCmsBlockModel
{
    public $manifest = array(
        'css' => array(
            'admin' => array(
                'views/css/blocks/AdvancedCmsBlockManufacturerSlider/admin/AdvancedCmsBlockManufacturerSlider.css'
            )
        ),
        'jquery_ui' => array(
            'admin' => array(
                'autocomplete'
            ),
            'front' => array(
                'bxslider'
            )
        ),
        'js' => array(
            'front' => array(
                'views/js/blocks/AdvancedCmsBlockManufacturerSlider/front/AdvancedCmsBlockManufacturerSlider.js'
            )
        )
    );
    
    public function __construct()
    {
        $this->type = 'mslider';
        
        parent::__construct();
        
        $this->name = $this->l('Manufacturer Slider Block');
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        $this->context->smarty->assign(array(
            'manufacturer_image_types' => ImageType::getImagesTypes('manufacturers')
        ));
    }
    
    public function prefilterResults($data, $settings)
    {
        $context = Context::getContext();
        $id_lang = $context->language->id;
        
        $id_list = array();

        if (!is_array($data) || !count($data)) {
            return;
        }

        foreach ($data as $manufacturer) {
            array_push($id_list, (int)$manufacturer['id_manufacturer']);
        }
            
        if (!count($id_list)) {
            return;
        }
        
        $sql = '
            SELECT
                *
            FROM
                `'._DB_PREFIX_.'manufacturer` m
                '.Shop::addSqlAssociation('manufacturer', 'm').'
            LEFT JOIN `'._DB_PREFIX_.'manufacturer_lang` ml
                ON (ml.`id_manufacturer` = m.`id_manufacturer` AND ml.`id_lang` = '.(int)$id_lang.')
            WHERE
                m.`id_manufacturer` IN ('.implode(',', $id_list).')';
                    
        $items = Db::getInstance()->executeS($sql);
        
        if ($items) {
            if (is_array($settings) && isset($settings['display_products_amount'])
                && $settings['display_products_amount']) {
                $results = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS(
                    'SELECT  p.`id_manufacturer`, COUNT(DISTINCT p.`id_product`) as nb_products
                    FROM `'._DB_PREFIX_.'product` p
                    '.Shop::addSqlAssociation('product', 'p').'
                    LEFT JOIN `'._DB_PREFIX_.'manufacturer` as m ON (m.`id_manufacturer`= p.`id_manufacturer`)
                    WHERE product_shop.`visibility` NOT IN ("none")
                    GROUP BY p.`id_manufacturer`'
                );
    
                $counts = array();
                foreach ($results as $result) {
                    $counts[(int)$result['id_manufacturer']] = (int)$result['nb_products'];
                }
    
                if (count($counts)) {
                    foreach ($items as $key => $manufacturer) {
                        $items[$key]['nb_products'] = isset($counts[(int)$manufacturer['id_manufacturer']]) ? $counts[(int)$manufacturer['id_manufacturer']] : 0;
                    }
                }
            }
            
            $results = array();
            $type = false;

            if (is_array($settings) && isset($settings['image_type'])
                && Validate::isLoadedObject($type = new ImageType((int)$settings['image_type']))) {
                $type = $type->name;
            }
        
            foreach ($items as $item) {
                $image = false;
                
                if (file_exists(_PS_MANU_IMG_DIR_.(int)$item['id_manufacturer'].'-'.$type.'.jpg')) {
                    $image = _THEME_MANU_DIR_.(int)$item['id_manufacturer'].'-'.$type.'.jpg';
                }
                    
                $item['image'] = $image;
                $item['link'] = $context->link->getManufacturerLink($item['id_manufacturer']);
                
                array_push($results, $item);
            }
        
            $results = array_values($results);
        
            $settings['image_type'] = $type;
    
            $context->smarty->assign(array(
                'settings'      => $settings,
                'manufacturer_list' => $results
            ));
        }
    }
    
    private function getMediumImageType()
    {
        $types = ImageType::getImagesTypes('manufacturers');
        $located_type = false;
    
        if ($types) {
            $max_size = 100;
    
            foreach ($types as $type) {
                if ($type['width'] <= $max_size && $type['height'] <= $max_size) {
                    if (!$located_type || ($located_type['width'] < $type['width']
                        && $located_type['height'] < $type['height'])) {
                        $located_type = array(
                            'name' => $type['name'],
                            'width' => $type['width'],
                            'height' => $type['height']
                        );
                    }
                }
            }
        }
    
        return $located_type ? $located_type['name'] : false;
    }
    
    public function ajaxSearch($query, $exclude_ids)
    {
        $exclude_ids = is_array($exclude_ids) && count($exclude_ids) ? $exclude_ids : false;

        $sql = 'SELECT
                    m.`id_manufacturer`,
                    m.`name`
                FROM
                    `'._DB_PREFIX_.'manufacturer` m
                    '.Shop::addSqlAssociation('manufacturer', 'm').'
                WHERE
                    (m.name LIKE \'%'.pSQL($query).'%\')
                '.(!empty($exclude_ids) ? '
                    AND
                        m.`id_manufacturer` NOT IN ('.$exclude_ids.') ' : ' ');
                    
        $items = Db::getInstance()->executeS($sql);
        
        if ($items) {
            $results = array();
        
            $image_type = $this->getMediumImageType();
        
            foreach ($items as $item) {
                $image = false;
                
                if (file_exists(_PS_MANU_IMG_DIR_.(int)$item['id_manufacturer'].'-'.$image_type.'.jpg')) {
                    $image = _THEME_MANU_DIR_.(int)$item['id_manufacturer'].'-'.$image_type.'.jpg';
                }
                
                $manufacturer = array(
                    'id'    => (int)$item['id_manufacturer'],
                    'name'  => $item['name'],
                    'image' => $image,
                );
                
                array_push($results, $manufacturer);
            }
        
            $results = array_values($results);
        
            die(Tools::jsonEncode($results));
        }
    }
}
