<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCmsBlockModule extends AdvancedCmsBlockModel
{
    private $installed_modules;
    
    public $manifest = array(
        'no_lang' => true,
        'css' => array(
            'admin' => array(
                'views/css/blocks/AdvancedCmsBlockModule/admin/AdvancedCmsBlockModule.css'
            )
        )
    );
    
    public function __construct()
    {
        $this->type = 'module';
        
        parent::__construct();
        
        $this->name = $this->l('Module Block');
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        $this->context->smarty->assign(array(
            'modules' => $this->getInstalledModules()
        ));
    }
    
    private function getInstalledModules()
    {
        if (!is_array($this->installed_modules)) {
            $controller = $this->context->controller;
            $modules = Module::getModulesOnDisk(true, false, $this->context->employee->id);
            $installed_modules = array();
            $id_list = array();
            
            foreach ($modules as $module) {
                if ($module->id && property_exists($module, 'tab') && $module->tab != 'payments_gateways') {
                    $installed_modules[$module->id] = $module;
                    $id_list[] = $module->id;
                }
            }
            
            if (count($installed_modules)) {
                $module_hooks = Db::getInstance()->ExecuteS(
                    'SELECT
                        m.*,
                        hm.`id_module`,
                        hm.`id_hook`,
                        h.`position`,
                        h.`title`,
                        h.`name`
                    FROM
                        `'._DB_PREFIX_.'module` m
                    LEFT JOIN
                        `'._DB_PREFIX_.'hook_module` hm ON (m.`id_module` = hm.`id_module`)
                    LEFT JOIN
                        `'._DB_PREFIX_.'hook` h ON (hm.`id_hook` = h.`id_hook`)
                    WHERE
                        m.`id_module` IN ('.(implode(',', $id_list)).')
                    AND
                        h.`position` = 1
                    AND
                        hm.`id_shop` = '.Context::getContext()->shop->id
                );
                
                $result = array();
                
                if (count($module_hooks)) {
                    foreach ($module_hooks as $db_module) {
                        if (array_key_exists($db_module['id_module'], $installed_modules)) {
                            if (!array_key_exists($db_module['id_module'], $result)) {
                                $result[$db_module['id_module']] = array(
                                    'name' => $installed_modules[$db_module['id_module']]->displayName,
                                    'title' => $installed_modules[$db_module['id_module']]->name,
                                    'id_module' => (int)$db_module['id_module'],
                                    'hooks' => array()
                                );
                            }
                            
                            $result[$db_module['id_module']]['hooks'][] = array(
                                'id_hook' => $db_module['id_hook'],
                                'title' => $db_module['title'],
                                'name' => $db_module['name']
                            );
                        }
                    }
                    
                    if (count($result)) {
                        $this->installed_modules = $result;
                    }
                }
            }
        }
        
        return is_array($this->installed_modules) ? $this->installed_modules : false;
    }
    
    public function prefilterResults($data, $settings)
    {
        if (is_array($settings) && count($settings)) {
            $context = Context::getContext();
            
            if (Validate::isLoadedObject($module = Module::getInstanceByName($settings['module']))) {
                if (method_exists($module, 'hook'.$settings['hook'])) {
                    $context->smarty->assign('module_output', Hook::exec($settings['hook'], array(), $module->id));
                } else {
                    $old_name = Hook::getRetroHookName($settings['hook']);
                    
                    if ($old_name && method_exists($module, 'hook'.$old_name)) {
                        $context->smarty->assign('module_output', Hook::exec($old_name, array(), $module->id));
                    } else {
                        $context->smarty->assign('module_output', false);
                    }
                }
            }
        }
    }
}
