<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdvancedCmsBlockProduct extends AdvancedCmsBlockModel
{
    public $manifest = array(
        'css' => array(
            'admin' => array(
                'views/css/blocks/AdvancedCmsBlockProduct/admin/AdvancedCmsBlockProduct.css'
            )
        ),
        'jquery_ui' => array(
            'admin' => array(
                'autocomplete'
            )
        ),
        'js' => array(
            'front' => array(
                'views/js/blocks/AdvancedCmsBlockProduct/front/AdvancedCmsBlockProduct.js'
            )
        )
    );
    
    public function __construct()
    {
        $this->type = 'product';
        
        parent::__construct();
        
        $this->name = $this->l('Product Block');
    }
    
    public function assignBlockTemplateValues($context, $block = false)
    {
        $this->context->smarty->assign(array(
            'product_image_types' => ImageType::getImagesTypes('products')
        ));
    }
    
    public function prefilterResults($data, $settings)
    {
        $id_list = array();

        if (!is_array($data) || !count($data)) {
            return;
        }

        foreach ($data as $product) {
            array_push($id_list, (int)$product['id_product']);
        }

        if (count($id_list)) {
            $npd = Configuration::get('PS_NB_DAYS_NEW_PRODUCT');
            $context = Context::getContext();
            $id_lang = $context->language->id;
            $currency = $context->currency;

            $products = Db::getInstance()->ExecuteS(
                'SELECT
                    p.*,
                    product_shop.*,
                    stock.out_of_stock,
                    IFNULL(stock.quantity, 0) as quantity
                    '.(Combination::isFeatureActive() ? ',
                        MAX(product_attribute_shop.id_product_attribute) id_product_attribute,
                        MAX(product_attribute_shop.minimal_quantity) AS product_attribute_minimal_quantity' : ''
                    ).',
                    pl.`description`,
                    pl.`description_short`,
                    pl.`available_now`,
                    pl.`available_later`,
                    pl.`link_rewrite`,
                    pl.`meta_description`,
                    pl.`meta_keywords`,
                    pl.`meta_title`,
                    pl.`name`,
                    MAX(image_shop.`id_image`) id_image,
                    MAX(il.`legend`) as legend,
                    m.`name` AS manufacturer_name,
                    cl.`name` AS category_default,
                    DATEDIFF(
                        product_shop.`date_add`,
                        DATE_SUB(
                            NOW(),
                            INTERVAL '.(Validate::isUnsignedInt($npd) ? $npd : 20).' DAY
                        )
                    ) > 0 AS new,
                    product_shop.price AS orderprice
                FROM
                    `'._DB_PREFIX_.'category_product` cp
                LEFT JOIN
                    `'._DB_PREFIX_.'product` p ON p.`id_product` = cp.`id_product`
                '.Shop::addSqlAssociation('product', 'p').
                (Combination::isFeatureActive() ? 'LEFT JOIN `'._DB_PREFIX_.'product_attribute` pa
                ON (p.`id_product` = pa.`id_product`)
                '.Shop::addSqlAssociation('product_attribute', 'pa', false, 'product_attribute_shop.`default_on` = 1').'
                '.Product::sqlStock('p', 'product_attribute_shop', false, $context->shop):Product::sqlStock('p', 'product', false, Context::getContext()->shop)).'
                LEFT JOIN
                    `'._DB_PREFIX_.'category_lang` cl
                ON (product_shop.`id_category_default` = cl.`id_category`
                AND cl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').')
                LEFT JOIN
                    `'._DB_PREFIX_.'product_lang` pl
                ON (p.`id_product` = pl.`id_product`
                AND pl.`id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('pl').')
                LEFT JOIN
                    `'._DB_PREFIX_.'image` i
                ON (i.`id_product` = p.`id_product`)'.
                Shop::addSqlAssociation('image', 'i', false, 'image_shop.cover=1').'
                LEFT JOIN
                    `'._DB_PREFIX_.'image_lang` il
                ON (image_shop.`id_image` = il.`id_image`
                AND il.`id_lang` = '.(int)$id_lang.')
                LEFT JOIN
                    `'._DB_PREFIX_.'manufacturer` m
                ON m.`id_manufacturer` = p.`id_manufacturer`
                WHERE
                    product_shop.`id_shop` = '.(int)$context->shop->id.'
                AND
                    product_shop.`id_product` IN ('.implode(',', $id_list).')
                AND
                    product_shop.`active` = 1
                AND
                    product_shop.`visibility` IN ("both", "catalog")
                GROUP BY
                    product_shop.id_product'
            );

            if (!$products) {
                return;
            }

            $products = Product::getProductsProperties($id_lang, $products);

            $type = false;

            if (is_array($settings) && isset($settings['image_type'])
                && Validate::isLoadedObject($type = new ImageType((int)$settings['image_type']))) {
                $type = $type->name;
            }

            $settings['image_type'] = $type;

            foreach ($products as &$product) {
                $product['price_tax_exc_cur'] = Tools::convertPrice($product['price_tax_exc'], $currency);
                $product['price_cur'] = Tools::convertPrice($product['price'], $currency);
                $product['price_wr_cur'] = Tools::displayPrice($product['price_without_reduction'], $currency);

                $product['ad_link'] = $context->link->getProductLink(
                    $product['id_product'],
                    $product['link_rewrite'],
                    $product['category'],
                    $product['ean13']
                );
                $product['image_link'] = $context->link->getImageLink(
                    $product['link_rewrite'],
                    $product['id_image'],
                    $type
                );
                $product['cart_link'] = $context->link->getPageLink(
                    'cart',
                    true,
                    null,
                    "qty=1&id_product=".$product['id_product']."&token=".Tools::getToken(false)."&add"
                );
            }

            $context->smarty->assign(array(
                'settings'      => $settings,
                'product_list'  => $products,
                'static_token'  => Tools::getToken(false),
                'currency'      => $currency,
                'stock_manag'   => Configuration::get('PS_STOCK_MANAGEMENT'),
                'catalog_mode'  => (bool)Configuration::get('PS_CATALOG_MODE') || (Group::isFeatureActive() && !(bool)Group::getCurrent()->show_prices),
                'price_display' => Product::getTaxCalculationMethod((int)$context->cookie->id_customer),
            ));
        }
    }
    
    public function ajaxSearch($query, $exclude_ids)
    {
        $exclude_ids = is_array($exclude_ids) && count($exclude_ids) ? $exclude_ids : false;
            
        $sql = 'SELECT
                    p.`id_product`,
                    pl.`link_rewrite`,
                    p.`reference`,
                    pl.`name`,
                    MAX(image_shop.`id_image`) id_image,
                    il.`legend`,
                    p.`cache_default_attribute`
                FROM
                    `'._DB_PREFIX_.'product` p
                    '.Shop::addSqlAssociation('product', 'p').'
                    LEFT JOIN
                        `'._DB_PREFIX_.'product_lang` pl ON (pl.id_product = p.id_product
                            AND pl.id_lang = '.(int)Context::getContext()->language->id
                        .Shop::addSqlRestrictionOnLang('pl').')
                    LEFT JOIN
                        `'._DB_PREFIX_.'image` i ON (i.`id_product` = p.`id_product`)'
                        .Shop::addSqlAssociation('image', 'i', false, 'image_shop.cover=1').'
                    LEFT JOIN
                        `'._DB_PREFIX_.'image_lang` il ON (i.`id_image` = il.`id_image`
                            AND il.`id_lang` = '.(int)Context::getContext()->language->id.')
                    WHERE
                        (pl.name LIKE \'%'.pSQL($query).'%\' OR p.reference LIKE \'%'.pSQL($query).'%\')
                    AND
                        i.`id_image` != ""'.(!empty($exclude_ids) ? '
                    AND
                        p.id_product NOT IN ('.$exclude_ids.') ' : ' ').'
                    GROUP BY
                        p.id_product';
        
        $items = Db::getInstance()->executeS($sql);
        
        if ($items) {
            $results = array();
        
            $image_type = $this->getMediumImageType();
        
            foreach ($items as $item) {
                $product = array(
                    'id'    => (int)$item['id_product'],
                    'name'  => $item['name'],
                    'ref'   => (!empty($item['reference']) ? $item['reference'] : ''),
                    'image' => $image_type ? str_replace(
                        'http://',
                        Tools::getShopProtocol(),
                        Context::getContext()->link->getImageLink(
                            $item['link_rewrite'],
                            $item['id_image'],
                            $image_type
                        )
                    ) : false,
                );
                array_push($results, $product);
            }
        
            $results = array_values($results);
        
            die(Tools::jsonEncode($results));
        }
    }
    
    private function getMediumImageType()
    {
        $types = ImageType::getImagesTypes('products');
        $located_type = false;
    
        if ($types) {
            $max_size = 100;
    
            foreach ($types as $type) {
                if ($type['width'] <= $max_size && $type['height'] <= $max_size) {
                    if (!$located_type || ($located_type['width'] < $type['width']
                        && $located_type['height'] < $type['height'])) {
                        $located_type = array(
                            'name' => $type['name'],
                            'width' => $type['width'],
                            'height' => $type['height']
                        );
                    }
                }
            }
        }
    
        return $located_type ? $located_type['name'] : false;
    }
}
