<?php
/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

class AdminAdvancedCMSSettingsController extends ModuleAdminController
{
    private $installed_blocks;
    
    public function __construct()
    {
        $this->table = 'ad_cms';
        $this->className = 'AdvancedCMSPage';
        $this->lang = true;
        $this->bootstrap = true;
        if (method_exists('Context', 'getTranslator')) {
            $this->translator = Context::getContext()->getTranslator();
        }

        $this->addRowAction('edit');
        $this->addRowAction('duplicate');
        $this->addRowAction('delete');

        $this->fields_list = array(
            'id_ad_cms' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'width' => 20
            ),
            'name' => array(
                'title' => $this->l('Name'),
                'width' => 'auto'
            ),
            'link_rewrite' => array(
                'title' => $this->l('URL'),
                'width' => 'auto',
                'remove_onclick' => true,
                'callback' => 'getCMSURL'
            ),
            'active' => array(
                'title' => $this->l('Enabled'),
                'align' => 'text-center',
                'active' => 'status',
                'type' => 'bool',
                'orderby' => false,
                'filter_key' => 'a!active'
            ),
        );

        parent::__construct();
        
        $this->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );
        
        $active_cms = array(
            array(
                'value' => 0,
                'name' => $this->l('None (display standard index page)')
            )
        );
        
        $pages = AdvancedCMSPage::getCmsPages();
        
        foreach ($pages as $page) {
            $active_cms[] = array(
                'value' => $page['id_ad_cms'],
                'name' => $page['name']
            );
        }

        // $this->toolbar_btn['preview'] = array(
        //     'href' => self::$currentIndex.'&updategallery&id_gallery=&token='.$this->token,
        //     'desc' => $this->l('Generate XML')
        // );
        
        $this->fields_options = array(
            'general' => array(
                'title' =>    $this->l('Settings'),
                'fields' =>    array(
                    'ADCMS_URL_WORD' => array(
                        'title' => $this->l('Word in URL'),
                        'hint' => $this->l('Replacing the word "cms" (/cms/ID/REWRITE/) in the url-address page.'),
                        'validation' => 'isSubDomainName',
                        'type' => 'textLang'
                    ),
                    'ADCMS_LANDING' => array(
                        'title' => $this->l('Landing page'),
                        'hint' => $this->l('This page will be displayed instead of your main page.'),
                        'validation' => 'isInt',
                        'cast' => 'intval',
                        'type' => 'select',
                        'list' => $active_cms,
                        'identifier' => 'value'
                    ),
                    'ADCMS_IMAGE_PATH' => array(
                        'title' => $this->l('Individual folders for images.'),
                        'hint' => $this->l('If you select "Yes", each page will have its own folder for images. If "no" - folder will be common.'),
                        'validation' => 'isBool',
                        'cast' => 'intval',
                        'type' => 'bool'
                    ),
                ),
                'submit' => array('title' => $this->l('Save')),
            ),
        );
        
        $this->installed_blocks = AdvancedCmsBlockModel::getInstalledBlocks();
    }
    
    public function updateOptionAdCMSLanding()
    {
        Configuration::updateValue('ADCMS_LANDING', (int)Tools::getValue('ADCMS_LANDING'));
        
        $this->module->hookActionHtaccessCreate(array());
    }

    public function init()
    {
        if (Tools::getIsset('action')) {
            $result = false;

            if (Tools::getIsset('block_controller')) {
                $result = AdvancedCmsBlockModel::callBlockMethod(
                    Tools::getValue('block_controller'),
                    Tools::getValue('action'),
                    Tools::getValue('params')
                );
            } else {
                switch (Tools::getValue('action')) {
                    case 'getBlockData':
                        $result = $this->getBlockData(
                            Tools::getValue('block_type'),
                            Tools::getValue('id_ad_cms'),
                            Tools::getValue('id_ad_cms_block')
                        );
                        break;
                    case 'getContainerData':
                        $result = $this->renderContainerTemplate(Tools::getValue('id_ad_cms'));
                        break;
                    case 'upload':
                        $result = $this->module->uploadGalleryImage();
                        break;
                }
            }

            die($result);
        } else {
            return parent::init();
        }
    }

    public function setMedia()
    {
        parent::setMedia();

        $this->addJqueryUI(array(
            'ui.draggable',
            'ui.sortable',
            'ui.droppable'
        ));
        
        $admin_webpath = str_ireplace(_PS_CORE_DIR_, '', _PS_ADMIN_DIR_);
        $admin_webpath = preg_replace('/^'.preg_quote(DIRECTORY_SEPARATOR, '/').'/', '', $admin_webpath);
        $bo_theme = ((Validate::isLoadedObject(Context::getContext()->employee)
            && Context::getContext()->employee->bo_theme) ? Context::getContext()->employee->bo_theme : 'default');

        if (!file_exists(_PS_BO_ALL_THEMES_DIR_.$bo_theme.DIRECTORY_SEPARATOR.'template')) {
            $bo_theme = 'default';
        }

        $js_path = __PS_BASE_URI__.$admin_webpath.'/themes/'.$bo_theme.'/js/tree.js';

        $this->addJs($js_path);

        $this->addJqueryPlugin('fancybox');

        $this->addJS(_MODULE_DIR_.$this->module->name.'/views/js/cmsbuilder.js');
        $this->addCSS(_MODULE_DIR_.$this->module->name.'/views/css/cmsbuilder.css');
        
        if (is_array($this->installed_blocks) && count($this->installed_blocks)) {
            $plugins_to_add = array();
            
            foreach ($this->installed_blocks as $block) {
                $this->addJS(_MODULE_DIR_.$this->module->name.'/views/js/blocks/'.(get_class($block)).'.js');
                
                if (array_key_exists('js', $block->manifest) && array_key_exists('admin', $block->manifest['js'])) {
                    foreach ($block->manifest['js']['admin'] as $v) {
                        $this->addJS($this->module->getPathUri().$v);
                    }
                }
                
                if (array_key_exists('jquery_ui', $block->manifest)
                    && array_key_exists('admin', $block->manifest['jquery_ui'])) {
                    $plugins_to_add = array_merge($plugins_to_add, $block->manifest['jquery_ui']['admin']);
                }
                
                if (array_key_exists('css', $block->manifest) && array_key_exists('admin', $block->manifest['css'])) {
                    foreach ($block->manifest['css']['admin'] as $v) {
                        $this->addCSS($this->module->getPathUri().$v);
                    }
                }
            }
            
            if (count($plugins_to_add)) {
                $plugins_to_add = array_unique($plugins_to_add);
                
                call_user_func_array(array($this, 'addJqueryPlugin'), $plugins_to_add);
            }
        }
    }

    public function renderForm()
    {
        if (!($obj = $this->loadObject(true))) {
            return;
        }

        $this->context->controller->redirect_after = true;
        $this->context->controller->submit_action = 'submitAddad_cmsAndStay';

        $this->fields_form = array(
            'legend' => array(
                'title' => $this->l('CMS Page'),
                'icon' => 'icon-file'
            ),
            'input' => array(
                array(
                    'type' => 'free',
                    'label' => $this->l('CMS Preview'),
                    'name' => 'cms_preview'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('CMS Title:'),
                    'name' => 'name',
                    'size' => 33,
                    'required' => true,
                    'lang' => true,
                    'hint' => $this->l('Forbidden characters:').' 0-9!<>,;?=+()@#"�{}_$%:'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Friendly URL:'),
                    'name' => 'link_rewrite',
                    'size' => 33,
                    'required' => true,
                    'lang' => true,
                    'hint' => $this->l('Only letters, numbers, underscore (_) and the minus (-) character are allowed.')
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('Meta Description:'),
                    'name' => 'meta_description',
                    'size' => 33,
                    'required' => false,
                    'lang' => true,
                    'hint' => $this->l('Forbidden characters:').' 0-9!<>,;?=+()@#"�{}_$%:'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta Keywords:'),
                    'name' => 'meta_keywords',
                    'size' => 100,
                    'required' => false,
                    'lang' => true,
                    'hint' => $this->l('Forbidden characters:').' 0-9!<>,;?=+()@#"�{}_$%:'
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Meta Title:'),
                    'name' => 'meta_title',
                    'size' => 100,
                    'required' => false,
                    'lang' => true,
                    'hint' => $this->l('Forbidden characters:').' 0-9!<>,;?=+()@#"�{}_$%:'
                ),
                array(
                    'type' => 'textarea',
                    'label' => $this->l('CSS:'),
                    'name' => 'css_content',
                    'size' => 33,
                    'required' => false,
                    'lang' => true,
                    'hint' => $this->l('Only letters, numbers, underscore (_) and the minus (-) character are allowed.'),
                    'desc' => $this->l('The contents of this area will be saved as a separate css file and included on this cms page. Note: these styles will not be applied for cms included via [displayCms] tag.')
                ),
                array(
                    'type' => 'free',
                    'col' => 12,
                    'name' => 'cms_contents'
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Display Header:'),
                    'name' => 'display_header',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'display_header_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'display_header_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Display theme\'s default header')
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Left Column:'),
                    'name' => 'column_left',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'column_left_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'column_left_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Display theme\'s left column for this page')
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Right Column:'),
                    'name' => 'column_right',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'column_right_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'column_right_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Display theme\'s right column for this page')
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Allow embedding:'),
                    'name' => 'embedding',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Enable or disable embedding form in the different pages')
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Status:'),
                    'name' => 'active',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => 1,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => 0,
                            'label' => $this->l('Disabled')
                        )
                    ),
                    'desc' => $this->l('Enable or disable this form')
                ),
            ),
            'buttons' => array(
                array(
                    'href' => self::$currentIndex,
                    'icon' => 'process-icon-back',
                    'class' => 'pull-right',
                    'title' => $this->l('Back to List')
                )
            )
        );

        if (Tools::getIsset('cms_form_data')) {
            $this->context->smarty->assign('post_fields', Tools::jsonEncode(Tools::getValue('cms_form_data')));
        } elseif (Validate::isLoadedObject($obj) && $cms_data = $obj->getCmsData()) {
            $this->context->smarty->assign('post_fields', Tools::jsonEncode($cms_data));
        }

        $tab_name = 'AdminAdvancedCMSSettings';
        $preview_link = $this->context->link->getModuleLink(
            $this->module->name,
            'adcms',
            array(
                'id_cms'      => $obj->id,
                'adtoken'     => Tools::getAdminToken(
                    $tab_name.(int)Tab::getIdFromClassName($tab_name).(int)$this->context->employee->id
                ),
                'id_employee' => (int)$this->context->employee->id,
                'rewrite'     => $obj->link_rewrite[$this->context->language->id]
            )
        );

        $this->context->smarty->assign(array(
            'installed_blocks'    => $this->installed_blocks,
            'native_blocks'       => Tools::jsonEncode(AdvancedCMSBlock::getNativeBlockTypes()),
            'languages'           => Tools::jsonEncode(Language::getLanguages()),
            'HOOK_ADCMS_MENU'     => Hook::exec('displayAdminAdCmsMenu'),
            'current_language'    => $this->context->language->id,
            'preview'             => (Validate::isLoadedObject($obj) && !$obj->active) ? $preview_link : false,
            'fo_link'             => (Validate::isLoadedObject($obj) && $obj->active) ? $this->getFrontOfficeLink($obj) : false
        ));
        
        if (Tools::getIsset('block_errors')) {
            $this->context->smarty->assign('block_errors', Tools::getValue('block_errors'));
        }

        $this->fields_value['cms_contents'] = $this->context->smarty->fetch(
            _PS_MODULE_DIR_.$this->module->name.'/views/templates/hook/cmsbuilder.tpl'
        );
        $this->fields_value['cms_preview'] = $this->context->smarty->fetch(
            _PS_MODULE_DIR_.$this->module->name.'/views/templates/hook/cmspreview.tpl'
        );
        $this->fields_value['css_content'] = $obj->getCssContents();

        if (Shop::isFeatureActive()) {
            $this->fields_form['input'][] = array(
                'type' => 'shop',
                'label' => $this->l('Shop association:'),
                'name' => 'checkBoxShopAsso',
            );
        }

        $this->fields_form['submit'] = array(
            'id'    => 'submitAddad_cmsAndStay',
            'title' => $this->l('Save')
        );

        return parent::renderForm();
    }

    protected function updateAssoShop($id_object)
    {
        if (!Shop::isFeatureActive()) {
            return true;
        }

        $assos_shop = Tools::getValue('checkBoxShopAsso_ad_cms');

        if (empty($assos_shop)) {
            $this->errors[] = Tools::displayError('Select at least one Shop association!');
            return false;
        }

        Db::getInstance()->delete($this->table.'_shop', '`'.$this->identifier.'` = '.(int)$id_object);

        foreach (array_keys($assos_shop) as $id_shop) {
            Db::getInstance()->insert('ad_cms_shop', array(
                'id_ad_cms' => (int)$id_object,
                'id_shop' => (int)$id_shop,
            ));
        }

        return true;
    }

    private function getFrontOfficeLink($obj)
    {
        $id_lang = $this->context->language->id;

        return $this->context->link->getModuleLink(
            'advancedcms',
            'adcms',
            array('id_cms' => $obj->id, 'rewrite' => $obj->link_rewrite[$id_lang])
        );
    }

    private function getBlockData($block_type, $id_ad_cms = false, $id_ad_cms_block = false)
    {
        $block = AdvancedCMSBlock::getBlock($id_ad_cms, $id_ad_cms_block);

        die($this->renderBlockTemplateByType($block_type, $block));
    }

    private function renderBlockTemplateByType($block_type, $block = false)
    {
        $languages = Language::getLanguages();
        $count_languages = count($languages);

        $this->context->smarty->assign(array(
            'languages'            => $languages,
            'defaultFormLanguage'  => $this->context->language->id,
            'iso'                  => Language::getIsoById($this->context->language->id),
            'ad'                   => basename(_PS_ADMIN_DIR_)
        ));
        
        if (is_array($this->installed_blocks) && count($this->installed_blocks)) {
            foreach ($this->installed_blocks as $installed_block) {
                if ($block_type == $installed_block->type) {
                    $installed_block->assignBlockTemplateValues($this->context, $block);
                    $block_contents = $this->context->smarty->fetch(_PS_MODULE_DIR_.$this->module->name.'/views/templates/admin/blocks/'.(get_class($installed_block)).'.tpl');
                }
            }
        }

        if (!isset($block_contents)) {
            $block_contents = Hook::exec('displayAdminAdCmsBlockContents', array('type' => $block_type));
        }

        $this->context->smarty->assign(array(
            'block_contents' => $block_contents
        ));

        return (Tools::jsonEncode(array(
            'html'  => $this->context->smarty->fetch(
                _PS_MODULE_DIR_.$this->module->name.'/views/templates/admin/block.tpl'
            ),
            'block' => $block
        )));
    }

    private function renderContainerTemplate()
    {
        return (Tools::jsonEncode(array(
            'html' => $this->context->smarty->fetch(_PS_MODULE_DIR_.$this->module->name.'/views/templates/admin/container.tpl')
        )));
    }

    private static function getIndexedLanguageList()
    {
        $languages = Language::getLanguages();
        $indexed = array();

        foreach ($languages as $language) {
            $indexed[$language['id_lang']] = $language;
        }

        return $indexed;
    }

    private function getFormJson()
    {
        /* Using $_POST instead of Tools::getValue, because the latter unescapes
         * the quotes, thus ruining JSON string
         * Hacked POST to make validator happy :)
         */
        if (Tools::getIsset('form_json')) {
            $p = $
            {
                '_POST'
            };

            if (get_magic_quotes_gpc()) {
                $p['form_json'] = Tools::stripslashes($p['form_json']);
            }
            
            return Tools::jsonDecode($p['form_json'], true);
        }

        return false;
    }

    public function validateRules($class_name = false)
    {
        parent::validateRules($class_name);

        /* Using $_POST instead of Tools::getValue, because the latter unescapes
         * the quotes, thus ruining JSON string
         */
        $cms_data = $this->getFormJson();
        $languages = self::getIndexedLanguageList();
        $id_lang_default = Configuration::get('PS_LANG_DEFAULT');
        $erroneous_blocks = array();

        if (!is_array($cms_data) || !count($cms_data)) {
            $this->errors['cmsdata'] = $this->l('Please add blocks to your page');
        } else {
            $container_counter = 1;

            foreach ($cms_data as &$container) {
                $block_counter = 1;
                foreach ($container['items'] as &$item) {
                    $manifest = false;

                    if (isset($item['controller'])
                        && array_key_exists($item['controller'], $this->installed_blocks)
                        && count($this->installed_blocks[$item['controller']]->manifest)) {
                        $manifest = $this->installed_blocks[$item['controller']]->manifest;
                    }
                    
                    if ((!isset($container['deleted']) || !$container['deleted'])
                        && (!isset($item['deleted']) || !$item['deleted'])) {
                        if (Tools::isEmpty($item['content'])) {
                            $error = sprintf(
                                $this->l('A value is required in block %s (container #%d)'),
                                $item['reference'] ? $item['reference'] : '#'.$block_counter,
                                $container_counter
                            );
                            
                            $this->errors['cmsdata'] = $error;
                            
                            $erroneous_blocks[] = array(
                                'container' => $container_counter,
                                'block' => $block_counter,
                                'container_index' => $container_counter - 1,
                                'block_index' => $block_counter - 1,
                                'error' => $error
                            );
                            
                        } elseif (!Tools::isEmpty($item['reference']) && !Validate::isCatalogName($item['reference'])) {
                            $error = sprintf(
                                $this->l('Invalid reference for block #%d (container #%d)'),
                                $languages[$id_lang_default]['name'],
                                $block_counter,
                                $container_counter
                            );
                            
                            $this->errors['input'] = $error;
                            
                            $erroneous_blocks[] = array(
                                'container_index' => $container_counter - 1,
                                'block_index' => $block_counter - 1,
                                'error' => $error
                            );
                        } else {
                            if (!$manifest || !isset($manifest['no_lang']) || !$manifest['no_lang']) {
                                if (!isset($item['content']['lang_'.$id_lang_default])
                                    || Tools::isEmpty($item['content']['lang_'.$id_lang_default]['value'])) {
                                    $error = sprintf(
                                        $this->l('Value is required at least for %s language in block %s (container #%d)'),
                                        $languages[$id_lang_default]['name'],
                                        $item['reference'] ? '"'.$item['reference'].'"' : '#'.$block_counter,
                                        $container_counter
                                    );
                                    
                                    $this->errors['input'] = $error;
                                    
                                    $erroneous_blocks[] = array(
                                        'container_index' => $container_counter - 1,
                                        'block_index' => $block_counter - 1,
                                        'error' => $error
                                    );
                                } else {
                                    foreach ($languages as $lang) {
                                        foreach ($item['content'] as &$lang_data) {
                                            if (!isset($item['content']['lang_'.$lang['id_lang']]) || Tools::isEmpty($item['content']['lang_'.$lang['id_lang']]['value'])) {
                                                $item['content']['lang_'.$lang['id_lang']]['value'] = $item['content']['lang_'.$id_lang_default]['value'];
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        Hook::exec(
                            'actionAdminAdCmsBlockValidate',
                            array('block' => $item, 'container' => $container, 'errors' => &$this->errors)
                        );

                        $block_counter++;
                    }
                }

                $container_counter++;
            }
        }

        $_POST['cms_form_data'] = $cms_data;
        $_POST['block_errors'] = count($erroneous_blocks) ? $erroneous_blocks : null;
    }
    
    public function getCMSURL($name, $cms)
    {
        $link = $this->context->link->getModuleLink(
            $this->module->name,
            'adcms',
            array('id_cms' => $cms['id_ad_cms'], 'rewrite' => $cms['link_rewrite'])
        );
        
        return '<a href="'.$link.'" target="_blank">'.$link.'</a>';
    }
    
    public function postProcess()
    {
        if (Tools::getIsset('duplicate'.$this->table)) {
            if ($this->tabAccess['edit'] === '1') {
                $id = (int)Tools::getValue('id_'.$this->table);
                
                $obj = new AdvancedCMSPage($id);
                
                if (!Validate::isLoadedObject($obj)) {
                    $this->errors[] = Tools::displayError('Unable to load this CMS.');
                } else {
                    if ($new_obj = $obj->duplicateObject()) {
                        Tools::redirectAdmin(
                            self::$currentIndex.'&id_'.$this->table.'='.$new_obj->id.'&conf=4&token='.$this->token
                        );
                    } else {
                        $this->errors[] = Tools::displayError('Unable to clone this CMS.');
                    }
                }
            } else {
                $this->errors[] = Tools::displayError('You do not have permission to edit this.');
            }
        } else {
            parent::postProcess();
        }
    }
    
    public function renderList()
    {
        if (Tools::isSubmit('submitAdvancedCmsBackup')) {
            $active = Tools::getValue('active');
            
            if (!Validate::isBool($active)) {
                $active = false;
            }
            
            if ($filename = AdvancedCMSPage::backUpPages($active)) {
                $this->context->smarty->assign(
                    'backup_download',
                    self::$currentIndex.'&token='.Tools::getAdminTokenLite(
                        'AdminAdvancedCMSSettings',
                        $this->context
                    ).'&backup_download='.$filename
                );
            } else {
                $this->context->smarty->assign('backup_error', true);
            }
        } elseif ($download_file = Tools::getValue('backup_download')) {
            $path = $this->module->getLocalPath().'backup/'.$download_file;
            
            if (Validate::isFileName($download_file) && file_exists($path)) {
                header('Pragma: private');
                header('Cache-control: private, must-revalidate');
                header('Content-Type: application/octet-stream');
                header('Content-Length: '.(string)filesize($path));
                header('Content-Disposition: attachment; filename="'.($download_file).'"');
                readfile($path);
            }
        } elseif (Tools::isSubmit('submitAdvancedCmsRestore')) {
            $file = isset($_FILES['backup_restore']) ? $_FILES['backup_restore'] : false;
            
            if (!$file) {
                $this->context->smarty->assign('restore_error', $this->l('Please select the backup file'));
            } elseif (pathinfo($file['name'], PATHINFO_EXTENSION) != 'xml') {
                $this->context->smarty->assign('restore_error', $this->l('Invalid file extension'));
            } else {
                $remove = Tools::getValue('remove');
            
                if (!Validate::isBool($remove)) {
                    $remove = false;
                }
                if (AdvancedCMSPage::restoreFromBackup($file['tmp_name'], $remove)) {
                    Tools::redirectAdmin(
                        self::$currentIndex.'&token='.Tools::getAdminTokenLite(
                            'AdminAdvancedCMSSettings',
                            $this->context
                        ).'&conf=18'
                    );
                } else {
                    $this->context->smarty->assign(
                        'restore_error',
                        $this->l('Unable to restore the entries, please check file validity.')
                    );
                }
            }
        }
        
        $this->context->smarty->assign(array(
            'form_target' => self::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminAdvancedCMSSettings', $this->context)
        ));
        
        return parent::renderList();
    }
}
