<?php
/**
 * advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2016 silbersaiten
 * @version   1.2.0
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

require_once(dirname(__FILE__).'/../../classes/AdvancedCMSPage.php');
require_once(dirname(__FILE__).'/../../classes/AdvancedCMSContainer.php');
require_once(dirname(__FILE__).'/../../classes/AdvancedCMSBlock.php');

class AdvancedCMSAdCmsModuleFrontController extends ModuleFrontController
{
    public $cms = false;
    public $cms_data = false;
    public $vt = 't17';

    public function __construct()
    {
        parent::__construct();

        if (version_compare('1.7.0.0', _PS_VERSION_, '>')) {
            $this->vt = 't16';
        }
        
        $preview_token = Tools::getAdminToken(
            'AdminAdvancedCMSSettings'.(int)Tab::getIdFromClassName('AdminAdvancedCMSSettings').(int)Tools::getValue('id_employee')
        );

        if (!Validate::isLoadedObject($cms = new AdvancedCMSPage(
            (int)Tools::getValue('id_cms'),
            $this->context->language->id
        )) || (!$cms->active && Tools::getValue('adtoken') != $preview_token)) {
            Tools::redirect('index.php');
        }

        if (Shop::isFeatureActive()) {
            if (!$cms->isAssociatedToShop($this->context->shop->id)) {
                Tools::redirect('index.php');
            }
        }

        $this->cms = $cms;

        $this->display_column_left = $this->cms->column_left;
        $this->display_column_right = $this->cms->column_right;
    }

    public function init()
    {
        parent::init();
        
        $this->cms_data = $this->cms->getCmsData($this->context->language->id, true);
        
        if (!$this->cms_data) {
            Tools::redirect('index.php');
        }

        $metas = Meta::getHomeMetas($this->context->language->id, null);

        if (!Tools::isEmpty($this->cms->meta_title)) {
            $metas['meta_title'] = $this->cms->meta_title.' - '.$metas['meta_title'];
        }

        if (!Tools::isEmpty($this->cms->meta_description)) {
            $metas['meta_description'] = $this->cms->meta_description;
        }

        if (!Tools::isEmpty($this->cms->meta_keywords)) {
            $metas['meta_keywords'] = $this->cms->meta_keywords;
        }

        $this->context->smarty->assign($metas);
    }
    
    public function initHeader()
    {
        parent::initHeader();
        $this->context->smarty->assign('content_only', !$this->cms->display_header);
    }

    public function setMedia()
    {
        parent::setMedia();

        if ($this->cms && $css_file = $this->cms->getCssFile($this->context->language->id)) {
            $this->addCSS($css_file);
        }
    }

    public function initContent()
    {
        parent::initContent();

        if ($this->cms && $this->cms_data) {
            $this->context->smarty->assign(array(
                'verp'          => $this->vt,
                'cms'           => $this->cms,
                'cms_data'      => $this->cms_data,
                'cms_template'  => _PS_MODULE_DIR_.$this->module->name.'/views/templates/front/'.$this->vt.'/cms.tpl',
                'landing'       => Configuration::get('ADCMS_LANDING', null, null, (int)$this->context->shop->id),
            ));

            if (version_compare('1.7.0.0', _PS_VERSION_, '>')) {
                $this->setTemplate($this->vt.'/cms_wrapper.tpl');
            } else {
                $template = 'cms_wrapper.tpl';

                if (!$this->cms->display_header) {
                    $template = 'content_only.tpl';
                } else {
                    if ($this->cms->column_left && !$this->cms->column_right) {
                        $template = 'with_left.tpl';
                    } elseif (!$this->cms->column_left && $this->cms->column_right) {
                        $template = 'with_right.tpl';
                    } elseif (!$this->cms->column_left && !$this->cms->column_right) {
                        $template = 'full_width.tpl';
                    } elseif ($this->cms->column_left && $this->cms->column_right) {
                        $template = 'with_both.tpl';
                    }
                }

                $this->setTemplate('module:advancedcms/views/templates/front/t17/'.$template);
            }
        }
    }
}
