/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2015 silbersaiten
 * @version   1.0.4
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

var AdvancedCmsBlockImage = {
	init: function()
	{
		cmsbuilder.click('div.adcms-gallery-image-select', function(){
			AdvancedCmsBlockImage.processGalleryImageClick($(this));
		});
		
		cmsbuilder.click('a.adcms-gallery-image-delete', function(){
			if (confirm(cmsbuilder.tr('Are you sure you want to delete this image?'))) {
				AdvancedCmsBlockImage.processGalleryImageDelete($(this).parents('.adcms-gallery-image-select:first'));
			}
		});

		cmsbuilder.click('a.adcms-gallery-image-config', function(){
			AdvancedCmsBlockImage.processGalleryImageInfo($(this).parents('.adcms-gallery-image-select:first'));
		});

		cmsbuilder.click('a.adcms-gallery-image-preview-close', function(e){
			AdvancedCmsBlockImage.processGalleryImageInfoClose($(this).parents('.adcms-gallery-image-select:first'));
		});

		cmsbuilder.mouseover('a.adcms-gallery-image-config', function(e){
			AdvancedCmsBlockImage.processGalleryImageInfo($(this).parents('.adcms-gallery-image-select:first'));
		});

		cmsbuilder.mouseleave('a.adcms-gallery-image-config', function(e){
			AdvancedCmsBlockImage.processGalleryImageInfoClose($(this).parents('.adcms-gallery-image-select:first'));
		});
	},
	
	localSave: function(containerIndex, blockIndex, fancyboxContent)
	{
		var imageUrl = fancyboxContent.find('input[name=image_url]').val(),
			collection = fancyboxContent.find('div.adcms-gallery-image-select-active'),
			id_lang,
			image;
		
		cmsbuilder.blockData[containerIndex].items[blockIndex].settings = {};
		cmsbuilder.blockData[containerIndex].items[blockIndex].content = {};
		cmsbuilder.blockData[containerIndex].items[blockIndex].settings['image_url'] = imageUrl;
		
		if (collection.length) {
			collection.each(function(){
				id_lang = $(this).data('languageId');
				image = $(this).data('imageName');
				
				if (!('lang_' + id_lang in cmsbuilder.blockData[containerIndex].items[blockIndex].content))
					cmsbuilder.blockData[containerIndex].items[blockIndex].content['lang_' + id_lang] = {id_lang: id_lang, value: image};
			});
		}
	},
	
	localLoad: function(containerIndex, blockIndex, fancyboxContents, blockData)
	{
		fancyboxContents.find('input[name=image_url]').val((typeof(blockData.settings.image_url) != 'undefined' && blockData.settings.image_url) ? blockData.settings.image_url : '');
	},
	
	localOpen: function(containerIndex, blockIndex, fancyboxContent)
	{
		fancyboxContent.find('input[name=adcms_image]').fileupload({
			maxNumberOfFiles: 1,
			formData: {
				'block_controller': 'AdvancedCmsBlockImage',
				'action': 'uploadGalleryImage'
            },
			add: function (e, data)
			{
				data.context = fancyboxContent.find('#uploadContext');
				
				var jqXHR = data.submit().success(function(result, textStatus, jqXHR){
					var json = JSON.parse(result);
					var status = json['status'];
					
					if (status == 'error')
					{
						data.context.addClass('error');
						
						if (typeof(json.message) != 'undefined')
							data.context.append($(document.createElement('div')).addClass('alert alert-danger upload_err clear').html(json.message));
					}
					else if (status == 'success')
					{
						if (typeof(json.filename) != 'undefined' && typeof(json.thumbnail) != 'undefined' && json.filename.length && json.thumbnail.length)
						{
							var galleries_collection = $('.adcms-gallery-lang-wrapper');
							
							if (galleries_collection.length)
							{
								galleries_collection.each(function(){
									$(this).prepend(
										$(document.createElement('div')).addClass('adcms-gallery-image-select').data('image-name', json.filename).data('language-id', $(this).attr('rel'))
										.append(
											$(document.createElement('img')).attr('src', json.thumbnail).addClass('thumbnail')
										)
										.append(
											$(document.createElement('a')).attr('href', '#').addClass('adcms-gallery-image-delete')
											.append($(document.createElement('i')).addClass('icon-remove'))
										)
									);
								});
							}
						}
					}
					
					setTimeout(function(){
						data.context.fadeOut('slow');
					}, 100000);
				});
			}
		});

		for (var i in cmsbuilder.blockData[containerIndex].items[blockIndex].content)
		{
			var image = cmsbuilder.blockData[containerIndex].items[blockIndex].content[i].value,
				lang  = cmsbuilder.blockData[containerIndex].items[blockIndex].content[i].id_lang,
				img = fancyboxContent.find('div.adcms-gallery-image-select[data-image-name="' + image + '"][data-language-id="' + lang + '"]');
			
			if (img.length)
				img.addClass('adcms-gallery-image-select-active');
		}
	},
	
	processGalleryImageClick: function(clickedLink)
	{
		var parent = clickedLink.parents('.adcms-gallery-lang-wrapper:first');
		
		if (parent.length)
		{
			var blockIndex = cmsbuilder.getBlockIndex(cmsbuilder.currentActiveBlock),
				containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(cmsbuilder.currentActiveBlock));
				
			if (!clickedLink.hasClass('.adcms-gallery-image-select-active'))
			{
				parent.find('.adcms-gallery-image-select').removeClass('adcms-gallery-image-select-active');
				
				clickedLink.addClass('adcms-gallery-image-select-active');
				
				if (typeof(cmsbuilder.blockData[containerIndex].items[blockIndex]) != 'undefined')
				{
					cmsbuilder.blockData[containerIndex].items[blockIndex].content['lang_' + parseInt(clickedLink.data('languageId'))] = {
						'id_lang': parseInt(clickedLink.data('languageId')),
						'value': clickedLink.data('imageName')
					};
				}
			}
		}
	},

	processGalleryImageInfo: function(galleryImage)
	{
		var blockIndex = cmsbuilder.getBlockIndex(cmsbuilder.currentActiveBlock),
			containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(cmsbuilder.currentActiveBlock)),
			imageName = galleryImage.data('imageName'),
			imageSize = galleryImage.data('imageSize'),
			galleryPath = galleryImage.closest('.adcms-gallery').data('galleryPath');

		$(galleryImage).append(
			$(document.createElement('div')).addClass('adcms-gallery-image-preview').css({'left':'15px', 'top':'115px'})
				.append(
					$(document.createElement('img')).attr('src', galleryPath+imageName)
				)
				.append(
					$(document.createElement('a')).attr('href', '#').addClass('adcms-gallery-image-preview-close')
					.append($(document.createElement('i')).addClass('icon-remove'))
				)
				.append(
					$(document.createElement('div')).addClass('adcms-gallery-image-preview-info')
					// .append($(document.createElement('span')).addClass('adcms-gallery-image-preview-info-title').html(imageName))
					.append($(document.createElement('span')).addClass('adcms-gallery-image-preview-info-size').html(imageSize))
				)
		);
	},

	processGalleryImageInfoClose: function(galleryImage)
	{
		galleryImage.find('.adcms-gallery-image-preview').remove();
	},
	
	processGalleryImageDelete: function(galleryImage)
	{
		var blockIndex = cmsbuilder.getBlockIndex(cmsbuilder.currentActiveBlock),
			containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(cmsbuilder.currentActiveBlock)),
			imageName = galleryImage.data('imageName');
			
		galleryImage.fadeOut('fast', function(){
			var img = $(this);
			
			$.post(
				window.location.href,
				{
					block_controller: 'AdvancedCmsBlockImage',
					action: 'deleteGalleryImage',
					params: {
						gallery_image: imageName
					}
				},
				function(data)
				{
					if (typeof(data.status) != 'undefined') {
						if (data.status == 'success') {
							img.remove();
							
							if (typeof(cmsbuilder.blockData[containerIndex].items[blockIndex]) != 'undefined') {
								for (var i in cmsbuilder.blockData[containerIndex].items[blockIndex].content) {
									if (cmsbuilder.blockData[containerIndex].items[blockIndex].content[i].value == imageName) {
										delete(cmsbuilder.blockData[containerIndex].items[blockIndex].content[i]);
									}
								}
							}
						} else {
							img.fadeIn('fast');
						}
					}
				},
				'json'
			);
		});
	},
}

$(function(){
	AdvancedCmsBlockImage.init();
});