/**
 * Advancedcms
 *
 * @category  Module
 * @author    silbersaiten <info@silbersaiten.de>
 * @support   silbersaiten <support@silbersaiten.de>
 * @copyright 2015 silbersaiten
 * @version   0.0.1
 * @link      http://www.silbersaiten.de
 * @license   See joined file licence.txt
 */

var cmsbuilder = {
	currentActiveRow: false,
	currentActiveBlock: false,
	blockData: {},
	onBlockOpen: [],
	onBlockLoad: [],
	onBlockSave: [],
	
	init: function()
	{
		$('#cms_container').parent('div').removeClass().addClass('col-lg-12');
		
		this.initStickyHeader();
		
		cmsbuilder.click('.cms-controller-menu', function(){
			cmsbuilder.processMenuClick($(this));
		});
		
		cmsbuilder.click('#containerDataSave', function(){
			cmsbuilder.processContainerClosed(function(){
				cmsbuilder.writeHiddenJson();
			});
			
			$.fancybox.close();
		});
		
		cmsbuilder.click('.cms-builder-row', function(){
			if (!$(this).hasClass('cms-builder-row-active'))
				cmsbuilder.setActiveContainer($(this));
		});
		
		cmsbuilder.click('a.cms-builder-container-delete', function(){
			cmsbuilder.setActiveContainer($(this).parents('div.cms-builder-row:first'), function(container){
				if (confirm(cmsbuilder.tr('Are you sure you want to delete this container? All it\'s blocks will also be deleted!')))
					cmsbuilder.setContainerDeleted(container);
			});
		});
		
		cmsbuilder.click('a.cms-builder-container-settings', function(){
			cmsbuilder.setActiveContainer($(this).parents('div.cms-builder-row:first'), function(container){
				cmsbuilder.displayContainerSettings(container);
			});
		});
		
		cmsbuilder.click('#blockDataSave', function(){
			cmsbuilder.processBlockClosed(function(){
				cmsbuilder.writeHiddenJson();
			});
			
			$.fancybox.close();
		});
		
		cmsbuilder.click('.cms-builder-block', function(){
			if (!$(this).hasClass('cms-builder-block-active'))
				cmsbuilder.setActiveBlock($(this));
		});
		
		cmsbuilder.click('a.cms-builder-block-delete', function(){
			cmsbuilder.setActiveBlock($(this).parents('div.cms-builder-block:first'), function(block){
				if (confirm(cmsbuilder.tr('Are you sure you want to delete this block?')))
					cmsbuilder.setBlockDeleted(block);
			});
		});
		
		cmsbuilder.click('a.cms-builder-block-settings', function(){
			cmsbuilder.setActiveBlock($(this).parents('div.cms-builder-block:first'), function(block){
				cmsbuilder.displayBlockSettings(block);
			});
		});

		cmsbuilder.click('a.cms-builder-block-duplicate', function(){
			var block = $(this).closest('.cms-builder-block'),
				container = cmsbuilder.getBlocksContainer(block);
			
			cmsbuilder.duplicateBlock(block, container);
		});

		cmsbuilder.click('a.cms-builder-container-duplicate', function(){
			var container = $(this).closest('.cms-builder-row');
			cmsbuilder.duplicateContainer(container);
		});

		cmsbuilder.click('a.cms-builder-block-active', function(){
			var block = $(this).closest('.cms-builder-block');
			
			cmsbuilder.changeActiveBlock(block);
		});

		cmsbuilder.click('a.cms-builder-container-active', function(){
			var container = $(this).closest('.cms-builder-row');
			
			cmsbuilder.changeActiveContainer(container);
		});
	
		$('#cms_contents').sortable({
			items: 'div.cms-builder-row',
			handle: '.cms-builder-container-drag',
			start: function(event, ui)
			{
				cmsbuilder.setActiveContainer(ui.item);
			},
			stop: function (event, ui)
			{
				cmsbuilder.recalculatePositions();
			}
		});
		
		cmsbuilder.loadCmsData();
		cmsbuilder.writeHiddenJson();
    },
    
	initStickyHeader: function()
	{
		if (!!$('#cms_tools_container').offset()) {
			var container = $('#cms_container'),
				floater = $('#cms_tools_container'),
				floaterHeight = floater.outerHeight(),
				floaterMargin = floaterHeight + 40;
			  
			floater.css('top', -(floaterMargin)+'px');
			container.css({'margin-top': floaterMargin+'px', 'margin-bottom': floaterMargin+'px'});
	  
			var containerTopOffset = $('#cms_container').offset().top,
				floaterTopOffset = $('#cms_tools_container').offset().top,
				floaterTopCss = parseInt($('#cms_tools_container').css('top'), 10),
				containerHeight = $('#cms_container').outerHeight(),
				headerHeight = $('.navbar-header:first').height() + $('.page-head:first').outerHeight() + 7;
	  
			$(window).scroll(function(){
				var windowTop = $(window).scrollTop() + headerHeight;
			
				if (floaterTopOffset < windowTop) {
					containerHeight = container.height();
			
					if ((windowTop - containerTopOffset) <= containerHeight + 40) {
						floater.css({top: (windowTop - containerTopOffset)});
					}
				} else {
					floater.css({top: floaterTopCss});
				}
			});
	  
		}	
	},
	
	registerEvent: function(event, selector, callback)
	{
		$(document).on(event, selector, function(e){
			e.preventDefault();
			
			if (typeof(callback) == 'function')
				callback.call(this, e);
			
			return false;
		});
	},
	
	click: function(selector, callback)
	{
		this.registerEvent('click', selector, callback);
	},

	mouseover: function(selector, callback)
	{
		this.registerEvent('mouseover', selector, callback);
	},

	mouseleave: function(selector, callback)
	{
		this.registerEvent('mouseleave', selector, callback);
	},
	
	change: function(selector, callback)
	{
		this.registerEvent('change', selector, callback);
	},
    
    registerSortable: function(container)
    {
		container.addClass('cms-sortable');
		
		container.sortable({
			items: 'div.cms-builder-block:not(.unsortable)',
			placeholder: 'sortable-placeholder',
			handle: '.drag-handle',
			connectWith: '.cms-builder-row',
			start: function (event, ui)
			{
				ui.placeholder.addClass(ui.helper.attr('class'));
				ui.placeholder.height(ui.helper.height());
				
				cmsbuilder.setActiveBlock(ui.item);
				cmsbuilder.setActiveContainer(cmsbuilder.getBlocksContainer(ui.item));
			},
			stop: function (event, ui)
			{
				cmsbuilder.recalculatePositions();
			}
		});
		
		container.disableSelection();
    },
    
    processMenuClick: function(btn)
    {
		switch (btn.attr('rel'))
		{
			case 'container':
				cmsbuilder.createContainer(false, true, false, true);
			break;
			default:
				if (!cmsbuilder.currentActiveRow) {
					$.fancybox.open([{
						content: '<div class="bootstrap"><div class="alert alert-danger">' + cmsbuilder.tr('Please add or select a container for this block first.') + '</div></div>',
						autoSize: false,
						autoHeight: true,
						fitToView: false,
						minHeight: 50,
						width: 500,
						helpers: {
							overlay: {
								locked: false
							}
						}
					}]);
				} else {
					cmsbuilder.createBlock(btn.attr('rel'), btn.data('controller'), false, false, true, false, false, function(block){
						cmsbuilder.setActiveBlock(block);
					});
				}
			break;
		}
    },
    
    getBlocksContainer: function(block)
    {
		var container = block.parents('.cms-builder-row:first');
		
		return container.length ? container : false;
    },
	
	getContainerBlocks: function(container)
	{
		var blocks = container.find('.cms-builder-block');
		
		if (blocks.length)
			return blocks;
		
		return false;
	},
    
    processBlockClick: function(block)
    {
		var container = cmsbuilder.getBlocksContainer(block);
		
		if (container)
			cmsbuilder.setActiveContainer(container);
		
		cmsbuilder.setActiveBlock(block);
    },
    
    setActiveContainer: function(container, callback)
    {
		$('.cms-builder-row').removeClass('cms-builder-row-active');
		container.addClass('cms-builder-row-active');
		
		cmsbuilder.currentActiveRow = container;
		
		if (callback)
			callback.call(cmsbuilder, container);
    },
    
    setActiveBlock: function(block, callback)
    {
		if (!cmsbuilder.currentActiveBlock || cmsbuilder.currentActiveBlock.attr('id') != block.id) {
			$('.cms-builder-block').removeClass('cms-builder-block-active');
			block.addClass('cms-builder-block-active');
		}
		
		cmsbuilder.currentActiveBlock = block;
		
		if (callback)
			callback.call(cmsbuilder, block);
    },
    
    createContainer: function(id, active, containerIndex, recalculatePositions)
    {
		id = id || 0;
		active = active || false;
		containerIndex = containerIndex || cmsbuilder.getNextContainerId();
		recalculatePositions = recalculatePositions || false;
		
		var container = $(document.createElement('div'))
				.addClass('row cms-builder-row sortable')
				.data('container-id', id)
				.attr('id', 'cms_builder_container_' + containerIndex);
		
		if (active)
			cmsbuilder.setActiveContainer(container);
			
		container.append(
			$(document.createElement('a')).attr('href', '#').addClass('cms-builder-container-control cms-builder-container-delete')
				.append(
					$(document.createElement('i')).addClass('icon-remove')
				)
		).append(
			$(document.createElement('a')).attr('href', '#').addClass('cms-builder-container-control cms-builder-container-active')
				.append(
					$(document.createElement('i')).addClass('icon-check')
				)
		).append(
			$(document.createElement('a')).attr('href', '#').addClass('cms-builder-container-control cms-builder-container-settings')
				.append(
					$(document.createElement('i')).addClass('icon-cogs')
				)
		).append(
			$(document.createElement('a')).attr('href', '#').addClass('cms-builder-container-control cms-builder-container-duplicate')
				.append(
					$(document.createElement('i')).addClass('icon-copy')
				)
		).append(
			$(document.createElement('a')).attr('href', '#').addClass('cms-builder-container-control cms-builder-container-drag')
				.append(
					$(document.createElement('i')).addClass('icon-arrows')
				)
		);
		
		$('#cms_contents').append(container);
		
		cmsbuilder.blockData[containerIndex] = {
			'id_container': id,
			'css_class': 'row',
			'active': 1,
			'deleted': 0,
			'items': {}
		};
		
		if (recalculatePositions)
			cmsbuilder.recalculatePositions();
		
		if (active) {
			cmsbuilder.currentActiveRow = container;
			
			cmsbuilder.registerSortable(cmsbuilder.currentActiveRow);
		}
    },
    
    createBlock: function(blocktype, controller, id, blockIndex, recalculatePositions, reference, css_class, callback)
    {
		if (cmsbuilder.currentActiveRow) {
			id = id || 0;
			blockIndex = blockIndex || cmsbuilder.getNextBlockId();
			recalculatePositions = recalculatePositions || false;
			reference = reference || false;
			css_class = css_class || 'col-md-4';
			
			var containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.currentActiveRow),
				blockTitle = $('.cms-controller-menu[rel=' + blocktype + ']').text();
			
			if (!(parseInt(blockIndex) in cmsbuilder.blockData[containerIndex].items)) {
				cmsbuilder.blockData[containerIndex].items[blockIndex] = {
					'id': id,
					'type': blocktype,
					'controller': controller,
					'css_class': css_class,
					'reference': reference,
					'active': 1,
					'deleted': 0,
					'settings': {},
					'content': {}
				};
			}
			
			var block = $(document.createElement('div'))
					.addClass('cms-builder-block')
					.data('block-type', blocktype)
					.data('block-controller', controller)
					.data('block-id', id)
					.attr({'id': 'cms_builder_block_' + blockIndex, 'data-block-type': blocktype, 'data-block-controller': controller})
					.addClass(css_class)
			.append(
				$(document.createElement('div')).addClass('block-contents')
				.append(
					$(document.createElement('h3'))
					.text(blockTitle)
				)
				.append(reference ? $(document.createElement('h4')).text(reference) : null)
				.append(
					$(document.createElement('a'))
					.attr('href', '#')
					.addClass('cms-builder-block-control drag-handle')
					.append(
						$(document.createElement('i')).addClass('icon-arrows')
					)
				)
				.append(
					$(document.createElement('a'))
					.attr('href', '#')
					.addClass('cms-builder-block-control cms-builder-block-active')
					.append(
						$(document.createElement('i')).addClass('icon-check')
					)
				)
				.append(
					$(document.createElement('a'))
					.attr('href', '#')
					.addClass('cms-builder-block-control cms-builder-block-settings')
					.append(
						$(document.createElement('i')).addClass('icon-cogs')
					)
				)
				.append(
					$(document.createElement('a'))
					.attr('href', '#')
					.addClass('cms-builder-block-control cms-builder-block-duplicate')
					.append(
						$(document.createElement('i')).addClass('icon-copy')
					)
				)
				.append(
					$(document.createElement('a'))
					.attr('href', '#')
					.addClass('cms-builder-block-control cms-builder-block-delete')
					.append(
						$(document.createElement('i')).addClass('icon-remove')
					)
				)
			);
			
			cmsbuilder.currentActiveRow.append(block);
			
			if (recalculatePositions)
				cmsbuilder.recalculatePositions();
			
			if (callback)
				callback.call(this, block);
		}
    },

    changeActiveBlock: function(block)
    {
    	var blockIndex = cmsbuilder.getBlockIndex(block),
			containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(block));

    	if (block.hasClass('cms-builder-block-disabled')) {
    		cmsbuilder.changeActiveBlockClass(block, 1);
    		cmsbuilder.blockData[containerIndex].items[blockIndex].active = 1;
    	} else {
    		cmsbuilder.changeActiveBlockClass(block, 0);
    		cmsbuilder.blockData[containerIndex].items[blockIndex].active = 0;
    	}

    	cmsbuilder.writeHiddenJson();
    },

    changeActiveBlockClass: function(block, active)
    {
    	var button = block.find('.cms-builder-block-active');

    	if (active == 1) {
    		block.removeClass('cms-builder-block-disabled');
    		button.removeClass('active-dis');
    	} else {
    		block.addClass('cms-builder-block-disabled');
    		button.addClass('active-dis');
    	}
    },

    changeActiveContainer: function(container)
    {
    	var containerIndex = cmsbuilder.getContainerIndex(container);

    	if (container.hasClass('cms-builder-row-disabled')) {
    		cmsbuilder.changeActiveContainerClass(container, 1);
    		cmsbuilder.blockData[containerIndex].active = 1;
    	} else {
    		cmsbuilder.changeActiveContainerClass(container, 0);
    		cmsbuilder.blockData[containerIndex].active = 0;
    	}

    	cmsbuilder.writeHiddenJson();
    },

    changeActiveContainerClass: function(container, active)
    {
    	var button = container.find('.cms-builder-container-active');

    	if (active == 1) {
    		container.removeClass('cms-builder-row-disabled');
    		button.removeClass('active-dis');
    	} else {
    		container.addClass('cms-builder-row-disabled');
    		button.addClass('active-dis');
    	}
    },

    duplicateBlock: function(block, container)
    {
    	var blockIndex = cmsbuilder.getBlockIndex(block),
			containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(block));
		
		if (container) {
			cmsbuilder.setActiveContainer(container);
		}

		cmsbuilder.createBlock(block.data('blockType'), block.data('blockController'), false, false, true, cmsbuilder.blockData[containerIndex].items[blockIndex].reference, cmsbuilder.getBlockClasses(block), function(block){
			cmsbuilder.setActiveBlock(block);

			var blockIndexNew = cmsbuilder.getBlockIndex(cmsbuilder.currentActiveBlock),
				containerIndexNew = cmsbuilder.getContainerIndex(container);

			if (cmsbuilder.blockData[containerIndex].items[blockIndex].active == 0) {
				cmsbuilder.changeActiveBlockClass(block, 0);
			}
			cmsbuilder.blockData[containerIndexNew].items[blockIndexNew].active = cmsbuilder.blockData[containerIndex].items[blockIndex].active;
			cmsbuilder.blockData[containerIndexNew].items[blockIndexNew].name = cmsbuilder.blockData[containerIndex].items[blockIndex].name;
			cmsbuilder.blockData[containerIndexNew].items[blockIndexNew].settings = cmsbuilder.blockData[containerIndex].items[blockIndex].settings;
			cmsbuilder.blockData[containerIndexNew].items[blockIndexNew].content = cmsbuilder.blockData[containerIndex].items[blockIndex].content;

			cmsbuilder.writeHiddenJson();
		});
    },

    duplicateContainer: function(container)
    {
    	cmsbuilder.createContainer(false, true, false, true);

    	var containerIndex = cmsbuilder.getContainerIndex(container),
    		containerIndexNew = cmsbuilder.getContainerIndex(cmsbuilder.currentActiveRow);

    	cmsbuilder.blockData[containerIndexNew].active = cmsbuilder.blockData[containerIndex].active;
		cmsbuilder.blockData[containerIndexNew].css_class = cmsbuilder.blockData[containerIndex].css_class;
		cmsbuilder.blockData[containerIndexNew].css_class_w = cmsbuilder.blockData[containerIndex].css_class_w;
		cmsbuilder.blockData[containerIndexNew].divider = cmsbuilder.blockData[containerIndex].divider;
		cmsbuilder.blockData[containerIndexNew].wrapper = cmsbuilder.blockData[containerIndex].wrapper;

		$.each(cmsbuilder.blockData[containerIndex].items, function(i, bl) {
			var block = $('#cms_builder_block_'+i);
			cmsbuilder.duplicateBlock(block, cmsbuilder.currentActiveRow);
		});

		cmsbuilder.writeHiddenJson();

    },
	
	setContainerDeleted: function(container)
	{
		var containerIndex = cmsbuilder.getContainerIndex(container),
			blocks = cmsbuilder.getContainerBlocks(container);
			
		cmsbuilder.blockData[containerIndex].deleted = 1;
			
		if (blocks) {
			blocks.each(function(){
				var blockIndex = cmsbuilder.getBlockIndex($(this));
				
				cmsbuilder.blockData[containerIndex].items[blockIndex].deleted = 1;
				
				$(this).hide();
			});
		}
		
		container.hide();
		cmsbuilder.writeHiddenJson();
	},
	
	setBlockDeleted: function(block)
	{
		var blockIndex     = cmsbuilder.getBlockIndex(block),
			containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(block));
			
		cmsbuilder.blockData[containerIndex].items[blockIndex].deleted = 1;
		
		block.hide();
		
		cmsbuilder.writeHiddenJson();
	},
	
    displayContainerSettings: function(container)
    {
		$.fancybox.open(
			[{
				autoScale: true,
				autoSize: false,
				width: 895,
				minHeight: 30,
				content: '<div id="cms_content_loading"><i class="icon-spinner icon-spin icon-large"></i></div>',
				padding: 0,
				margin: 0,
				afterShow: function(){
					$.post(
						window.location.href,
						{
							action: 'getContainerData',
							'id_ad_cms': 0
						},
						function(data)
						{
							if (typeof(data.html) != 'undefined') {
								var containerIndex = cmsbuilder.getContainerIndex(container),
								fancyboxContent = $('.fancybox-inner');
								
								$('.fancybox-inner').html(data.html);
								
								$('.fancybox-inner').find('input[name=css_class]').val(cmsbuilder.getContainerClasses(container));
								
								cmsbuilder.containerDataLocalLoad(containerIndex, fancyboxContent);
								
								$.fancybox.update();
							}
						},
						'json'
					);
				},
				helpers: {
					overlay: {
						locked: false,
						closeClick: false
					}
				}
			}],
			{
				padding: 0
			}
		);
    },
    
    displayBlockSettings: function(block)
    {
		var blockId = parseInt(block.data('blockId'));
		
		if (isNaN(blockId) || blockId < 0) {
			blockId = 0;
		}
		
		$.fancybox.open(
			[{
				autoScale: true,
				autoSize: false,
				width: 895,
				minHeight: 30,
				content: '<div id="cms_content_loading"><i class="icon-spinner icon-spin icon-large"></i></div>',
				padding: 0,
				margin: 0,
				afterShow: function(){
					$.post(
						window.location.href,
						{
							action: 'getBlockData',
							'id_ad_cms': $('input[name=id_ad_cms]').val(),
							'id_ad_cms_block': blockId,
							'block_type': block.data('blockType')
						},
						function(data)
						{
							if (typeof(data.html) != 'undefined') {
								var blockType = block.data('blockType'),
									blockIndex = cmsbuilder.getBlockIndex(block),
									containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(block)),
									fancyboxContent = $('.fancybox-inner'),
									blockObj = cmsbuilder.blockData[containerIndex].items[blockIndex];
								
								$('.fancybox-inner').html(data.html);
								
								$('.fancybox-inner').find('input[name=css_class]').val(cmsbuilder.getBlockClasses(block));
								
								if (cmsbuilder.blockData[containerIndex].items[blockIndex].reference) {
									$('.fancybox-inner').find('input[name=block_reference]').val(cmsbuilder.blockData[containerIndex].items[blockIndex].reference);
								}
								
								cmsbuilder.blockDataLocalLoad(containerIndex, blockIndex, blockType, fancyboxContent);
								
								if (typeof(blockObj.controller) != 'undefined' && typeof(window[blockObj.controller]) != 'undefined') {
									if (typeof(window[blockObj.controller].localOpen) == 'function') {
										window[blockObj.controller].localOpen.call(cmsbuilder, containerIndex, blockIndex, fancyboxContent);
									}
								} else {
									var openProcessor = cmsbuilder.getBlockOpenProcessor(blockType);
									
									if (openProcessor)
										openProcessor.call(cmsbuilder, containerIndex, blockIndex, fancyboxContent);
								}
								
								$.fancybox.update();
							}
						},
						'json'
					);
				},
				helpers: {
					overlay: {
						locked: false,
						closeClick: false
					}
				}
			}],
			{
				padding: 0
			}
		);
    },
	
	getContainerClasses: function(container)
	{
		var classes = container.attr('class').split(' '),
			prepared = [];
		
		if (classes.length) {
			for (var i in classes) {
				if (classes[i].trim() != 'cms-builder-row' && classes[i].trim() != 'sortable' && classes[i].trim() != 'cms-builder-row-disabled')
					prepared.push(classes[i].trim());
			}
			
			return prepared.join(' ');
		}
		
		return false;
	},
    
    getBlockClasses: function(block)
    {
		var classes = block.attr('class').split(' '),
			prepared = [];
		
		if (classes.length) {
			for (var i in classes) {
				if (classes[i].trim() != 'cms-builder-block' && classes[i].trim() != 'cms-builder-block-active' && classes[i].trim() != 'cms-builder-block-disabled')
					prepared.push(classes[i].trim());
			}
			
			return prepared.join(' ');
		}
		
		return false;
    },
    
    getClassesFromString: function(str)
    {
		var prepared = [],
			classes = str.split(' ');
		
		if (classes.length) {
			for (var i in classes) {
				if (classes[i].trim() != 'cms-builder-block'
					&& classes[i].trim() != 'cms-builder-block-active'
					&& classes[i].trim() != 'cms-builder-block-disabled'
					&& classes[i].trim() != 'cms-builder-row'
					&& classes[i].trim() != 'cms-builder-row-active'
					&& classes[i].trim() != 'cms-builder-row-disabled'
					&& classes[i].trim() != 'sortable')
					prepared.push(classes[i].trim());
			}
			
			return prepared;
		}
		
		return false;
    },
    
    processBlockClosed: function(callback)
    {
		var fancybox = $('.fancybox-inner');
		
		if (fancybox.length) {
			var css_class = fancybox.find('input[name=css_class]').val(),
				blockReference = fancybox.find('input[name=block_reference]').val(),
				active = parseInt(fancybox.find('input[name=pactive]:checked').val()),
				new_block_class = 'cms-builder-block cms-builder-block-active ' + (!active ? 'cms-builder-block-disabled ' : ''),
				blockIndex = cmsbuilder.getBlockIndex(cmsbuilder.currentActiveBlock),
				containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.getBlocksContainer(cmsbuilder.currentActiveBlock)),
				blockType = cmsbuilder.currentActiveBlock.data('blockType');
				blockDisplayName = {};
				
			var blockName = fancybox.find('input[name^=block_display_name_]');
		
			if (blockName.length) {
				cmsbuilder.blockData[containerIndex].items[blockIndex].content = {};
				
				blockName.each(function(){
					var n = cmsbuilder.getLanguageInfoFromInputName($(this).attr('name'));
					
					if (n) {
						blockDisplayName['lang_' + n.id_lang] = {'id_lang': n.id_lang, 'value': $(this).val()};
					}
				});
			}
	
			if (!(blockIndex in cmsbuilder.blockData[containerIndex].items))
				cmsbuilder.blockData[containerIndex].items[blockIndex] = {
					'id': 0,
					'type': blockType,
					'active': active,
					'reference': blockReference,
					'css_class': 'col-md-4',
					'name': blockDisplayName,
					'settings': {},
					'content': {}
				};
				
			if (blockReference.length) {
				var existing_reference = cmsbuilder.currentActiveBlock.find('h4');
				
				if (existing_reference.length)
					existing_reference.text(blockReference);
				else
					$(document.createElement('h4')).text(blockReference).insertAfter(cmsbuilder.currentActiveBlock.find('h3'));
			} else {
				cmsbuilder.currentActiveBlock.find('h4').remove();
			}
			
			cmsbuilder.blockData[containerIndex].items[blockIndex].name = blockDisplayName;
			cmsbuilder.blockData[containerIndex].items[blockIndex].css_class = css_class.length ? css_class : 0;
			cmsbuilder.blockData[containerIndex].items[blockIndex].active = active;
			cmsbuilder.blockData[containerIndex].items[blockIndex].reference = blockReference;
			
			var blockObj = cmsbuilder.blockData[containerIndex].items[blockIndex];
			
			if (css_class.length)
				new_block_class+= cmsbuilder.getClassesFromString(css_class).join(' ');
			
			if (typeof(blockObj.controller) != 'undefined' && typeof(window[blockObj.controller]) != 'undefined') {
				if (typeof(window[blockObj.controller].localSave) == 'function') {
					window[blockObj.controller].localSave.call(cmsbuilder, containerIndex, blockIndex, fancybox);
				}
			} else {
				var saveProcessor = cmsbuilder.getBlockSaveProcessor(blockType);
				
				if (saveProcessor)
					saveProcessor.call(cmsbuilder, containerIndex, blockIndex, fancybox);
			}
			
			cmsbuilder.currentActiveBlock.removeClass().addClass(new_block_class);
			
			if (callback)
				callback.call();
		}
    },
	
	processContainerClosed: function(callback)
	{
		var fancybox = $('.fancybox-inner');
		
		if (fancybox.length) {
			var css_class = fancybox.find('input[name=css_class]').val(),
				css_class_w = fancybox.find('input[name=css_class_w]').val(),
				active = parseInt(fancybox.find('input[name=cactive]:checked').val()),
				divider = parseInt(fancybox.find('input[name=cdivider]:checked').val()),
				wrapper = parseInt(fancybox.find('input[name=cwrapper]:checked').val()),
				new_container_class = 'cms-builder-row cms-builder-row-active ' + (!active ? 'cms-builder-row-disabled ' : ''),
				containerIndex = cmsbuilder.getContainerIndex(cmsbuilder.currentActiveRow);

			cmsbuilder.blockData[containerIndex].css_class = css_class.length ? css_class : 0;
			cmsbuilder.blockData[containerIndex].css_class_w = css_class_w.length ? css_class_w : '';
			cmsbuilder.blockData[containerIndex].active = active;
			cmsbuilder.blockData[containerIndex].divider = divider;
			cmsbuilder.blockData[containerIndex].wrapper = wrapper;
			
			if (css_class.length)
				new_container_class+= cmsbuilder.getClassesFromString(css_class).join(' ');
			
			cmsbuilder.currentActiveRow.removeClass().addClass(new_container_class);
			
			if (callback)
				callback.call();
		}
	},
    
    getLanguageInfoFromInputName: function(inputName)
    {
		var r = (inputName.match(/(.*)\_(\d+)$/));
		
		if (typeof(r) == 'object' && r.length)
			return {'name': r[1], 'id_lang': parseInt(r[2])};
		
		return false;
    },
	
	containerDataLocalLoad: function(containerIndex, fancyboxContents)
	{
		if (typeof(cmsbuilder.blockData[containerIndex]) != 'undefined') {
			fancyboxContents.find('input[name=css_class]').val(cmsbuilder.blockData[containerIndex].css_class);
			fancyboxContents.find('input[name=css_class_w]').val(cmsbuilder.blockData[containerIndex].css_class_w);
			fancyboxContents.find('input[id=cactive_' + (cmsbuilder.blockData[containerIndex].active ? 'on' : 'off') + ']').prop('checked', true);
			fancyboxContents.find('input[id=cdivider_' + (cmsbuilder.blockData[containerIndex].divider ? 'on' : 'off') + ']').prop('checked', true);
			fancyboxContents.find('input[id=cwrapper_' + (cmsbuilder.blockData[containerIndex].wrapper ? 'on' : 'off') + ']').prop('checked', true);
		}
	},
    
    blockDataLocalLoad: function(containerIndex, blockIndex, blockType, fancyboxContents)
    {
		if (typeof(cmsbuilder.blockData[containerIndex].items[blockIndex]) != 'undefined') {
			var blockObj = cmsbuilder.blockData[containerIndex].items[blockIndex];
			
			fancyboxContents.find('input[name=css_class]').val(cmsbuilder.blockData[containerIndex].items[blockIndex].css_class);
			fancyboxContents.find('input[id=pactive_' + (cmsbuilder.blockData[containerIndex].items[blockIndex].active ? 'on' : 'off') + ']').prop('checked', true);
			
			if (typeof(cmsbuilder.blockData[containerIndex].items[blockIndex].name) != 'undefined') {
				for (var i in cmsbuilder.blockData[containerIndex].items[blockIndex].name) {
					var input = fancyboxContents.find('input[name=block_display_name_'+(cmsbuilder.blockData[containerIndex].items[blockIndex].name[i].id_lang)+']');
					
					if (input.length) {
						input.val(cmsbuilder.blockData[containerIndex].items[blockIndex].name[i].value);
					}
				}
			}
			
			if (typeof(blockObj.controller) != 'undefined' && typeof(window[blockObj.controller]) != 'undefined') {
				if (typeof(window[blockObj.controller].localLoad) == 'function') {
					window[blockObj.controller].localLoad.call(cmsbuilder, containerIndex, blockIndex, fancyboxContents, cmsbuilder.blockData[containerIndex].items[blockIndex]);
				}
			} else {
				var loadProcessor = cmsbuilder.getBlockLoadProcessor(blockType);

				if (loadProcessor)
					loadProcessor.call(cmsbuilder, containerIndex, blockIndex, fancyboxContents, cmsbuilder.blockData[containerIndex].items[blockIndex]);
			}
		}
    },
    
    getNextBlockId: function()
    {
		var collection = $('.cms-builder-block'),
			block_id = 0;
		
		if (collection.length) {
			collection.each(function(){
				if (parseInt($(this).attr('id').split('_')[3]) > block_id)
					block_id = parseInt($(this).attr('id').split('_')[3]);
			});
		}
		
		return (block_id + 1);
    },
    
    getNextContainerId: function()
    {
		var collection = $('.cms-builder-row'),
			container_id = 0;
		
		if (collection.length) {
			collection.each(function(){
				if (parseInt($(this).attr('id').split('_')[3]) > container_id) {
					container_id = parseInt($(this).attr('id').split('_')[3]);
				}
			});
		}
	
		return (container_id + 1);
    },
    
    getBlockIndex: function(block)
    {
		return parseInt(block.attr('id').split('_')[3]);
    },
    
    getContainerIndex: function(container)
    {
		return parseInt(container.attr('id').split('_')[3]);
    },
	
	recalculateBlockContainers: function()
	{
		var containers = $('#cms_contents .cms-builder-row'),
			containerBlocks = false,
			containerIndex = false,
			blockIndex = false,
			containerPosition = 0,
			containerLocated = false,
			blockPosition = 0,
			blockContainerIndex = [];
			
		if (containers.length) {
			containers.each(function(){
				containerIndex = cmsbuilder.getContainerIndex($(this));
				
				containerBlocks = cmsbuilder.getContainerBlocks($(this));
				
				if (containerBlocks) {
					containerBlocks.each(function(){
						for (var c in cmsbuilder.blockData) {
							for (var i in cmsbuilder.blockData[c].items) {
								blockIndex = cmsbuilder.getBlockIndex($(this));
								
								if (blockIndex == i) {
									if (typeof(blockContainerIndex[containerIndex]) == 'undefined')
										blockContainerIndex[containerIndex] = [];
									
									blockContainerIndex[containerIndex].push({'index': blockIndex, 'item': cmsbuilder.blockData[c].items[i]});
								}
							}
						}
					});
				}
			});
			
			if (blockContainerIndex.length) {
				for (var c in cmsbuilder.blockData)
					cmsbuilder.blockData[c].items = {};
					
				for (var c in cmsbuilder.blockData)
					for (var i in blockContainerIndex[c])
						cmsbuilder.blockData[c].items[blockContainerIndex[c][i].index] = blockContainerIndex[c][i].item;
			}
		}
	},
    
    recalculatePositions: function()
    {
		cmsbuilder.recalculateBlockContainers();
		
		var containers = $('#cms_contents .cms-builder-row'),
			containerBlocks = false,
			containerIndex = false,
			blockIndex = false,
			containerPosition = 0,
			blockPosition = 0;
		
		if (containers.length) {
			containers.each(function(){
				containerIndex = cmsbuilder.getContainerIndex($(this));
				
				cmsbuilder.blockData[containerIndex].position = containerPosition;
				
				containerBlocks = cmsbuilder.getContainerBlocks($(this));
				
				if (containerBlocks) {
					containerBlocks.each(function(){
						blockIndex = cmsbuilder.getBlockIndex($(this));
						
						cmsbuilder.blockData[containerIndex].items[blockIndex].position = blockPosition;
						
						blockPosition++;
					});
					
					blockPosition = 0;
				}
				
				containerPosition++;
			});
		}
		
		cmsbuilder.writeHiddenJson();
    },
    
    writeHiddenJson: function()
    {
		var v = JSON.stringify(cmsbuilder.blockData);
		
		$('textarea[name=form_json]').text(v);
    },
    
    loadCmsData: function()
    {
		if (typeof(blockData) != 'undefined') {
			var containerSort = [],
				blockSort = {};
			
			for (var i in blockData) {
				containerSort.push({key: i, value: blockData[i].position});
				
				for (var y in blockData[i].items) {
					if (!(i in blockSort))
						blockSort[i] = [];
					
					blockSort[i].push({key: y, value: blockData[i].items[y].position});
				}
			}
			
			containerSort.sort(function(x,y){return x.value - y.value});
			
			for (var i in blockSort)
				blockSort[i].sort(function(x,y){return x.value - y.value});
			
			for (i = 0; i < containerSort.length; i++) {
				var container = blockData[containerSort[i].key];
				
				cmsbuilder.createContainer(container.id_container, true, containerSort[i].key);
				
				cmsbuilder.currentActiveRow.removeClass().addClass(container.css_class).addClass('cms-builder-row sortable');
				
				if (!container.active) {
					cmsbuilder.currentActiveRow.addClass('cms-builder-row-disabled')
					cmsbuilder.changeActiveContainerClass(cmsbuilder.currentActiveRow, 0);
				}
				
				if (typeof(blockSort[containerSort[i].key]) != 'undefined') {
					for (y = 0; y < blockSort[containerSort[i].key].length; y++) {
						var block = container.items[blockSort[containerSort[i].key][y].key];

						cmsbuilder.createBlock(block.type, block.controller, block.id, blockSort[containerSort[i].key][y].key, false, block.reference, block.css_class, function(createdBlock){
							cmsbuilder.setActiveBlock(createdBlock);
							
							if (block.deleted)
								cmsbuilder.setBlockDeleted(createdBlock);
								
							if (!block.active) {
								createdBlock.addClass('cms-builder-block-disabled');
								cmsbuilder.changeActiveBlockClass(createdBlock, 0);
							}
						});
					}
				}
			}
			
			cmsbuilder.blockData = blockData;
			
			if (typeof(blockErrors) != 'undefined') {
				var erroneousContainer;
				var erroneousBlock;
				
				for (var i in blockErrors) {
					erroneousContainer = $('.cms-builder-row').eq(blockErrors[i].container_index);
					
					if (erroneousContainer.length) {
						erroneousBlock = erroneousContainer.find($('.cms-builder-block')).eq(blockErrors[i].block_index);
						
						if (erroneousBlock.length) {
							erroneousBlock.addClass('cms-builder-block-error');
							erroneousBlock.find('.block-contents').append(
								$(document.createElement('div')).addClass('alert alert-danger adcms-block-error').html(blockErrors[i].value)
							);
						}
					}
				}
			}
		}
    },
    
    tr: function(str)
    {
		if (typeof(adcmsTrans) != 'undefined' && str in adcmsTrans)
			return adcmsTrans[str];
		
		return str;
    },
	
	registerLoadProcessor: function(blockType, callback)
	{
		cmsbuilder.onBlockLoad[blockType] = callback;
	},
	
	registerOpenProcessor: function(blockType, callback)
	{
		cmsbuilder.onBlockOpen[blockType] = callback;
	},
	
	registerSaveProcessor: function(blockType, callback)
	{
		cmsbuilder.onBlockSave[blockType] = callback;
	},
	
	getBlockLoadProcessor: function(blockType)
	{
		return typeof(cmsbuilder.onBlockLoad[blockType]) != 'undefined' ? cmsbuilder.onBlockLoad[blockType] : false;
	},
	
	getBlockOpenProcessor: function(blockType)
	{
		return typeof(cmsbuilder.onBlockOpen[blockType]) != 'undefined' ? cmsbuilder.onBlockOpen[blockType] : false;
	},
	
	getBlockSaveProcessor: function(blockType)
	{
		return typeof(cmsbuilder.onBlockSave[blockType]) != 'undefined' ? cmsbuilder.onBlockSave[blockType] : false;
	}
}

$(function(){
    cmsbuilder.init();

	$('body.adminadvancedcmssettings').keydown(function(e){
		if ($(this).find('.fancybox-opened').length == 1) {
			if (e.ctrlKey && e.keyCode === 13) {
				cmsbuilder.processBlockClosed(function(){
					cmsbuilder.writeHiddenJson();
				});				
				$.fancybox.close();
			}
		} else {
			if (e.ctrlKey && e.altKey && e.keyCode === 13) {
				$('#ad_cms_form').submit();
			}
		}
	});

	$('.alert.adcms-shortcuts h4').click(function(){
		$(this).next('div').slideToggle('slow');
	});

	$("body.adminadvancedcmssettings").on('click', 'button.adcms-copy-all-language', function(e){
		var form_groups = $("#block_settings_wrapper").find(".form-group.clearfix").children(".adcms-lang");


		for (var i = 0; i < form_groups.length; i++) {
			var translate_form = $(form_groups[i]).find(".translatable-field:visible");

			if (translate_form.length != 0) {

				var translate_form_input = $(translate_form).find("input[type=text]").not('[name="image_url"]');
				if(translate_form_input.length != 0) {
					$(form_groups[i]).find(".translatable-field:hidden").find("input[type=text]").val(translate_form_input.val());
				}

				var translate_form_textarea = $(translate_form).find("textarea");
				if(translate_form_textarea.length != 0){
					$(form_groups[i]).find(".translatable-field:hidden").find("textarea").val(translate_form_textarea.val());
				}

				var translate_form_iframe = $(translate_form).find("iframe").contents().find("body#tinymce");
				if(translate_form_iframe.length != 0){
					$(form_groups[i]).find(".translatable-field:hidden").find("iframe").contents().find("body#tinymce").html(translate_form_iframe.html())
				}

				var translate_form_cslider = $(translate_form).find(".cslider-wrapper ul.cattree");
				if(translate_form_cslider.length != 0){
					var other = $(form_groups[i]).find(".translatable-field:hidden").find(".cslider-wrapper ul.cattree");

					if(other.length != 0){
						$.each(other, function(i, bl) {
							var def = translate_form_cslider.clone(),
								par = $(bl).closest('.panel'),
								idT = $(bl).attr('id');

							def.attr('id', idT);
							par.append(def);
							$(bl).remove();
						});
					}
				}

				var translate_form_gallery = $(translate_form).find("div.adcms-gallery-image-select");
				var translate_form_gallery_active = $(translate_form).find("div.adcms-gallery-image-select.adcms-gallery-image-select-active");
				if(translate_form_gallery.length != 0) {
					$(form_groups[i]).find(".translatable-field:hidden .adcms-gallery-image-select").removeClass('adcms-gallery-image-select-active');

					if(translate_form_gallery_active.length != 0) {
						var other = $(form_groups[i]).find(".translatable-field:hidden").find(".adcms-gallery-lang-wrapper"),
							name = translate_form_gallery_active.data('imageName');
						if(other.length != 0) {
							$.each(other, function(i, bl) {
								AdvancedCmsBlockImage.processGalleryImageClick($(bl).find('div[data-image-name="'+name+'"]'));
							});
						}
					}
				}

				var translate_form_gallery_slider = $(translate_form).find("div.adcms-gallery-image-wrapper");
				if(translate_form_gallery_slider.length != 0) {
					var other = $(form_groups[i]).find(".translatable-field:hidden").find(".adcms-gallery-lang-wrapper");
					$.each(translate_form_gallery_slider, function(i, bl) {
						var selected = $(bl).find('input[type=checkbox].adcms-gallery-image-slider-checkbox:checked'),
							name = $(bl).data('imageName'),
							url = $(bl).find('input[name="image_url"]').val();

						if(selected.length != 0) {
							var check = other.find('div.adcms-gallery-image-wrapper[data-image-name="'+name+'"] input[type=checkbox].adcms-gallery-image-slider-checkbox');
							$.each(check, function(ind, che) {
								$(che).prop('checked', true); //.attr('checked','checked');
								$(che).closest('.adcms-gallery-image-wrapper').find('input[name="image_url"]').val(url);
								AdvancedCmsBlockImageSlider.processGalleryImageClick($(che));
							});
							
						}
					});
				}

				var mslider = $(translate_form).find("div.mslider-wrapper");
				if(mslider.length != 0) {
					var lang = mslider.data('languageId'),
						list = mslider.find('input[name="manufacturer_slider_'+lang+'_manufacturers"]'),
						id = mslider.find('input[name="manufacturer_slider_'+lang+'_ids"]'),
						image = mslider.find('input[name="manufacturer_slider_'+lang+'_image"]'),
						contentdef = mslider.find('.selected_manufacturers'),
						other = $(form_groups[i]).find(".translatable-field:hidden").find(".mslider-wrapper");

					$.each(other, function(i, bl) {
						var content = contentdef.clone();
						var bllang = $(bl).data('languageId');
						$(bl).find('input[name="manufacturer_slider_'+bllang+'_manufacturers"]').val(list.val());
						$(bl).find('input[name="manufacturer_slider_'+bllang+'_ids"]').val(id.val());
						$(bl).find('input[name="manufacturer_slider_'+bllang+'_image"]').val(image.val());
						content.find('button.delCmsManufacturer').attr('data-language-id', bllang);
						$(bl).find('.selected_manufacturers').remove();
						$(bl).append(content);
					});
				}

				var pslider = $(translate_form).find("div.pslider-wrapper");
				if(pslider.length != 0) {
					var lang = pslider.data('languageId'),
						list = pslider.find('input[name="product_slider_'+lang+'_products"]'),
						id = pslider.find('input[name="product_slider_'+lang+'_ids"]'),
						ref = pslider.find('input[name="product_slider_'+lang+'_reference"]'),
						image = pslider.find('input[name="product_slider_'+lang+'_image"]'),
						contentdef = pslider.find('.selected_products'),
						other = $(form_groups[i]).find(".translatable-field:hidden").find(".pslider-wrapper");

					$.each(other, function(i, bl) {
						var content = contentdef.clone();
						var bllang = $(bl).data('languageId');
						$(bl).find('input[name="product_slider_'+bllang+'_products"]').val(list.val());
						$(bl).find('input[name="product_slider_'+bllang+'_ids"]').val(id.val());
						$(bl).find('input[name="product_slider_'+bllang+'_reference"]').val(ref.val());
						$(bl).find('input[name="product_slider_'+bllang+'_image"]').val(image.val());
						content.find('button.delCmsProduct').attr('data-language-id', bllang);
						$(bl).find('.selected_products').remove();
						$(bl).append(content);
					});
				}

				var blockproduct = $(translate_form).find("div.block_product-wrapper");
				if(blockproduct.length != 0) {
					var lang = blockproduct.data('languageId'),
						list = blockproduct.find('input[name="block_product_'+lang+'_products"]'),
						id = blockproduct.find('input[name="block_product_'+lang+'_ids"]'),
						ref = blockproduct.find('input[name="block_product_'+lang+'_reference"]'),
						image = blockproduct.find('input[name="block_product_'+lang+'_image"]'),
						contentdef = blockproduct.find('.selected_products'),
						other = $(form_groups[i]).find(".translatable-field:hidden").find(".block_product-wrapper");

					$.each(other, function(i, bl) {
						var content = contentdef.clone();
						var bllang = $(bl).data('languageId');
						$(bl).find('input[name="block_product_'+bllang+'_products"]').val(list.val());
						$(bl).find('input[name="block_product_'+bllang+'_ids"]').val(id.val());
						$(bl).find('input[name="block_product_'+bllang+'_reference"]').val(ref.val());
						$(bl).find('input[name="block_product_'+bllang+'_image"]').val(image.val());
						content.find('button.delCmsProduct').attr('data-language-id', bllang);
						$(bl).find('.selected_products').remove();
						$(bl).append(content);
					});
				}
			};
		};

		$(this).next("p").show();
	});
});
