<?php
/**
 * Geo IP Ban
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 * @author    FMM Modules
 * @copyright Copyright 2020 © FMM Modules All right reserved
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category  administration
 * @package   geoipban
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once dirname(__FILE__) . '/models/GeoipbanModel.php';
include_once _PS_MODULE_DIR_ . 'geoipban/lib/CSVReader.php';
include_once _PS_MODULE_DIR_ . 'geoipban/lib/Csv.php';

class Geoipban extends Module
{
    public function __construct()
    {
        $this->name = 'geoipban';
        $this->tab = 'administration';
        $this->version = '2.1.0';
        $this->author = 'FMM Modules';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->module_key = '4d8724824dde7c118ba8c575f84eeae4';
        $this->author_address = '0xcC5e76A6182fa47eD831E43d80Cd0985a14BB095';
        parent::__construct();
        $this->context = Context::getContext();
        $this->displayName = $this->l('Geo IP Ban');
        $this->description = $this->l('Allow administrator to block a country or IPs for front office.');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    public function install()
    {
        Configuration::updateValue(
            'BLOCKING_OPTION_MESSAGE',
            $this->l('You can not access this website'),
            true,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        return parent::install() &&
        $this->registerHook('backOfficeHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName('BLOCKING_OPTION');
        Configuration::deleteByName('BLOCKING_OPTION_BLACK_VISITORS');
        Configuration::deleteByName('BLOCKING_MSG_OPTION');
        Configuration::deleteByName('BLOCKING_OPTION_URL');
        Configuration::deleteByName('BLOCKING_METHOD_OPTION');
        Configuration::deleteByName('BLOCKING_OPTION_MESSAGE');
        Configuration::deleteByName('BLOCKING_OPTION_BLACK_IPS');
        Configuration::deleteByName('BLOCKING_OPTION_WHITE_IPS');
        Configuration::deleteByName('BLOCKING_OPTION_COUNTRIES');
        return parent::uninstall();
    }

    public function init()
    {
        if (((bool) Tools::isSubmit('submitGeoipbanModule')) == true) {
            $id_lang = $this->context->language->id;
            $blocking_option = pSQL(Tools::getValue("BLOCKING_OPTION"));
            $msg_option = pSQL(Tools::getValue("BLOCKING_MSG_OPTION"));
            $msg = pSQL(Tools::getValue("BLOCKING_OPTION_MESSAGE_" . (int) $id_lang));
            $url_option = pSQL(Tools::getValue("BLOCKING_OPTION_URL"));
            $black_ip = pSQL(Tools::getValue("BLOCKING_OPTION_BLACK_IPS"));
            $country = Tools::getValue('countries');
            if ($blocking_option == 'fmm_block_ips') {
                if ($black_ip == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please Enter Ips to block'
                    );
                } elseif ($msg_option == 'fmm_redirect' && $url_option == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please enter url to redirct your visitors'
                    );
                } elseif ($msg_option == 'fmm_msg' && $msg == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please enter message to notify your visitors'
                    );
                } else {
                    $this->context->controller->confirmations[] = $this->l(
                        'Updated Successfully'
                    );
                }
            } else {
                if ($country == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please select countries to block'
                    );
                } elseif ($msg_option == 'fmm_redirect' && $url_option == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please enter url to redirct your visitors'
                    );
                } elseif ($msg_option == 'fmm_msg' && $msg == '') {
                    $this->context->controller->errors[] = $this->l(
                        'Please enter message to notify your visitors'
                    );
                } else {
                    $this->context->controller->confirmations[] = $this->l(
                        'Updated Successfully'
                    );
                }
            }
        }
    }

    public function getContent()
    {
        if (((bool) Tools::isSubmit('submitGeoipbanModule')) == true) {
            $this->init();
            $this->postProcess();
        }
        $action = Tools::getValue('action');
        if ($action == 'blockVisitorIP') {
            $ip = long2ip(Tools::getValue('ip'));
            $blocked_ip = explode(',', Configuration::get(
                "BLOCKING_OPTION_BLACK_VISITORS",
                false,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            ));
            if ($blocked_ip[0] == "") {
                Configuration::updateValue(
                    "BLOCKING_OPTION_BLACK_VISITORS",
                    $ip,
                    false,
                    $this->context->shop->id_shop_group,
                    $this->context->shop->id
                );
                die(json_encode($ip));
            } else {
                array_push($blocked_ip, $ip);
                $blocked_ip = implode($blocked_ip, ',');
                Configuration::updateValue(
                    "BLOCKING_OPTION_BLACK_VISITORS",
                    $blocked_ip,
                    false,
                    $this->context->shop->id_shop_group,
                    $this->context->shop->id
                );
                die(json_encode($blocked_ip));
            }
        }
        if ($action == 'unBlockVisitorIP') {
            $ip = Tools::getValue('ip');
            $blocked_ip = explode(',', Configuration::get(
                "BLOCKING_OPTION_BLACK_VISITORS",
                false,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            ));

            if (($key = array_search($ip, $blocked_ip)) !== false) {
                unset($blocked_ip[$key]);
                $blocked_ip = implode($blocked_ip, ',');
                Configuration::updateValue(
                    "BLOCKING_OPTION_BLACK_VISITORS",
                    $blocked_ip,
                    false,
                    $this->context->shop->id_shop_group,
                    $this->context->shop->id
                );
            }
            die(json_encode($blocked_ip));
        }
        $this->html = $this->display(__FILE__, 'views/templates/hook/info.tpl');

        return $this->html . $this->renderForm();
    }

    protected function renderForm()
    {
        $msg = array();
        $list = Country::getCountries($this->context->language->id);
        $languages = Language::getLanguages(false);
        $this->context->smarty->assign('defaultlangiso', $this->context->language->iso_code);
        $this->context->smarty->assign('current_lang', $this->context->language->id);
        $this->context->controller->addJS(array(
            _PS_JS_DIR_ . 'tiny_mce/tiny_mce.js',
            _PS_JS_DIR_ . 'admin/tinymce.inc.js',
        ));
        $blocking_option = pSQL(Configuration::get(
            "BLOCKING_OPTION",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $selected_countries = explode(',', Configuration::get(
            "BLOCKING_OPTION_COUNTRIES",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $msg_option = pSQL(Configuration::get(
            "BLOCKING_MSG_OPTION",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $method_option = pSQL(Configuration::get(
            "BLOCKING_METHOD_OPTION",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $url_option = pSQL(Configuration::get(
            "BLOCKING_OPTION_URL",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        foreach ($languages as $lang) {
            $msg['BLOCKING_OPTION_MESSAGE'][(int) $lang['id_lang']] = Configuration::get(
                "BLOCKING_OPTION_MESSAGE",
                (int) $lang['id_lang'],
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            );
        }
        $black_ip = Configuration::get(
            "BLOCKING_OPTION_BLACK_IPS",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        $blocked_ip = explode(',', Configuration::get(
            "BLOCKING_OPTION_BLACK_IPS",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $white_ip = Configuration::get(
            "BLOCKING_OPTION_WHITE_IPS",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        $blocked_visitors = explode(',', Configuration::get(
            "BLOCKING_OPTION_BLACK_VISITORS",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $current_index = $this->context->link->getAdminLink('AdminModules', false);
        $current_token = Tools::getAdminTokenLite('AdminModules');
        $action_url = $current_index . '&configure=' . $this->name . '&token=' . $current_token;
        $ip_count = GeoipbanModel::getVisitorsOnline();
        $this->context->smarty->assign(array(
            'blocking_option' => $blocking_option,
            'msg_option' => $msg_option,
            'method_option' => $method_option,
            'url_option' => $url_option,
            'msg' => $msg,
            'ip_count' => $ip_count,
            'block_ip' => _PS_MODULE_DIR_ .
            'geoipban/views/templates/admin/configure/form.tpl',
            'online_visitors' => _PS_MODULE_DIR_ .
            'geoipban/views/templates/admin/configure/visitors.tpl',
            'blocked_ip' => $blocked_ip,
            'blocked_visitors' => $blocked_visitors,
            'black_ip' => $black_ip,
            'white_ip' => $white_ip,
            'selectd_countri' => $selected_countries,
            'list' => $list,
            'action_url' => $action_url,
            'ps_version' => _PS_VERSION_,
            'languages' => $languages,
        ));

        if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') == true) {
            $form = $this->fetch(
                $this->local_path . 'views/templates/admin/configure/tab.tpl'
            );
        } else {
            $form = $this->display(__FILE__, 'views/templates/admin/configure/tab.tpl');
        }
        return $form;
    }

    protected function postProcess()
    {
        $blocking_option = pSQL(Tools::getValue("BLOCKING_OPTION"));
        $msg_option = pSQL(Tools::getValue("BLOCKING_MSG_OPTION"));
        $method_option = pSQL(Tools::getValue("BLOCKING_METHOD_OPTION"));
        $url_option = pSQL(Tools::getValue("BLOCKING_OPTION_URL"));
        $black_ip = pSQL(Tools::getValue("BLOCKING_OPTION_BLACK_IPS"));
        $white_ip = pSQL(Tools::getValue("BLOCKING_OPTION_WHITE_IPS"));
        $country = Tools::getValue('countries');
        if (!empty($country)) {
            $country = implode(',', $country);
        }
        $languages = Language::getLanguages(false);
        $values = array();
        foreach ($languages as $lang) {
            $values['BLOCKING_OPTION_MESSAGE'][$lang['id_lang']] =
            Tools::getValue('BLOCKING_OPTION_MESSAGE_' . $lang['id_lang']);
        }
        Configuration::updateValue(
            'BLOCKING_OPTION',
            $blocking_option,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_MSG_OPTION',
            $msg_option,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_METHOD_OPTION',
            $method_option,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_OPTION_BLACK_IPS',
            $black_ip,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_OPTION_COUNTRIES',
            $country,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_OPTION_WHITE_IPS',
            $white_ip,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_OPTION_MESSAGE',
            $values['BLOCKING_OPTION_MESSAGE'],
            true,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        Configuration::updateValue(
            'BLOCKING_OPTION_URL',
            $url_option,
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
    }

    public function hookBackOfficeHeader()
    {

        if (Module::isInstalled('geoipban') && Module::isEnabled('geoipban') && Tools::getvalue("configure")) {
            $iso_tiny_mce = $this->context->language->iso_code;
            $iso_tiny_mce = (file_exists(_PS_JS_DIR_ . 'tiny_mce/langs/' . $iso_tiny_mce . '.js') ?
            $iso_tiny_mce : 'en');
            $current_index = $this->context->link->getAdminLink('AdminModules', false);
            $current_token = Tools::getAdminTokenLite('AdminModules');
            $form_url = $current_index . '&configure=' . $this->name . '&token=' . $current_token;
            Media::addJsDef(array(
                'iso_tiny_mce' => $iso_tiny_mce,
                'form_url' => $form_url,
            ));
            $this->context->controller->addJquery();
            $this->context->controller->addJS($this->_path . 'views/js/geoip_back.js');
            $this->context->controller->addJS($this->_path . 'views/js/datatables.min.js');
            $this->context->controller->addCSS($this->_path . 'views/css/geoip-back.css');
            $this->context->controller->addCSS($this->_path . 'views/css/datatables.min.css');
        }
    }
    public function dotToLongIP($ipaddress)
    {
        $pton = @inet_pton($ipaddress);
        if (!$pton) {
            return false;
        }
        $number = '';
        foreach (unpack('C*', $pton) as $byte) {
            $number .= str_pad(decbin($byte), 8, '0', STR_PAD_LEFT);
        }
        return base_convert(ltrim($number, '0'), 2, 10);
    }

    public function forOverrides()
    {
        $ip = Tools::getRemoteAddr();
        $blocked_visitors = explode(',', Configuration::get(
            "BLOCKING_OPTION_BLACK_VISITORS",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $selected_blocking_option = Configuration::get(
            'BLOCKING_OPTION',
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        $black_ip = explode(',', Configuration::get(
            'BLOCKING_OPTION_BLACK_IPS',
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        $white_ip = explode(
            ',',
            Configuration::get(
                'BLOCKING_OPTION_WHITE_IPS',
                false,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            )
        );
        $selected_countries = explode(
            ',',
            Configuration::get(
                "BLOCKING_OPTION_COUNTRIES",
                false,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            )
        );
        $method_option = pSQL(Configuration::get(
            "BLOCKING_METHOD_OPTION",
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        ));
        if ($method_option == 'fmm_file') {
            if ($selected_blocking_option == 'fmm_block_ips') {
                if (in_array($ip, $black_ip) || in_array($ip, $blocked_visitors)) {
                    $this->infoMethodAboutBlocking();
                }
            } else {
                if (Tools::strlen($ip) <= 15) {
                    $csvPath = _PS_MODULE_DIR_ . 'geoipban/src/ipv4.CSV';
                    if (!file_exists($csvPath)) {
                        //$this->bootstrap = true;
                        $file = "ipv4";
                        $this->context->smarty->assign('file', $file);
                        $this->context->smarty->display(_PS_MODULE_DIR_ . 'geoipban/views/templates/front/errors.tpl');
                        return false;
                    } else {
                        $ip = ip2long($ip);
                        ini_set('memory_limit', '-1');
                        $csv = new Varien_File_Csv();
                        $data = $csv->getData($csvPath);
                        foreach ($data as $range) {
                            if ($ip >= $range[0] && $ip <= $range[1]) {
                                if (in_array($range[2], $selected_countries) &&
                                    !in_array(long2ip($ip), $white_ip) || in_array(long2ip($ip), $blocked_visitors)) {
                                    $this->infoMethodAboutBlocking();
                                }
                            }
                        }
                    }
                } else {
                    $csvPath = _PS_MODULE_DIR_ . 'geoipban/src/ipv6.CSV';
                    if (!file_exists($csvPath)) {
                        $file = "ipv6";
                        $this->context->smarty->assign('file', $file);
                        $this->context->smarty->display(_PS_MODULE_DIR_ . 'geoipban/views/templates/front/errors.tpl');
                        return false;
                    } else {
                        $ip_range = $this->dotToLongIP($ip);
                        ini_set('memory_limit', '-1');
                        $csv = new Varien_File_Csv();
                        $data = $csv->getData($csvPath);
                        foreach ($data as $range) {
                            if ($ip_range >= $range[0] && $ip_range <= $range[1]) {
                                if (in_array($range[2], $selected_countries) &&
                                    !in_array($ip, $white_ip) || in_array($ip, $blocked_visitors)) {
                                    $this->infoMethodAboutBlocking();
                                }
                            }
                        }
                        die;
                    }
                }
            }
        } else {
            if ($selected_blocking_option == 'fmm_block_ips') {
                if (in_array($ip, $black_ip) || in_array($ip, $blocked_visitors)) {
                    $this->infoMethodAboutBlocking();
                }
            } else {
                $iptolocation = 'http://api.hostip.info/country.php?ip=' . $ip;
                $creatorlocation = Tools::file_get_contents($iptolocation);
                if (in_array($creatorlocation, $selected_countries) &&
                    !in_array($ip, $white_ip) || in_array($ip, $blocked_visitors)) {
                    $this->infoMethodAboutBlocking();
                }
            }
        }
    }

    public function redirectTo($redirect_link)
    {
        $this->context->smarty->assign('redirect_url', $redirect_link);
        $this->context->smarty->display(_PS_MODULE_DIR_ . 'geoipban/views/templates/front/redirect.tpl');
    }

    public function infoMethodAboutBlocking()
    {
        $selected_msg_option = Configuration::get(
            'BLOCKING_MSG_OPTION',
            false,
            $this->context->shop->id_shop_group,
            $this->context->shop->id
        );
        if ($selected_msg_option == 'fmm_msg') {
            echo Configuration::get(
                "BLOCKING_OPTION_MESSAGE",
                true,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            );
            die();
        } else {
            $this->redirectTo(Configuration::get(
                "BLOCKING_OPTION_URL",
                false,
                $this->context->shop->id_shop_group,
                $this->context->shop->id
            ));
        }
    }
}
