<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class HiCookieConsent extends ObjectModel
{
    public $id_consent;
    public $id_shop;
    public $ip_address;
    public $id_guest;
    public $id_customer;
    public $id_web_browser;
    public $id_operating_system;
    public $consents;
    public $date_add;
    public $date_upd;

    public static $definition = [
        'table' => 'hicookieconsent',
        'primary' => 'id_consent',
        'multilang' => false,
        'fields' => [
            'id_shop' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'ip_address' => ['type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255],
            'id_guest' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'id_customer' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'id_web_browser' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'id_operating_system' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'consents' => ['type' => self::TYPE_STRING, 'validate' => 'isCleanHtml'],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate'],
            'date_upd' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'copy_post' => false],
        ],
    ];

    public static function getConsents($limit = false, $ofset = 0)
    {
        $id_shop = Context::getContext()->shop->id;

        $query = new DbQuery();
        $query
            ->select('cc.*')
            ->from('hicookieconsent', 'cc')
            ->where('cc.`id_shop` = ' . (int) $id_shop);

        if ($limit) {
            $query->limit((int) $limit, (int) $ofset);
        }

        return Db::getInstance()->executeS($query);
    }

    public static function getConsentsForPreview()
    {
        $id_shop = Context::getContext()->shop->id;

        $logs = Db::getInstance()->executeS('
            SELECT l.`id_consent`, l.`ip_address`, l.`id_guest`, l.`id_customer`, l.`consents`, l.`date_add`, l.`date_upd`, wb.`name` browser, os.`name` os
            FROM `' . _DB_PREFIX_ . 'hicookieconsent` l
            LEFT JOIN `' . _DB_PREFIX_ . 'web_browser` wb
                ON (l.`id_web_browser` = wb.`id_web_browser`)
            LEFT JOIN `' . _DB_PREFIX_ . 'operating_system` os
                ON (l.`id_operating_system` = os.`id_operating_system`)');

        if (!is_array($logs) || !$logs) {
            return [];
        }

        foreach ($logs as $key => $log) {
            $logs[$key]['consents'] = json_decode($log['consents'], true);
        }

        return $logs;
    }

    public static function getUserConsent($ip_address, $id_guest, $id_customer)
    {
        $id_shop = Context::getContext()->shop->id;

        if ($ip_address) {
            $res = Db::getInstance()->getRow('
                SELECT * FROM `' . _DB_PREFIX_ . 'hicookieconsent`
                WHERE `ip_address` = \'' . pSQL($ip_address) . '\'
                AND `id_shop` = ' . (int) $id_shop);

            if ($res) {
                return $res;
            }
        }

        if ($id_guest) {
            $res = Db::getInstance()->getRow('
                SELECT * FROM `' . _DB_PREFIX_ . 'hicookieconsent`
                WHERE `id_guest` = ' . (int) $id_guest . '
                AND `id_shop` = ' . (int) $id_shop);

            if ($res) {
                return $res;
            }
        }

        if ($id_customer) {
            $res = Db::getInstance()->getRow('
                SELECT * FROM `' . _DB_PREFIX_ . 'hicookieconsent`
                WHERE `id_guest` = ' . (int) $id_customer . '
                AND `id_shop` = ' . (int) $id_shop);

            if ($res) {
                return $res;
            }
        }

        return false;
    }

    public static function clearLogs()
    {
        return Db::getInstance()->delete('hicookieconsent');
    }
}
