<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class HiCookieType extends ObjectModel
{
    public $id_type;
    public $disabled;
    public $default_status;
    public $position;
    public $name;
    public $description;
    public $script;
    public $date_add;
    public $date_upd;

    public static $definition = [
        'table' => 'hicookietype',
        'primary' => 'id_type',
        'multilang' => true,
        'fields' => [
            'disabled' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'default_status' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'position' => ['type' => self::TYPE_INT, 'validate' => 'isInt'],
            'name' => ['type' => self::TYPE_STRING, 'validate' => 'isCleanHtml', 'size' => 255, 'lang' => true],
            'description' => ['type' => self::TYPE_HTML, 'validate' => 'isCleanHtml', 'lang' => true],
            'script' => ['type' => self::TYPE_HTML],
            'date_add' => ['type' => self::TYPE_DATE, 'validate' => 'isDate'],
            'date_upd' => ['type' => self::TYPE_DATE, 'validate' => 'isDate', 'copy_post' => false],
        ],
    ];

    public function delete()
    {
        $res = parent::delete();
        $res &= Db::getInstance()->delete('hicookietype_shop', 'id_type = ' . (int) $this->id);
        $res &= Db::getInstance()->delete('hicookietypemodule', 'id_type = ' . (int) $this->id);

        return $res;
    }

    public static function getTypes()
    {
        $id_lang = Context::getContext()->language->id;
        $id_shop = Context::getContext()->shop->id;

        $query = new DbQuery();
        $query
            ->select('t.*')
            ->select('tl.*')
            ->from('hicookietype', 't')
            ->leftJoin('hicookietype_lang', 'tl', 't.`id_type` = tl.`id_type`')
            ->leftJoin('hicookietype_shop', 'ts', 't.`id_type` = ts.`id_type`')
            ->where('tl.`id_lang` = ' . (int) $id_lang)
            ->where('ts.`id_shop` = ' . (int) $id_shop)
            ->orderBy('t.position');

        return Db::getInstance()->executeS($query);
    }

    public static function getAllBlockedModules()
    {
        $query = new DbQuery();
        $query
            ->select('tm.`id_module`')
            ->from('hicookietypemodule', 'tm');

        return Db::getInstance()->executeS($query);
    }

    public function getBlockedModules()
    {
        $query = new DbQuery();
        $query
            ->select('tm.`id_module`')
            ->from('hicookietypemodule', 'tm')
            ->where('tm.`id_type` = ' . (int) $this->id);

        return Db::getInstance()->executeS($query);
    }

    public function deleteModules()
    {
        return Db::getInstance()->delete('hicookietypemodule', 'id_type = ' . (int) $this->id);
    }

    public function blockModule($id_module)
    {
        return Db::getInstance()->insert('hicookietypemodule', [
            'id_type' => (int) $this->id,
            'id_module' => (int) $id_module,
        ]);
    }

    public static function getCookieTypesByModule($id_module)
    {
        $query = new DbQuery();
        $query
            ->select('tm.`id_type`')
            ->from('hicookietypemodule', 'tm')
            ->where('tm.`id_module` = ' . (int) $id_module);

        return Db::getInstance()->executeS($query);
    }

    public static function getTypesById($ids)
    {
        if (!is_array($ids) || !$ids) {
            return [];
        }

        $id_lang = Context::getContext()->language->id;

        return Db::getInstance()->executeS('
            SELECT * FROM `' . _DB_PREFIX_ . 'hicookietype` t
            LEFT JOIN `' . _DB_PREFIX_ . 'hicookietype_lang` tl
                ON (tl.`id_type` = t.`id_type`)
            WHERE tl.`id_lang` = ' . (int) $id_lang . '
            AND tl.id_type IN (' . implode(',', array_map('intval', $ids)) . ')
        ');
    }

    public static function getPosition()
    {
        return (int) DB::getInstance()->getValue('SELECT MAX(position) FROM ' . _DB_PREFIX_ . 'hicookietype') + 1;
    }
}
