<?php
/**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class HiCookieLawCookieModuleFrontController extends ModuleFrontController
{
    public function __construct()
    {
        $this->secure_key = Tools::getValue('secure_key');
        parent::__construct();
    }

    public function init()
    {
        if ($this->ajax && $this->secure_key == $this->module->secure_key) {
            if (Tools::getValue('action') == 'set_cookie') {
                $this->context->cookie->hicookie = 1;
                $acceptedTypes = Tools::getValue('acceptedTypes');
                $notAcceptedTypes = Tools::getValue('notAcceptedTypes');

                if (is_array($acceptedTypes) && $acceptedTypes) {
                    foreach ($acceptedTypes as $id_accepted_type) {
                        $this->context->cookie->{'hiThirdPartyCookies_' . $id_accepted_type} = 1;
                    }
                }

                if (is_array($notAcceptedTypes) && $notAcceptedTypes) {
                    foreach ($notAcceptedTypes as $id_not_accepted_type) {
                        $this->context->cookie->{'hiThirdPartyCookies_' . $id_not_accepted_type} = 0;
                    }
                }

                $this->context->cookie->write();

                $user_consent = HiCookieConsent::getUserConsent(Tools::getRemoteAddr(), $this->context->customer->id_guest, $this->context->customer->id);

                if ($user_consent) {
                    $consent = new HiCookieConsent($user_consent['id_consent']);
                } else {
                    $consent = new HiCookieConsent();
                }

                // To-Do fix for new version
                $consents = [];
                if ($acceptedTypes) {
                    $acceptedCookies = HiCookieType::getTypesById($acceptedTypes);

                    if (is_array($acceptedCookies) && $acceptedCookies) {
                        foreach ($acceptedCookies as $acceptedCookie) {
                            $consents[$acceptedCookie['name']] = 1;
                        }
                    }
                }

                if ($notAcceptedTypes) {
                    $notAcceptedCookies = HiCookieType::getTypesById($notAcceptedTypes);

                    if (is_array($notAcceptedCookies) && $notAcceptedCookies) {
                        foreach ($notAcceptedCookies as $notAcceptedCookie) {
                            $consents[$notAcceptedCookie['name']] = 0;
                        }
                    }
                }

                $consent->consents = json_encode($consents);

                $consent->ip_address = Tools::getRemoteAddr();
                $consent->id_guest = $this->context->customer->id_guest;
                $consent->id_customer = $consent->id_customer ? $consent->id_customer : $this->context->customer->id;
                $consent->id_shop = $consent->id_shop ? $consent->id_shop : $this->context->shop->id;

                $consent->id_web_browser = $this->module->getBrowser($_SERVER['HTTP_USER_AGENT']);
                $consent->id_operating_system = $this->module->getOs($_SERVER['HTTP_USER_AGENT']);
                $consent->save();

                // Google Consent Mode
                $adStorageCookieTypeGranted = 'denied';
                $analyticsStorageCookieTypeGranted = 'denied';
                if ($this->module->consentMode) {
                    if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->module->adStorageCookieType} == 1) {
                        $adStorageCookieTypeGranted = 'granted';
                    }

                    if ($this->context->cookie->{'hiThirdPartyCookies_' . $this->module->analyticsStorageCookieType} == 1) {
                        $analyticsStorageCookieTypeGranted = 'granted';
                    }
                }

                exit(json_encode([
                    'cookie_set' => true,
                    'adStorageCookieTypeGranted' => $adStorageCookieTypeGranted,
                    'analyticsStorageCookieTypeGranted' => $analyticsStorageCookieTypeGranted,
                ]));
            }
        }

        exit;
    }
}
