// changelog
// version 1.0.0
// initial version

// Actions
// File upload action is defnined from configs: dropZoneElem.attr('url-upload')
// Load{inputName}ActionForm
// Sort{inputName}
// Delete{inputName}

// extend DropZone plugin for HiPresta modules
// version 1.0.0, Jan 11th, 2023
// by Suren Mikaelyan

Dropzone.autoDiscover = false;

$(function() {
	if (typeof ajaxErrorMessage == 'undefined') {
		window.ajaxErrorMessage = 'Something went wrong, please reload the page and try again.';
	}

	hiDropZone.initEvents();
});

hiDropZone = {
    init: function() {
		this.findAndTriggerDropZone();
        // this.initEvents();
    },

    initEvents: function() {
        $(document).on('click', '.hi-module-dropzone-action-form .dz-preview', function () {
            if (!$(this).attr('data-id')) {
              	return;
            }

			var $this = $(this);
			const dropZoneElem = $(this).closest('.hi-module-dropzone');
			const inputNameClean = dropZoneElem.attr('data-input-name-clean');
        	const formZoneElem = $('#hi-module-dropzone-form-container-' + inputNameClean);
			var idFile = $this.attr('data-id');
			let idLang = dropZoneElem.attr('data-id-lang');
			let idElement = dropZoneElem.attr('data-id-element');

			dropZoneElem.find('.dz-preview.active').removeClass('active');
			$.ajax({
                type: 'POST',
                dataType: 'JSON',
                url: dropZoneElem.attr('url-action'),
                data: {
                    ajax: true,
                    action: 'load' + inputNameClean + 'ActionForm',
					idElement: idElement,
                    idFile: idFile,
					idLang: idLang
                },
				beforeSend: function() {
					$this.addClass('hi-presta-module-spinner');
				},
                success: function(response) {
					$this.removeClass('hi-presta-module-spinner');

					if (response.error) {
						showErrorMessage(response.error);
					} else {
						formZoneElem.find('#hi-module-dropzone-form-' + inputNameClean).html(response.content);

						formZoneElem.show();
						$this.addClass('active');

						$([document.documentElement, document.body]).animate({
							scrollTop: formZoneElem.offset().top - 300
						}, 800);
					}
                },
                error: function(jqXHR, error, errorThrown) {
					$this.removeClass('hi-presta-module-spinner');
					showErrorMessage(ajaxErrorMessage);
                },
            });
        });

		$(document).on('click', '.dropzone-action-form-close-button', function(e) {
			e.preventDefault();

			var $this = $(this);
			const dropZoneElem = $this.closest('.hi-module-dropzone-container').find('.hi-module-dropzone');
			const inputNameClean = dropZoneElem.attr('data-input-name-clean');
        	const formZoneElem = $('#hi-module-dropzone-form-container-' + inputNameClean);

			formZoneElem.find('#hi-module-dropzone-form-' + inputNameClean).html('');
			formZoneElem.hide();
			dropZoneElem.find('.dz-preview.active').removeClass('active');
		});
    },

    findAndTriggerDropZone: function() {
        $('.hi-module-dropzone').each(function() {
            if (!$(this).find('.hi-module-dropzone-openfilemanager').hasClass('.dz-clickable')) {
                hiDropZone.dropZone($(this).attr('id'));
            }
        });
    },

    checkDropzoneMode: function(dropZoneElem, maxFiles) {
        if (maxFiles > 1) {
            if (dropZoneElem.find('.dz-preview:not(.hi-module-dropzone-openfilemanager)').length) {
                dropZoneElem.find('.dz-preview.hi-module-dropzone-openfilemanager').show();
            } else {
                dropZoneElem.find('.dz-preview.hi-module-dropzone-openfilemanager').hide();
            }
        } else {
            if (dropZoneElem.find('.dz-preview:not(.hi-module-dropzone-openfilemanager)').length) {
                dropZoneElem.find('.dz-preview.hi-module-dropzone-openfilemanager').hide();
            } else {
                dropZoneElem.find('.dz-preview.hi-module-dropzone-openfilemanager').show();
            }
        }
    },

    dropZone: function(dropZoneElemId) {
        const dropZoneElem = $('#' + dropZoneElemId);
        const errorElem = $('#' + dropZoneElemId + '-error');
        const maxFiles = dropZoneElem.attr('data-max-files');
        const isSortable = (dropZoneElem.attr('data-sortable') == 1 ? true : false);
		const inputName = dropZoneElem.attr('data-input-name');
		const inputNameClean = dropZoneElem.attr('data-input-name-clean');
		const filesLocation = dropZoneElem.attr('data-files-location');
		const actionUrl = dropZoneElem.attr('url-action');
		const idElement = dropZoneElem.attr('data-id-element');
		const formZoneElem = $('#hi-module-dropzone-form-container-' + inputNameClean);
		const acceptedFiles = dropZoneElem.attr('data-accepted-files');

		var existingImagesArray = dropZoneElem.attr('data-existing-images');
		var existingImages = [];
		if (existingImagesArray) {
			existingImages = JSON.parse(existingImagesArray);
		}
		var actionForm = dropZoneElem.attr('data-action-form');
		if (typeof actionForm == 'undefined') {
			actionForm = false;
		}
		var idLang = dropZoneElem.attr('data-id-lang');
		if (typeof idLang == 'undefined') {
			idLang = 0;
		}
        
        const dropzoneOptions = {
            url: dropZoneElem.attr('url-upload') + '&idLang=' + idLang + '&idElement=' + idElement,
            paramName: inputName,
            maxFilesize: dropZoneElem.attr('data-max-size'),
            addRemoveLinks: true,
            clickable: '#' + dropZoneElemId + ' .hi-module-dropzone-openfilemanager',
            thumbnailWidth: 250,
            thumbnailHeight: null,
            acceptedFiles: acceptedFiles,
            timeout: 0,
            dictRemoveFile: '<i class="icon icon-remove"></i>',
            dictMaxFilesExceeded: dropZoneElem.attr('data-dictMaxFilesExceeded'),
            maxFiles: maxFiles,
            sending() {
                hiDropZone.checkDropzoneMode(dropZoneElem, maxFiles);
				errorElem.html('');
            },
            queuecomplete() {
                hiDropZone.checkDropzoneMode(dropZoneElem, maxFiles);
                if (isSortable == 1) {
                    dropZoneElem.sortable('enable');
                }
            },
            processing() {
                if (isSortable == 1) {
                    dropZoneElem.sortable('disable');
                }
            },
            success(file, response) {
                // manage error on uploaded file
				if (typeof response.error == 'undefined') {
					errorElem.append($('<p></p>').text(ajaxErrorMessage));
                    this.removeFile(file);
                    return;
				}

                if (response.error) {
                    errorElem.append($('<p></p>').text(`${file.name}: ${response.error}`));
                    this.removeFile(file);
                    return;
                }
        
                // define id image to file preview
                $(file.previewElement).attr('data-id', response.id);
				$(file.previewElement).find('.hi-dropzone-input-item').remove();
				if (maxFiles > 1) {
					$(file.previewElement).append('<input type="hidden" name="'+ dropZoneElemId +'[]" class="hi-dropzone-input-item" value="'+ response.id +'">');
				} else {
					$(file.previewElement).append('<input type="hidden" name="'+ dropZoneElemId + '" class="hi-dropzone-input-item" value="'+ response.id +'">');
				}
            },
            error(file, response) {
                let message = '';
            
                if ($.type(response) === 'undefined') {
                    return;
                } if ($.type(response) === 'string') {
                    message = response;
                } else if (response.message) {
                    // eslint-disable-next-line
                    message = response.message;
                }
            
                if (message === '') {
                    return;
                }
            
                // append new error
                errorElem.append($('<p></p>').text(`${file.name}: ${message}`));
            
                // remove uploaded item
                this.removeFile(file);
            },
            init() {
				if (existingImages.length > 0) {
					for (var i = 0; i < existingImages.length; i++) {
						if (!existingImages[i].fileName || !existingImages[i].id) {
							continue;
						}
						var mockFile = {
							name: existingImages[i].fileName, 
							// type: 'image/jpeg',
							accepted: true
						};

						this.files.push(mockFile);
						this.emit("addedfile", mockFile);
						this.emit("thumbnail", mockFile, filesLocation + existingImages[i].fileName);
						this.emit("complete", mockFile);

						if (maxFiles > 1) {
							$(this.files[this.files.length - 1].previewElement).append('<input type="hidden" name="'+ dropZoneElemId +'[]" class="hi-dropzone-input-item" value="'+ existingImages[i].id +'">');
						} else {
							$(this.files[this.files.length - 1].previewElement).append('<input type="hidden" name="'+ dropZoneElemId +'" class="hi-dropzone-input-item" value="'+ existingImages[i].id +'">');
						}

						$(this.files[this.files.length - 1].previewElement).attr('data-id', existingImages[i].id);
					}
				}

                hiDropZone.checkDropzoneMode(dropZoneElem, maxFiles);
            
                // init sortable
                if (isSortable == 1) {
                    dropZoneElem.sortable({
                        items: 'div.dz-preview:not(.disabled)',
                        opacity: 0.9,
                        containment: 'parent',
                        distance: 32,
                        tolerance: 'pointer',
                        cursorAt: {
                        left: 64,
                        top: 64,
                        },
                        cancel: '.disabled',
                        stop() {
                        let sort = {};
                        $.each(dropZoneElem.find('.dz-preview:not(.disabled)'), (index, value) => {
                            if (!$(value).attr('data-id')) {
                            	sort = false;
                            	return;
                            }
                            sort[$(value).attr('data-id')] = index + 1;
                        });
                
                        // if sortable ok, update it
                        if (sort) {
                            $.ajax({
								type: 'POST',
								url: actionUrl,
								dataType: 'json',
								data: {
									json: JSON.stringify(sort),
									idElement: idElement,
									action: 'sort' + inputNameClean,
									idLang: idLang,
									ajax: true
								},
								success: function(response) {
									if (response.error) {
										showErrorMessage(response.error);
									} else {
										if (response.message) {
											showSuccessMessage(response.message);
										}
									}
								},
								error: function() {
									showErrorMessage(ajaxErrorMessage);	
								}
                            });
                        }
                        },
                        start(event, ui) {
							// init zindex
							dropZoneElem.find('.dz-preview').css('zIndex', 1);
							ui.item.css('zIndex', 10);
                        },
                    });
                }
            
                dropZoneElem.disableSelection();
            },
        
            removedfile(file) {
                if (file.previewElement != null && file.previewElement.parentNode != null) {
					file.previewElement.parentNode.removeChild(file.previewElement);

					hiDropZone.checkDropzoneMode(dropZoneElem, maxFiles);
					formZoneElem.find('.dropzone-action-form-close-button').click();

					let idFile = $(file.previewElement).attr('data-id');
					// if (typeof file.accepted != 'undefined' && file.accepted && (typeof file.status == 'undefined' || file.status != 'error') && typeof idFile != 'undefined') {
					if (typeof idFile != 'undefined') {
						$.ajax({
							type: 'POST',
							url: actionUrl,
							dataType: 'json',
							data: {
								ajax: true,
								action: 'delete' + inputNameClean,
								idFile: idFile,
								idElement: idElement,
							},
							beforeSend: function() {
								$(file.previewElement).addClass('hi-presta-module-spinner');
							},
							success: function(response) {
								$(file.previewElement).removeClass('hi-presta-module-spinner');

								if (response.error) {
									showErrorMessage(response.error);
								} else {
									if (response.message) {
										showSuccessMessage(response.message);
									}
								}
							},
							error: function() {
								$(file.previewElement).removeClass('hi-presta-module-spinner');
								showErrorMessage(ajaxErrorMessage);
							}
						});
					}
                }

				return this._updateMaxFilesReachedClass();
            },
        };

        dropZoneElem.dropzone(jQuery.extend(dropzoneOptions));
    },

	changeActionFormLanguage: function(idLang)
	{
		$('.hmd-action-form-translatable-field').hide();
		$('.hmd-action-form-lang-' + idLang).show();
	}
};