{**
 * 2012 - 2024 HiPresta
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0).
 * It is also available through the world-wide-web at this URL: https://opensource.org/licenses/AFL-3.0
 *
 * @author    HiPresta <support@hipresta.com>
 * @copyright HiPresta 2024
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 *
 * @website   https://hipresta.com
 *}
{extends file="helpers/form/form.tpl"}

{block name="label"}
    {if isset($input.label)}
        <label class="control-label col-lg-3{if isset($input.required) && $input.required && $input.type != 'radio'} required{/if}">
            {if isset($input.hint)}
            <span class="label-tooltip" data-toggle="tooltip" data-html="true" title="{if is_array($input.hint)}
                        {foreach $input.hint as $hint}
                            {if is_array($hint)}
                                {$hint.text|escape:'html':'UTF-8'}
                            {else}
                                {$hint|escape:'html':'UTF-8'}
                            {/if}
                        {/foreach}
                    {else}
                        {$input.hint|escape:'html':'UTF-8'}
                    {/if}">
            {/if}
            {$input.label|escape:'htmlall':'UTF-8'}
            {if isset($input.hint)}
            </span>
            {/if}

            {if isset($input.doc) && $input.doc}
                <div class="hi-module-whats-this">
                    <a href="#" data-doc="{$input.doc|escape:'htmlall':'UTF-8'}">{if isset($input.docTitle) && $input.docTitle}{$input.docTitle|escape:'htmlall':'UTF-8'}{else}{l s='What\'s this?' mod='hicookielaw'}{/if}</a>
                </div>
            {/if}
        </label>
    {/if}
{/block}

{block name="field"}
    {if $input.type == 'whitelist_ip'}
        <div class="col-lg-8">
            <div class="col-lg-5">
                <input type="text"
                    name="{$input.name|escape:'htmlall':'UTF-8'}"
                    id="{if isset($input.id)}{$input.id|escape:'htmlall':'UTF-8'}{else}{$input.name|escape:'htmlall':'UTF-8'}{/if}"
                    value="{$fields_value.whitelist_ips|escape:'htmlall':'UTF-8'}"
                    class="{if isset($input.class)}{$input.class|escape:'htmlall':'UTF-8'}{/if}"
                    {if isset($input.size)} size="{$input.size|escape:'htmlall':'UTF-8'}"{/if}
                    {if isset($input.maxchar) && $input.maxchar} data-maxchar="{$input.maxchar|intval}"{/if}
                    {if isset($input.maxlength) && $input.maxlength} maxlength="{$input.maxlength|intval}"{/if}
                    {if isset($input.readonly) && $input.readonly} readonly="readonly"{/if}
                    {if isset($input.disabled) && $input.disabled} disabled="disabled"{/if}
                    {if isset($input.autocomplete) && !$input.autocomplete} autocomplete="off"{/if}
                    {if isset($input.required) && $input.required } required="required" {/if}
                    {if isset($input.placeholder) && $input.placeholder } placeholder="{$input.placeholder|escape:'htmlall':'UTF-8'}"{/if}
                />
            </div>
            <div class="col-lg-3">
                <button type="button" class="btn btn-default hi-cookie-add-remote-ip" data-ip-address="{$input.remote_ip_address|escape:'htmlall':'UTF-8'}"><i class="icon-plus"></i> {l s='Add my IP' mod='hicookielaw'}</button>
            </div>
        </div>
    {elseif $input.type == 'spacing'}
        <ul class="hi-spacing-element">
            <li>
                <input
                    type="text"
                    name="{$input.name|escape:'htmlall':'UTF-8'}_top"
                    id="{if isset($input.id)}{$input.id|escape:'htmlall':'UTF-8'}_top{else}{$input.name|escape:'htmlall':'UTF-8'}_top{/if}"
                    value="{$fields_value[$input.name].top|escape:'htmlall':'UTF-8'}"
                    class="{if isset($input.class)}{$input.class|escape:'htmlall':'UTF-8'}{/if}"
                >
                <span>{l s='Top' mod='hicookielaw'}</span>
            </li>
            <li>
                <input
                    type="text"
                    name="{$input.name|escape:'htmlall':'UTF-8'}_right"
                    id="{if isset($input.id)}{$input.id|escape:'htmlall':'UTF-8'}_right{else}{$input.name|escape:'htmlall':'UTF-8'}_right{/if}"
                    value="{$fields_value[$input.name].right|escape:'htmlall':'UTF-8'}"
                    class="{if isset($input.class)}{$input.class|escape:'htmlall':'UTF-8'}{/if}"
                >
                <span>{l s='Right' mod='hicookielaw'}</span>
            </li>
            <li>
                <input
                    type="text"
                    name="{$input.name}_bottom"
                    id="{if isset($input.id)}{$input.id|escape:'htmlall':'UTF-8'}_bottom{else}{$input.name|escape:'htmlall':'UTF-8'}_bottom{/if}"
                    value="{$fields_value[$input.name].bottom}"
                    class="{if isset($input.class)}{$input.class|escape:'htmlall':'UTF-8'}{/if}"
                >
                <span>{l s='Bottom' mod='hicookielaw'}</span>
            </li>
            <li>
                <input
                    type="text"
                    name="{$input.name|escape:'htmlall':'UTF-8'}_left"
                    id="{if isset($input.id)}{$input.id|escape:'htmlall':'UTF-8'}_left{else}{$input.name|escape:'htmlall':'UTF-8'}_left{/if}"
                    value="{$fields_value[$input.name].left|escape:'htmlall':'UTF-8'}"
                    class="{if isset($input.class)}{$input.class|escape:'htmlall':'UTF-8'}{/if}"
                >
                <span>{l s='Left' mod='hicookielaw'}</span>
            </li>
        </ul>
        <style type="text/css">
            {literal}
                .hi-spacing-element {
                    list-style: none;
                }
                .hi-spacing-element li {
                    float: left;
                    width: 80px;
                }
                .hi-spacing-element li span {
                    display: block;
                    text-align: center;
                    font-size: 9px;
                    text-transform: uppercase;
                    padding-top: 5px;
                }
            {/literal}
        </style>
    {elseif $input.type == 'dropzone'}
        <div id="hi-module-dropzone-{$input.name}-container" class="hi-module-dropzone-container col-lg-8">
            {if isset($input.multilang) && $input.multilang}
                {foreach from=$languages item=language}
                    {if $languages|count > 1}
                        <div class="translatable-field lang-{$language.id_lang|intval}" {if $language.id_lang != $defaultFormLanguage}style="display:none"{/if}>
                    {/if}
                        <div class="form-group">
                            <div id="hi-module-dropzone-{$input.name|escape:'htmlall':'UTF-8'}_{$language.id_lang|intval}-error" class="text-danger"></div>
                            <div class="col-lg-10" {if isset($input.maxFiles) && $input.maxFiles == 1}style="width: 150px;"{/if}>
                                <div
                                    id="hi-module-dropzone-{$input.name|escape:'htmlall':'UTF-8'}_{$language.id_lang|intval}"
                                    class="panel hi-module-dropzone dropzone col-lg-12 {if isset($input.maxFiles) && $input.maxFiles == 1}hi-module-dropzone-single-file{/if} {if isset($input.sortable) && $input.sortable}ui-sortable{/if} {if $input.actionForm}hi-module-dropzone-action-form{/if}"
                                    url-upload="{$input.uploadUrl nofilter}" 
                                    url-position="/" 
                                    url-action="{$input.actionUrl nofilter}"
                                    data-max-size="{$input.maxFileSize|escape:'htmlall':'UTF-8'}"
                                    data-max-files="{if isset($input.maxFiles) && $input.maxFiles}{$input.maxFiles|intval}{else}10000{/if}"
                                    data-dictMaxFilesExceeded="{l s='Please delete existing image to upload the new one' mod='hicookielaw'}"
                                    data-id-lang={$language.id_lang|intval}
                                    data-input-name="{$input.name|escape:'htmlall':'UTF-8'}_{$language.id_lang|intval}"
                                    data-input-name-clean="{$input.name|escape:'htmlall':'UTF-8'}"
                                    data-files-location="{$input.filesLocation|escape:'htmlall':'UTF-8'}"
                                    data-existing-images='{if isset($fields_value[$input.name]) && isset($fields_value[$input.name][$language.id_lang])}{$fields_value[$input.name][$language.id_lang] nofilter}{else}[]{/if}'
                                    data-id-element="{$input.idElement|intval}"
                                    data-accepted-files="{if isset($input.acceptedFiles) && $input.acceptedFiles}{$input.acceptedFiles|escape:'htmlall':'UTF-8'}{else}image/*{/if}"
                                    {if isset($input.sortable) && $input.sortable}
                                        data-sortable="1"
                                    {else}    
                                        data-sortable="0"
                                    {/if}
                                    {if $input.actionForm}
                                        data-action-form="{$input.actionForm|escape:'htmlall':'UTF-8'}"
                                    {/if}
                                >
                                    {if !isset($input.maxFiles) || (isset($input.maxFiles) && $input.maxFiles > 1)}
                                        <div class="dz-default dz-message hi-module-dropzone-openfilemanager">
                                            {if isset($input.icon) && $input.icon}
                                                <i class="icon {$input.icon|escape:'htmlall':'UTF-8'}"></i><br>
                                            {else}
                                                <i class="material-icons">add_a_photo</i><br>
                                            {/if}
                                            {if isset($input.dropTxt)}
                                                {if $input.dropTxt}
                                                    {$input.dropTxt|escape:'htmlall':'UTF-8'}<br>
                                                {/if}
                                            {else}
                                                {l s='Drop images here' mod='hicookielaw'}
                                            {/if}
                                            {if isset($input.selectTxt)}
                                                {if $input.selectTxt}
                                                    <a>{$input.selectTxt|escape:'htmlall':'UTF-8'}</a> <br>
                                                {/if}
                                            {else}
                                                <a>{l s='or select files' mod='hicookielaw'}</a><br>
                                            {/if}
                                            {if isset($input.recSizeTxt) || isset($input.recFiles)}
                                                <small>
                                                    {if isset($input.recSizeTxt) && $input.recSizeTxt}
                                                        {$input.recSizeTxt|escape:'htmlall':'UTF-8'} <br>
                                                    {/if}

                                                    {if isset($input.recFiles) && $input.recFiles}
                                                        {$input.recFiles|escape:'htmlall':'UTF-8'}
                                                    {/if}
                                                </small>
                                            {/if}
                                        </div>

                                        <div class="dz-preview disabled hi-module-dropzone-openfilemanager">
                                            <div>
                                                <span>+</span>
                                            </div>
                                        </div>
                                    {else}
                                        <div class="dz-preview dz-message disabled hi-module-dropzone-openfilemanager">
                                            <div>
                                                <span>+</span>
                                            </div>
                                        </div>
                                    {/if}
                                </div>
                            </div>
                            {if $languages|count > 1}
                                <div class="col-lg-2">
                                    <button type="button" class="btn btn-default dropdown-toggle" tabindex="-1" data-toggle="dropdown">
                                        {$language.iso_code|escape:'htmlall':'UTF-8'}
                                        <span class="caret"></span>
                                    </button>
                                    <ul class="dropdown-menu">
                                        {foreach from=$languages item=lang}
                                        <li><a href="javascript:hideOtherLanguage({$lang.id_lang|escape:'htmlall':'UTF-8'});" tabindex="-1">{$lang.name|escape:'htmlall':'UTF-8'}</a></li>
                                        {/foreach}
                                    </ul>
                                </div>
                            {/if}
                        </div>
                    {if $languages|count > 1}
                        </div>
                    {/if}
                {/foreach}
            {else}
                <div
                    id="hi-module-dropzone-{$input.name|escape:'htmlall':'UTF-8'}"
                    class="panel hi-module-dropzone dropzone col-lg-12 {if isset($input.maxFiles) && $input.maxFiles == 1}hi-module-dropzone-single-file{/if} {if isset($input.sortable) && $input.sortable}ui-sortable{/if} {if $input.actionForm}hi-module-dropzone-action-form{/if}" 
                    url-upload="{$input.uploadUrl nofilter}" 
                    url-position="/"
                    url-action="{$input.actionUrl nofilter}"
                    data-max-size="{$input.maxFileSize|escape:'htmlall':'UTF-8'}"
                    data-max-files="{if isset($input.maxFiles) && $input.maxFiles}{$input.maxFiles|intval}{else}10000{/if}"
                    data-dictMaxFilesExceeded="{l s='Please delete existing image to upload the new one' mod='hicookielaw'}"
                    data-input-name="{$input.name|escape:'htmlall':'UTF-8'}"
                    data-input-name-clean="{$input.name|escape:'htmlall':'UTF-8'}"
                    data-files-location="{$input.filesLocation|escape:'htmlall':'UTF-8'}"
                    data-existing-images='{if isset($fields_value[$input.name])}{$fields_value[$input.name] nofilter}{else}[]{/if}'
                    data-id-element="{$input.idElement|intval}"
                    {if $input.actionForm}
                        data-action-form="{$input.actionForm|escape:'htmlall':'UTF-8'}"
                    {/if}
                    {if isset($input.sortable) && $input.sortable}
                        data-sortable="1"
                    {else}    
                        data-sortable="0"
                    {/if}
                >
                    <div id="hi-module-dropzone-{$input.name|escape:'htmlall':'UTF-8'}-error" class="text-danger"></div>
                    {if !isset($input.maxFiles) || (isset($input.maxFiles) && $input.maxFiles > 1)}
                        <div class="dz-default dz-message hi-module-dropzone-openfilemanager">
                            {if isset($input.icon) && $input.icon}
                                <i class="icon {$input.icon|escape:'htmlall':'UTF-8'}"></i><br>
                            {else}
                                <i class="material-icons">add_a_photo</i><br>
                            {/if}
                            {if isset($input.dropTxt)}
                                {if $input.dropTxt}
                                    {$input.dropTxt|escape:'htmlall':'UTF-8'}<br>
                                {/if}
                            {else}
                                {l s='Drop images here' mod='hicookielaw'}
                            {/if}
                            {if isset($input.selectTxt)}
                                {if $input.selectTxt}
                                    <a>{$input.selectTxt|escape:'htmlall':'UTF-8'}</a> <br>
                                {/if}
                            {else}
                                <a>{l s='or select files' mod='hicookielaw'}</a><br>
                            {/if}
                            {if isset($input.recSizeTxt) || isset($input.recFiles)}
                                <small>
                                    {if isset($input.recSizeTxt) && $input.recSizeTxt}
                                        {$input.recSizeTxt|escape:'htmlall':'UTF-8'} <br>
                                    {/if}

                                    {if isset($input.recFiles) && $input.recFiles}
                                        {$input.recFiles|escape:'htmlall':'UTF-8'}
                                    {/if}
                                </small>
                            {/if}
                        </div>

                        <div class="dz-preview disabled hi-module-dropzone-openfilemanager">
                            <div>
                                <span>+</span>
                            </div>
                        </div>
                    {else}
                        <div class="dz-preview dz-message disabled hi-module-dropzone-openfilemanager">
                            <div>
                                <span>+</span>
                            </div>
                        </div>
                    {/if}
                </div>
            {/if}
            {if $input.actionForm}
                <div id="hi-module-dropzone-form-container-{$input.name|escape:'htmlall':'UTF-8'}" class="hi-module-dropzone-form-container col-lg-10 {if !isset($input.multilang) || !$input.multilang}hi-module-dropzone-form-no-lang{/if}">
                    <div id="hi-module-dropzone-form-{$input.name|escape:'htmlall':'UTF-8'}"></div>
                </div>
            {/if}
        </div>
    {/if}
    {$smarty.block.parent}
{/block}
