<?php
/*
* 2007-2014 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2014 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_'))
	exit;

class labmanufacturer extends Module
{
    public function __construct()
    {
        $this->name = 'labmanufacturer';
        $this->tab = 'front_office_features';
        $this->version = '1.2.1';
		$this->author = 'Labersthemes';
		$this->need_instance = 0;

        $this->bootstrap = true;
		parent::__construct();	

		$this->displayName = $this->l('Lab Manufacturers logo Slider');
        $this->description = $this->l('Displays a block listing product manufacturers and/or brands.');
		$this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

	public function install()
	{
		Configuration::updateValue('MANUFACTURER_DISPLAY_TEXT_NB', 10);
/* 		Configuration::updateValue('MANUFACTURER_COLUMN', 6);
		Configuration::updateValue('MANUFACTURER_ROW', 1); */
		$success = (parent::install() &&
			$this->registerHook('header') &&
			$this->registerHook('actionObjectManufacturerDeleteAfter') &&
			$this->registerHook('actionObjectManufacturerAddAfter') &&
			$this->registerHook('actionObjectManufacturerUpdateAfter')
		);

		if ($success)
		{
			// Hook the module either on the left or right column
			$theme = new Theme(Context::getContext()->shop->id_theme);
			if ((!$theme->default_left_column || !$this->registerHook('manufacturer')))
			{
				// If there are no colums implemented by the template, throw an error and uninstall the module
				$this->_errors[] = $this->l('This module need to be hooked in a column and your theme does not implement one');
				parent::uninstall();
				return false;
			}
		}
		return $success;
    }

	public function hookmanufacturer($params)
	{
		if (!$this->isCached('labmanufacturer.tpl', $this->getCacheId()))
		{
			$manufacturers = Manufacturer::getManufacturers();
			foreach ($manufacturers as &$manufacturer)
			{
				$manufacturer['image'] = $this->context->language->iso_code.'-default';
				if (file_exists(_PS_MANU_IMG_DIR_.$manufacturer['id_manufacturer'].'-'.ImageType::getFormatedName('medium').'.jpg'))
					$manufacturer['image'] = $manufacturer['id_manufacturer'];
			}
			$languages = Language::getLanguages(true, $this->context->shop->id);
			$this->smarty->assign(array(
				'manufacturers' => $manufacturers,
				'languages' => $languages,
				'text_list_nb' => Configuration::get('MANUFACTURER_DISPLAY_TEXT_NB'),
				'column' => Configuration::get('MANUFACTURER_COLUMN'),
				'row' => Configuration::get('MANUFACTURER_ROW'),
				'display_link_manufacturer' => Configuration::get('PS_DISPLAY_SUPPLIERS'),
			));
		}
		return $this->display(__FILE__, 'labmanufacturer.tpl', $this->getCacheId());
	}
	/* public function hookblockPosition2($params)
	{
		return $this->hookblockPosition3($params);
	} */
	public function getContent()
	{
		$output = '';
		if (Tools::isSubmit('submitlabmanufacturers'))
		{
			$text_nb = (int)(Tools::getValue('MANUFACTURER_DISPLAY_TEXT_NB'));
			/* $column = (int)(Tools::getValue('MANUFACTURER_COLUMN'));
			$row = (int)(Tools::getValue('MANUFACTURER_ROW')); */
			
			Configuration::updateValue('MANUFACTURER_DISPLAY_TEXT_NB', $text_nb);
			/* Configuration::updateValue('MANUFACTURER_COLUMN', $column);
			Configuration::updateValue('MANUFACTURER_ROW', $row); */
			
			$this->_clearCache('labmanufacturer.tpl');
			if (isset($errors) && count($errors))
				$output .= $this->displayError(implode('<br />', $errors));
			else
				$output .= $this->displayConfirmation($this->l('Settings updated.'));
		}
		return $output.$this->renderForm();
	}

	public function hookHeader($params)
	{
		$this->context->controller->addCSS(($this->_path).'labmanufacturer.css', 'all');
	}
	
	public function hookActionObjectManufacturerUpdateAfter($params)
	{
		$this->_clearCache('labmanufacturer.tpl');
	}

	public function hookActionObjectManufacturerAddAfter($params)
	{
		$this->_clearCache('labmanufacturer.tpl');
	}

	public function hookActionObjectManufacturerDeleteAfter($params)
	{
		$this->_clearCache('labmanufacturer.tpl');
	}
	
	public function renderForm()
	{
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Settings'),
					'icon' => 'icon-cogs'
				),
				'input' => array(
					array(
						'type' => 'text',
						'label' => $this->l('Number of elements to display'),
						'name' => 'MANUFACTURER_DISPLAY_TEXT_NB',
						'class' => 'fixed-width-xs'
					),
					/* array(
						'type' => 'text',
						'label' => $this->l('Column'),
						'name' => 'MANUFACTURER_COLUMN',
						'class' => 'fixed-width-xs'
					),
					array(
						'type' => 'text',
						'label' => $this->l('Row'),
						'name' => 'MANUFACTURER_ROW',
						'class' => 'fixed-width-xs'
					) */
				),
				'submit' => array(
					'title' => $this->l('Save'),
				)
			),
		);
		
		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table =  $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitlabmanufacturers';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		return $helper->generateForm(array($fields_form));
	}
	
	public function getConfigFieldsValues()
	{		
		return array(
			'MANUFACTURER_DISPLAY_TEXT_NB' => Tools::getValue('MANUFACTURER_DISPLAY_TEXT_NB', Configuration::get('MANUFACTURER_DISPLAY_TEXT_NB')),
			/* 'MANUFACTURER_COLUMN' => Tools::getValue('MANUFACTURER_COLUMN', Configuration::get('MANUFACTURER_COLUMN')),
			'MANUFACTURER_ROW' => Tools::getValue('MANUFACTURER_ROW', Configuration::get('MANUFACTURER_ROW')), */
		);
	}
}
