<?php
/**
* LayerSlider v5.1.6 - Responsive Slideshow Module http://layerslider.webshopworks.com
*
*  @author    WebshopWorks <info@webshopworks.com>
*  @copyright 2015 WebshopWorks
*  @license   One Domain Licence
*/

if (!defined('_PS_VERSION_'))
	exit;

class LSExportUtil
{
	/**
	 * The managed ZipArchieve instance.
	 */
	private $zip;

	/**
	 * A temporary file in /wp-content/uploads/ to manipulate
	 * ZIPs on the fly without permanently saving to file system.
	 */
	private $file;

	/**
	 * Prepares a ZipArchieve instance and the file system
	 * to work with the class.
	 *
	 * @since 5.0.3
	 * @access public
	 * @return void
	 */
	public function __construct()
	{
		if (class_exists('ZipArchive'))
		{
			$upload_dir = ww_upload_dir();
			$tmp_dir = $upload_dir['basedir'];

			$this->file = tempnam($tmp_dir, 'zip');
			$this->zip = new ZipArchive;
			$this->zip->open($this->file, ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE);
		}
	}

	/**
	 * Adds slider settings .json file to ZIP
	 *
	 * @since 5.0.3
	 * @access public
	 * @param string $data Slider settings JSON
	 * @return void
	 */
	public function addSettings($data, $folder = '')
	{
		$folder = !empty($folder) ? $folder.'/' : '';
		$this->zip->addFromString($folder.'settings.json', $data);
	}

	/**
	 * Adds slider images to ZIP
	 *
	 * @since 5.0.3
	 * @access public
	 * @param string $path Image path to add
	 * @return void
	 */
	public function addImage($files, $folder = '')
	{
		if (empty($files))
			return false;

		if (!is_array($files))
			$files = array($files);

		$folder = is_string($folder) ? $folder.'/uploads/' : 'uploads/';

		foreach ($files as $file)

			if (!empty($file) && is_string($file))
				$this->zip->addFile($file, $folder.ww_sanitize_file_name(basename($file)));

	}

	/**
	 * Closes all pending operations and downloads the ZIP file.
	 *
	 * @since 5.0.3
	 * @access public
	 * @return void
	 */
	public function download()
	{
		$this->zip->close();

		header('Content-Type: application/zip');
		header('Content-Disposition: attachment; filename="LayerSlider_Export_'.date('Y-m-d').'_at_'.date('H.i.s').'.zip"');
		header('Content-length: '.filesize($this->file));
		header('Pragma: no-cache');
		header('Expires: 0');
		readfile($this->file);

		unlink($this->file);
		exit;
	}

	public function getImagesForSlider($data)
	{
		$images = array();

		if (!empty($data['properties']['backgroundimage']))
			$images[] = $data['properties']['backgroundimage'];

		if (!empty($data['properties']['yourlogo']))
			$images[] = $data['properties']['yourlogo'];

		if (!empty($data['layers']) && is_array($data['layers']))
		{
			foreach ($data['layers'] as $slide)
			{
				if (!empty($slide['properties']['background']))
					$images[] = $slide['properties']['background'];

				if (!empty($slide['properties']['thumbnail']))
					$images[] = $slide['properties']['thumbnail'];

				if (!empty($slide['sublayers']) && is_array($slide['sublayers']))

					foreach ($slide['sublayers'] as $layer)

						if (!empty($layer['image']))
							$images[] = $layer['image'];

			}
		}
		return $images;
	}

	public function getFSPaths($urls)
	{
		if (!empty($urls) && is_array($urls))
		{
			$paths = array();
			$root = rtrim(_PS_ROOT_DIR_, '/');
			foreach ($urls as $url)
			{
				$url = Tools::substr($url, Tools::strlen(__PS_BASE_URI__) - 1);
				$path = $root.parse_url($url, PHP_URL_PATH);

				if (file_exists($path))
					$paths[] = $path;

			}

			return $paths;
		}

		return array();
	}
}
